#! /usr/bin/python

import os
import tarfile
import logging
from powervc_oslo.config import data_mgr as mgr
from powervc_oslo.config import data_utils as utils

PVC_VERSION="1.3.1.2"
IMG_PATH = '/opt/ibm/powervc/images/powervm/'
IMG_PREFIX = 'powervc-powervm-compute-%s' % PVC_VERSION
POWERVM_NAMES = ['nova-powervm', 'networking-powervm', 'ceilometer-powervm']


def __get_remote_hosts():
    """Helper method to provide a list of Remote Hosts managed"""
    # Get a list of each of the Remote Host Nova Proxy Configuration Files
    files = utils.get_matching_files('/etc/nova', 'nova-', '.conf')
    return [f[5:-5] for f in files if __is_remote_host_conf(f)]

def __is_remote_host_conf(conf_file):
    """Helper method to determine if the CONF file is for a remote host"""
    full_path = conf_file.startswith('/etc/nova')
    conf_file = conf_file if full_path else ('/etc/nova/' + conf_file)
    # Query the Remote Credentials to use to determine if Proxy CONF
    config = utils.get_remote_host_credentials(conf_file)
    return config.get('ip') is not None and config.get('user') is not None

def __get_remote_host_credentials(host):
    """Helper method to provide a list of Remote Hosts managed"""
    conf_file = '/etc/nova/nova-%s.conf' % host
    # Use the Data Manager Utility to get the IP/Credential Information
    creds = utils.get_remote_host_credentials(conf_file)
    return creds

# update nova-powervm
def update_novalink_pkgs():
    """Updates the powervm pkgs on remote system"""
    # extract the compute tgz
    temp_path = utils.create_unique_temp_directory()
    # list the powervm debs to copy over and update 
    deb_list = []
    tar_name = IMG_PATH + IMG_PREFIX + '.tgz'
    tar = tarfile.open(tar_name, "r:gz")
    for afile in tar.getnames():
        for pkg_name in POWERVM_NAMES:
            if pkg_name in afile:
                tar.extract(afile, path=temp_path)
                deb_list.append(temp_path + '/' + afile)
    tar.close()

    # Loop through each of the Remote Hosts to update pkgs
    for host in __get_remote_hosts():
        _update_remote_host(host, deb_list, True)

def _update_remote_host(host, deb_list, restart=False):
    """Updates the Remote Host to the latest version available"""
    creds = __get_remote_host_credentials(host)
    image = deb_list
    print "Updating the remote novalink host %s" % host
    # If we got an error updating the Remote Host, log to the console
    for deb_name in image:
        if __update_remote_host_image(creds, image=deb_name) != 0:
            print "Failed to update %s on novalink host %s." % (deb_name, host)
            return False
    # If we were asked to restart the services, do that now
    if restart:
        print "Restarting services on remote host."
        svccmd = '/opt/ibm/powervc/bin/powervc-services restart'
        utils.run_ssh_command(creds, svccmd, requires_root=True)
    return True


def __update_remote_host_image(creds, image):
    """Helper method to update the image on the given Remote Host"""
    rc = 1
    # Make the directory on the remote system to transfer the image to
    todir = utils.create_remote_unique_temp_directory(creds)
    tofile = todir + '/' + os.path.basename(image)
    utils.LOG.info("remote package  %s" % tofile)
    # If the transfer succeeded then we want to do the install
    if utils.sftp_transfer_file(creds, image, tofile) == 0:
        utils.LOG.info("Transferred %s to host" % os.path.basename(image))
        dpkg_cmd = '/usr/bin/dpkg -i ' + tofile
        print "....Updating %s on host" % os.path.basename(image)
        rc = utils.run_ssh_command(creds, dpkg_cmd, requires_root=True)[0]
    utils.run_ssh_command(creds, '/bin/rm -rf ' + todir)
    return rc

if __name__ == '__main__':
    # Enable Logging to the Log File and Standard Out if they asked
    log_file = utils.setup_logging('pvc_novalink_update')
    try:
        update_novalink_pkgs()
    except Exception as exc:
        msg = ("There were errors during the host update.  See log file "
               "%s for more information."
               )
        utils.LOG.log(utils.CONSOLE, msg % log_file)
        utils.LOG.exception(exc)
        sys.exit(1)


