# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
from nova.api.openstack import extensions
from nova.ibm.ego import ego_gettextutils
from nova.ibm.ego import resource

from webob import exc

_ = ego_gettextutils._


ALIAS = "ibm-resource-plan"
authorize = extensions.os_compute_authorizer(ALIAS)


class ResourceplanController(object):
    """
    Resource Plan for the Openstack API.
    """

    def __init__(self):
        self.resourceAPI = resource.API()
        super(ResourceplanController, self).__init__()

    def index(self, req):
        context = req.environ['nova.context']
        authorize(context, action='index')
        return {"properties":
                self.resourceAPI.get_resource_plan_properties(context)}

    def create(self, req, body):
        if not isinstance(body, dict):
            raise exc.exc.HTTPBadRequest()

        context = req.environ['nova.context']
        authorize(context, action='create')
        self.resourceAPI.set_resource_plan_properties(
            context, body)
        return {"properties": body}


class Resourceplan(extensions.V21APIExtensionBase):
    """Resource Plan"""
    name = "IBM EGO Multi dimension Resource Plan"
    alias = ALIAS
    namespace = ("http://ibm.com/compute/ext/ego/prs/resource_plan/api/v3")
    version = 1

    def get_resources(self):
        """Returns a list of policy objects."""
        resources = []

        res = extensions.ResourceExtension('ego/prs/resource_plan',
                                           ResourceplanController(),
                                           member_actions={})
        resources.append(res)

        return resources

    def get_controller_extensions(self):
        return []
