# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
from nova.api.openstack import extensions
from nova.ibm.ego import ego_gettextutils
from nova.ibm.ego import resource

from oslo_log import log as logging
from webob import exc

_ = ego_gettextutils._

LOG = logging.getLogger(__name__)

ALIAS = "ibm-prs-resource-overview"


def authorize(context, action_name):
    action = '%s:%s' % (ALIAS, action_name)
    extensions.extension_authorizer('compute', action)(context)


class ResourceOverviewController(object):
    """
    Ego Resource Overview Openstack API.
    """

    def __init__(self):
        self.resourceAPI = resource.API()
        super(ResourceOverviewController, self).__init__()

    def index(self, req):
        """
        :param req: GET /v2/{tenant_id}/ego/prs/resource_overview

        Return a dict of
        {
            'usage': a dict contains total resource/used resource in cluster
            'host_info': a list of all hosts with their information.
            'instance_info': a list of all service instances.
        }.
        """
        context = req.environ['nova.context']
        authorize(context, 'index')
        try:
            return self.resourceAPI.get_services_info(context)
        except Exception as ex:
            LOG.exception(ex)
            raise exc.HTTPBadRequest(explanation=ex.value)


class Resourceoverview(extensions.ExtensionDescriptor):
    """Ego Resource Overview"""
    name = "IBM EGO Resource Overview"
    alias = ALIAS
    namespace = ("http://ibm.com/compute/ext/ego/prs/"
                 "resource_overview/api/v2.0")
    updated = "2013-02-24T00:00:00+00:00"

    def get_resources(self):
        """Returns a list of resource overview objects."""
        resources = []

        res = extensions.ResourceExtension('ego/prs/resource_overview',
                                           ResourceOverviewController(),
                                           member_actions={})
        resources.append(res)

        return resources
