# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
from nova.api.openstack import extensions
from nova.ibm.ego import ego_gettextutils
from nova.ibm.ego import resource

from oslo_log import log as logging
from webob import exc

_ = ego_gettextutils._

LOG = logging.getLogger(__name__)

ALIAS = "ibm-prs-resource-plan-details"


def authorize(context, action_name):
    action = '%s:%s' % (ALIAS, action_name)
    extensions.extension_authorizer('compute', action)(context)


class ResourcePlanDetailsController(object):
    """
    Plan details Openstack API.
    """

    def __init__(self):
        self.resourceAPI = resource.API()
        super(ResourcePlanDetailsController, self).__init__()

    def index(self, req):
        context = req.environ['nova.context']
        authorize(context, 'index')
        try:
            return self.resourceAPI.get_plan_details(context)
        except Exception as ex:
            LOG.exception(ex)
            raise exc.HTTPBadRequest(explanation=ex.value)

    def show(self, req, id):
        context = req.environ['nova.context']
        authorize(context, 'show')
        detail = None
        try:
            detail = self.resourceAPI.get_plan_detail_by_consumer(
                context, id)
        except Exception as ex:
            LOG.exception(ex)
            raise exc.HTTPBadRequest(explanation=ex.value)

        if not detail:
            msg = _("The system cannot get plan detail for "
                    "%(id)s") % {'id': id}
            raise exc.HTTPNotFound(explanation=msg)
        else:
            return detail

    def update(self, req, id, body):
        """
        :param req: POST /v2/{tenant_id}/ego/prs/resource_plan_details
        :id : not used.
        :param body: A dict contains all plan details for all consumers.
        Return None
        """
        if not isinstance(body, dict):
            raise exc.exc.HTTPBadRequest()

        context = req.environ['nova.context']
        authorize(context, 'update')
        try:
            self.resourceAPI.set_plan_details(context, body)
        except Exception as ex:
            LOG.exception(ex)
            raise exc.HTTPBadRequest(explanation=ex.value)
        return {}


class Resourceplandetails(extensions.ExtensionDescriptor):
    """Plan Details"""
    name = "IBM EGO Resource Plan Details"
    alias = ALIAS
    namespace = ("http://ibm.com/compute/ext/ego/prs/"
                 "resource_plan_details/api/v2.0")
    updated = "2013-02-24T00:00:00+00:00"

    def get_resources(self):
        """Returns a list of resource plan details objects."""
        resources = []

        res = extensions.ResourceExtension('ego/prs/resource_plan_details',
                                           ResourcePlanDetailsController(),
                                           member_actions={})
        resources.append(res)

        return resources
