# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""Start script for Nova IBM Workload Resource Group Service."""

import gettext
import sys

gettext.install('nova', unicode=1)

from nova import config
from nova import objects
from oslo_reports import guru_meditation_report as gmr
from nova import service
from nova import utils
from nova import version

from oslo_concurrency import processutils
from oslo_config import cfg
from oslo_log import log as logging

CONF = cfg.CONF
CONF.import_opt('topic', 'nova.ibm.ego.wrg.rpcapi',
                group='workload_resource_group')


def main():
    config.parse_args(sys.argv)
    logging.setup(CONF, "nova")
    utils.monkey_patch()
    objects.register_all()
    gmr.TextGuruMeditation.setup_autorun(version)
    server = service.Service.create(
               binary='nova-ibm-ego-workload-resource-group',
               topic=CONF.workload_resource_group.topic,
               manager='nova.ibm.ego.wrg.manager.WorkloadResourceGroupManager')
    workers = (CONF.workload_resource_group.workers or
                           processutils.get_worker_count())
    service.serve(server, workers=workers)
    service.wait()
