# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
from migrate import ForeignKeyConstraint
from sqlalchemy import MetaData, Table


DB2_FKEYS = (
    {'table': 'instance_type_projects',
     'columns': ['instance_type_id'],
     'refcolumns': ['instance_types.id'],
     'name': 'instance_type_projects_ibfk_1'},
    {'table': 'iscsi_targets',
     'columns': ['volume_id'],
     'refcolumns': ['volumes.id'],
     'name': 'iscsi_targets_volume_id_fkey'},
    {'table': 'reservations',
     'columns': ['usage_id'],
     'refcolumns': ['quota_usages.id'],
     'name': 'reservations_ibfk_1'},
    {'table': 'security_group_instance_association',
     'columns': ['security_group_id'],
     'refcolumns': ['security_groups.id'],
     'name': 'security_group_instance_association_ibfk_1'}
)


def _get_refcolumns(metadata, refcolumns):
    refcolumn_objects = []
    for refcol in refcolumns:
        table, column = refcol.split('.')
        table = Table(table, metadata, autoload=True)
        refcolumn_objects.append(table.c[column])
    return refcolumn_objects


def _process_db2_foreign_keys(migrate_engine, upgrade):
    metadata = MetaData(bind=migrate_engine)
    for values in DB2_FKEYS:
        # NOTE(mriedem): We have to load all of the tables in the same
        # MetaData object for the ForeignKey object to work, so we just
        # load up the Column objects here as well dynamically.
        params = dict(name=values['name'])
        table = Table(values['table'], metadata, autoload=True)
        params['table'] = table
        params['columns'] = [table.c[col] for col in values['columns']]
        params['refcolumns'] = _get_refcolumns(metadata, values['refcolumns'])
        fkey = ForeignKeyConstraint(**params)
        if upgrade:
            fkey.create()
        else:
            fkey.drop()


def upgrade(migrate_engine):
    if migrate_engine.name == 'ibm_db_sa':
        # create the foreign keys
        _process_db2_foreign_keys(migrate_engine, upgrade=True)
