# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from oslo_db.sqlalchemy import utils
import six

DB2_INDEXES = {
    # table_name: ((index_name, (*idx_columns)), ...)
    "instance_system_metadata": (
        ('instance_system_metadata_deleted_instance_uuid_idx',
         ('deleted', 'instance_uuid')),
    ),
    "instance_info_caches": (
        ('instance_info_caches_deleted_instance_uuid_idx',
         ('deleted', 'instance_uuid')),
    ),
    "security_group_instance_association": (
        ('security_group_instance_association_deleted_id_idx',
         ('deleted', 'security_group_id')),
    ),
    "compute_nodes": (
        ('compute_nodes_deleted_service_id_idx',
         ('deleted', 'service_id')),
    ),
    "instance_actions_events": (
        ('instance_actions_events_deleted_action_id_event_idx',
         ('deleted', 'action_id', 'event')),
    ),
    "instance_actions": (
        ('instance_actions_deleted_instance_uuid_id_idx',
         ('deleted', 'instance_uuid', 'request_id')),
    ),
    "block_device_mapping": (
        ('block_device_mapping_deleted_instance_uuid_idx',
         ('deleted', 'instance_uuid')),
    ),
}


def upgrade(migrate_engine):
    # DB2: this script will create the new indexes.
    if migrate_engine.name == 'ibm_db_sa':
        for table_name, indexes in six.iteritems(DB2_INDEXES):
            for index_name, idx_columns in indexes:
                try:
                    utils.add_index(migrate_engine, table_name,
                                    index_name, idx_columns)
                except ValueError:
                    # NOTE(mriedem): When upgrading from Havana with DB2 we
                    # might already have an index from our internal 163 script
                    # that is now gone in Icehouse and replaced with 194, 212
                    # or 214, so ignore the ValueError if the index already
                    # existed.
                    pass
