# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""
EGO common flags
"""
from oslo_config import cfg

from nova.ibm.ego import ego_constants as ego_const

ego_scheduler_driver_opts = [
    cfg.StrOpt('ego_master_list',
               default='',
               help='EGO master host (e.g., "centos1")'),
    cfg.StrOpt('ego_kd_port',
               default='',
               help='EGO kernel port; should be in range of (1024 - 65535)'),
    cfg.StrOpt('ego_lib_path',
               default='',
               help='EGO security plug-in library path '
               '(e.g., "/opt/stack/nova/nova/scheduler/ibm/ego")'),
    cfg.StrOpt('ego_user',
               help='The user name with which to connect to EGO'),
    cfg.StrOpt('ego_password',
               secret=True,
               help='The password with which to connect to EGO'),
    cfg.BoolOpt('ego_enable_ipv6',
                default=True,
                help='Flag to specify whether IPv6 is enabled')
]

ego_common_opts = [
    cfg.BoolOpt('policy_per_host_aggregate',
               default=False,
               help='True to enable host aggregate based policy mode; '
               'False to switch to global policy mode'),
    cfg.BoolOpt('prs_sync_aggregate_reserveon',
               default=True,
               help='Flag to specify whether PRS, when configured in '
               'aggregate mode (i.e., policy_per_host_aggregate = True), '
               'will auto-update "reserveon" element of allocation to align '
               'with changes to the host aggregate(s) in which the VM '
               'logically resides; will be updated upon receiving '
               'notifications and via periodic sync-up tasks.'),
    cfg.IntOpt('prs_policy_event_time_window_hours',
               default=24,
               help='The lifecycle (in hours) of a PRS policy event'),
    cfg.BoolOpt('prs_allocation_per_server_group',
               default=False,
               help='Specifies the PRS server group mode. True to create'
                    ' allocation for server group. False to create allocation'
                    ' for VM.'),
    cfg.ListOpt('ego_extension_managers',
                default=[],
                help='Full classes name of EGO manager extensions which can '
                     'do extra handling during VM instance scheduling')
]

cli_options = [
    cfg.StrOpt('username',
               default='',
               help='Ceilometer service user name'),
    cfg.StrOpt('password',
               secret=True,
               default='',
               help='Ceilometer service user password'),
    cfg.StrOpt('project_name',
               default='',
               help='Service tenant name'),
    cfg.StrOpt('auth_url',
               default='',
               help='Keystone authentication URI'),
    cfg.StrOpt('service_type',
               default='metering',
               help='Ceilometer service type (e.g., metering or telemetry)'),
    cfg.BoolOpt('insecure',
               default=None,
               help='Verify HTTPS connections'),
    cfg.StrOpt('project_domain_name',
               default='Default',
               help='Domain name for the user project'),
    cfg.StrOpt('user_domain_name',
               default='Default',
               help='The domain name of the user')
]

prs_ha_mt_options = [
    cfg.IntOpt('prs_ha_timeout_seconds',
               default=1200,
               help='The length of time (in seconds) within which a '
               'live migration, cold migration or rebuild operation needs to '
               'finish per VM before the overall operation state moves '
               'to the error state.'),
    cfg.StrOpt('prs_mt_migration_preferred_order',
               default=ego_const.MOST_MEM,
               help='The preferred order in which VM migrations will be '
               'attempted for maintenance mode operations; valid values '
               'include %(most-mem)r (default), %(least-mem)r, %(most-vcpus)r '
               'or %(least-vcpus)r' % {'most-mem': ego_const.MOST_MEM,
                                       'least-mem': ego_const.LEAST_MEM,
                                       'most-vcpus': ego_const.MOST_VCPUS,
                                       'least-vcpus': ego_const.LEAST_VCPUS}),
    cfg.StrOpt('prs_compute_status_file',
               default='/etc/nova/prs_compute_node_status.json',
               help='Compute node stats file')
]

prs_rtpolicy_options = [
    cfg.IntOpt('prs_rtpolicy_instance_recovery_time',
               default=3600,
               help='The recovery time (in seconds) before the ongoing'
                    ' optimization process will re-attempt a migration'
                    ' after a previously failed attempt.')
]

prs_slapolicy_options = [
    cfg.IntOpt('prs_slapolicy_instance_recovery_time',
               default=3600,
               help='The recovery time (in seconds) before the SLA ongoing'
                    ' optimization process will re-attempt a migration'
                    ' after a previously failed attempt.'),
    cfg.IntOpt('prs_slapolicy_instance_failed_times',
               default=3,
               help='The instance live migration failed times in the SLA'
                    ' ongoing optimization process. When the instance live'
                    ' migration failed times greater than the max times,'
                    ' it will be added in the failed list, then next SLA'
                    ' policy cycle will skip the instance migration, until'
                    ' the instance live migration failed time exceed the'
                    ' recovery time.')
]

prs_driver_options = [
    cfg.StrOpt('prs_driver',
               default='nova.ibm.ego.prs_driver.PRSDriver',
               help='The default PRS extension point to be shared '
                    'across PRS services')
]

CONF = cfg.CONF
CONF.register_opts(ego_scheduler_driver_opts)
CONF.register_opts(ego_common_opts)
CONF.register_opts(cli_options, 'ibm_prs_ceilometer')
CONF.register_opts(prs_ha_mt_options)
CONF.register_opts(prs_rtpolicy_options)
CONF.register_opts(prs_slapolicy_options)
CONF.register_opts(prs_driver_options)
