# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""Handles all requests related to the hapolicy service."""

from nova.db import base
from nova.ibm.ego.hapolicy import rpcapi as policy_rpcapi


class API(base.Base):
    """API for doing HA policy via the nova-ibm-ego-ha-service manager."""

    def __init__(self, **kwargs):
        self.policy_rpcapi = policy_rpcapi.HAPolicyAPI()
        super(API, self).__init__(**kwargs)

    def get_all(self, context):
        return self.policy_rpcapi.get_all(context)

    def get(self, context, policy_id):
        return self.policy_rpcapi.get(context, policy_id)

    def update(self, context, policy_id, updates):
        return self.policy_rpcapi.update(context, policy_id, updates)

    def set_maintenance(self, context, hv_name, status, migrate=None,
                        target_host=None):
        return self.policy_rpcapi.set_maintenance(context, hv_name,
                                                  status, migrate,
                                                  target_host)

    def get_hv_status(self, context):
        return self.policy_rpcapi.get_hv_status(context)

    def hypervisor_list(self, context, search_opts):
        return self.policy_rpcapi.hypervisor_list(context, search_opts)

    def event_list(self, context, search_opts, marker, limit):
        return self.policy_rpcapi.event_list(context, search_opts, marker,
                                               limit)
