# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

ENABLED = 'enabled'
DISABLED = 'disabled'

# valid values for get_target_service_host(...) HA actions
HA_ACTION_LIVE_MIGRATION = 'live-migration'
HA_ACTION_COLD_MIGRATION = 'cold-migration'
HA_ACTION_REBUILD = 'rebuild'

# keys for metadata
KEY_HAPOLICY = 'hapolicy'
KEY_ID = 'id'
KEY_NAME = 'name'
KEY_DESC = 'description'
KEY_STATE = 'state'
KEY_RUN_INTERVAL = 'run_interval'
KEY_STABILIZATION = 'stabilization'

KEY_HA_ID = KEY_HAPOLICY + '-' + KEY_ID
KEY_HA_RUN_INTERVAL = KEY_HAPOLICY + '-' + KEY_RUN_INTERVAL
KEY_HA_STABILIZATION = KEY_HAPOLICY + '-' + KEY_STABILIZATION

# ['id', 'name', 'description', 'state', 'run_interval', 'stabilization']
KEYS = [KEY_ID,
        KEY_NAME,
        KEY_DESC,
        KEY_STATE,
        KEY_RUN_INTERVAL,
        KEY_STABILIZATION]

# keys for compute_node_stats table
HYPERVISOR_STATE = 'prs:hypervisor_state'
PARALLEL_MIGRATE = 'prs:max_parallel_migrations'
PARALLEL_REBUILD = 'prs:max_parallel_rebuilds'
PARALLEL_DEST_REBUILD = 'prs:max_parallel_dest_rebuilds'

# default parallel values
DEFAULT_PARALLEL_MIGRATE = 5
DEFAULT_PARALLEL_REBUILD = 5

# default number of max parallel rebuild instances on the destination host
DEFAULT_PARALLEL_DEST_REBUILD = 0x7FFFFFFF

# prefix for request_id that HA did
PREFIX_PRS_HA = 'prs-ha-'

STATE_OK = 'ok'

# status for maintenance mode
STATE_MT_ENTERING = 'entering'
STATE_MT_ON = 'on'
STATE_MT_ERROR = 'error'
STATE_LIST_MT = [STATE_MT_ENTERING, STATE_MT_ON, STATE_MT_ERROR]

# status for failure mode
STATE_HA_STARTED = 'started'
STATE_HA_MIGRATING = 'migrating'
STATE_HA_REBUILDING = 'rebuilding'
STATE_HA_EVACUATED = 'evacuated'
STATE_HA_ERROR = 'error'
STATE_LIST_HA = [STATE_HA_STARTED, STATE_HA_MIGRATING, STATE_HA_REBUILDING,
                 STATE_HA_EVACUATED, STATE_HA_ERROR]

# max retry times for a single instance
MAX_RETRY_TIMES = 3

# when HA or maintenance engine failed to move an instance away and will
# not try to move it any more, a notification with this event will be sent.
ETYPE_MOVING_INSTANCE_FAILED = 'prs.ha.moving_instance.failed'

# event types for ha
ETYPE_HA_STARTED_START = 'prs.ha.ha_started.start'
ETYPE_HA_STARTED_END = 'prs.ha.ha_started.end'
ETYPE_HA_MIGRATING_START = 'prs.ha.ha_migrating.start'
ETYPE_HA_MIGRATING_END = 'prs.ha.ha_migrating.end'
ETYPE_HA_REBUILDING_START = 'prs.ha.ha_rebuilding.start'
ETYPE_HA_REBUILDING_END = 'prs.ha.ha_rebuilding.end'
ETYPE_HA_EVACUATED_START = 'prs.ha.ha_evacuated.start'
ETYPE_HA_EVACUATED_END = 'prs.ha.ha_evacuated.end'
ETYPE_HA_RECOVERED_START = 'prs.ha.ha_recovered.start'
ETYPE_HA_RECOVERED_END = 'prs.ha.ha_recovered.end'
ETYPE_HA_ERROR_START = 'prs.ha.ha_error.start'
ETYPE_HA_ERROR_END = 'prs.ha.ha_error.end'
ETYPE_HA_ALERT = 'prs.ha.ha_alert'
ETYPE_HA_ACTION_START = 'prs.ha.ha_action.start'
ETYPE_HA_ACTION_END = 'prs.ha.ha_action.end'

# event types for maintenance
ETYPE_MT_ENTERING_START = 'prs.ha.maintenance_entering.start'
ETYPE_MT_ENTERING_END = 'prs.ha.maintenance_entering.end'
ETYPE_MT_ON_START = 'prs.ha.maintenance_on.start'
ETYPE_MT_ON_END = 'prs.ha.maintenance_on.end'
ETYPE_MT_ERROR_START = 'prs.ha.maintenance_error.start'
ETYPE_MT_ERROR_END = 'prs.ha.maintenance_error.end'
ETYPE_MT_OFF_START = 'prs.ha.maintenance_off.start'
ETYPE_MT_OFF_END = 'prs.ha.maintenance_off.end'

MT_EVENT_TYPES = {
    STATE_MT_ENTERING: {
        'start': ETYPE_MT_ENTERING_START,
        'end': ETYPE_MT_ENTERING_END
    },
    STATE_MT_ON: {
        'start': ETYPE_MT_ON_START,
        'end': ETYPE_MT_ON_END
    },
    STATE_MT_ERROR: {
        'start': ETYPE_MT_ERROR_START,
        'end': ETYPE_MT_ERROR_END
    },
    STATE_OK: {
        'start': ETYPE_MT_OFF_START,
        'end': ETYPE_MT_OFF_END
    }
}

HA_EVENT_TYPES = {
    STATE_HA_STARTED: {
        'start': ETYPE_HA_STARTED_START,
        'end': ETYPE_HA_STARTED_END
    },
    STATE_HA_MIGRATING: {
        'start': ETYPE_HA_MIGRATING_START,
        'end': ETYPE_HA_MIGRATING_END
    },
    STATE_HA_REBUILDING: {
        'start': ETYPE_HA_REBUILDING_START,
        'end': ETYPE_HA_REBUILDING_END
    },
    STATE_HA_EVACUATED: {
        'start': ETYPE_HA_EVACUATED_START,
        'end': ETYPE_HA_EVACUATED_END
    },
    STATE_HA_ERROR: {
        'start': ETYPE_HA_ERROR_START,
        'end': ETYPE_HA_ERROR_END
    },
    STATE_OK: {
        'start': ETYPE_HA_RECOVERED_START,
        'end': ETYPE_HA_RECOVERED_END
    }
}
