# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""HA Service"""

from nova.ibm.ego import ego_gettextutils
from nova.ibm.ego.hapolicy import policy_engine_manager
from nova import manager
from oslo_service import periodic_task

from oslo_config import cfg
from oslo_log import log as logging
import oslo_messaging as messaging
from oslo_utils import importutils

_ = ego_gettextutils._

LOG = logging.getLogger(__name__)

CONF = cfg.CONF
policy_opts = [
    cfg.StrOpt('hapolicy_conf_path',
               default='/etc/nova/hapolicy_conf.xml',
               help='The hapolicy configuration file')
    ]
CONF.register_opts(policy_opts)
CONF.import_opt('prs_driver', 'nova.ibm.ego.ego_flags')


class HAPolicyManager(manager.Manager):

    target = messaging.Target(version="1.0")

    """
    The HA manager is responsible for handling the policy CRUD operations and
    invoking the periodic tasks.
    """
    def __init__(self, policy_driver=None, *args, **kwargs):
        super(HAPolicyManager, self).__init__(*args, **kwargs)
        """Initialize the EGO manager"""
        self.prs_driver = importutils.import_object(CONF.prs_driver)
        self.policy_engine = policy_engine_manager.PolicyEngineManager(
                                CONF.hapolicy_conf_path, self.prs_driver)
        LOG.debug("The policy engine manager successfully initialized "
                  "the HA policy service.")

    def notify_policy_resource_status(self, context, msg):
        self.policy_engine.notify_policy_resource_status(context, msg)

    def get(self, context, policy_id):
        return self.policy_engine.get(policy_id)

    def get_all(self, context):
        return self.policy_engine.get_all()

    def update(self, context, policy_id, **kwargs):
        return self.policy_engine.update(policy_id, kwargs)

    def set_maintenance(self, context, hv_name, status, migrate=None,
                        target_host=None):
        return self.policy_engine.set_maintenance(context, hv_name,
                                                  status, migrate,
                                                  target_host)

    def get_hv_status(self, context):
        return self.policy_engine.get_hv_status()

    def hypervisor_list(self, context, search_opts):
        return self.policy_engine.hypervisor_list(context, search_opts)

    def event_list(self, context, search_opts, marker, limit):
        return self.policy_engine.event_list(context, search_opts, marker,
                                             limit)

    @periodic_task.periodic_task(spacing=60)
    def __periodic_task(self, context):
        """This task is triggered every 60 seconds."""
        ctxt = self.prs_driver.get_context(ctxt=context)
        self.policy_engine.policy_function(ctxt)
