# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""Client talk to keystone"""

from oslo_config import cfg

from keystoneclient.v2_0 import client
from nova.ibm.ego.resource import constants

keystone_authtoken = [
    cfg.StrOpt('admin_user',
               help='User id for connecting to keystone in admin context'),
    cfg.StrOpt('admin_password',
               help='Password for connecting to keystone in admin context',
               secret=True),
    cfg.StrOpt('admin_tenant_name',
               help='Tenant name for connecting to keystone in admin context. '
                    'This option will be ignored if keystone_admin_tenant_id '
                    'is set. Note that with Keystone V3 tenant names are '
                    'only unique within a domain.'),
    cfg.StrOpt('auth_uri',
               default='http://localhost:5000/v2.0',
               help='Authorization URL for connecting to keystone in admin '
               'context'),
]


CONF = cfg.CONF
CONF.register_opts(keystone_authtoken, 'keystone_authtoken')


class KeystoneClient(object):
    def __init__(self, *args, **kwargs):
        super(KeystoneClient, self).__init__(*args, **kwargs)
        self.client = client.Client(
            username=CONF.keystone_authtoken.admin_user,
            password=CONF.keystone_authtoken.admin_password,
            tenant_name=CONF.keystone_authtoken.admin_tenant_name,
            auth_url=CONF.keystone_authtoken.auth_uri)

    def get_tenants(self):
        return self.client.tenants.list()

    def get_tenant_by_id(self, id):
        try:
            return self.client.tenants.get(id)._info.get("name", "")
        except Exception:
            # When a consumer becomes a orphan consumer,
            # it can't get tenant name anymore. Then we use
            # the id as the name and mark it as orphan.
            return constants.TAG_ORPHAN_CONSUMER + id
