# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""Resource Manager"""
import oslo_messaging as messaging

from nova.ibm.ego.resource import resource_engine_manager
from nova import manager
from oslo_service import periodic_task


class ResourceManager(manager.Manager):
    target = messaging.Target(version="1.0")

    """
    The resource manager is responsible for handling the resource manager
    service CRUD operations and invoking the periodic tasks.
    """
    def __init__(self, *args, **kwargs):
        super(ResourceManager, self).__init__(*args, **kwargs)
        self.resource_engine_manager = (
            resource_engine_manager.ResourceEngineManager())
        self.resource_engine_manager.sync_up_ego_consumer()

    def ego_get_resource_plan_properties(self, context):
        return self.resource_engine_manager.get_resource_plan_properties(
            context)

    def ego_set_resource_plan_properties(self, context, properties):
        self.resource_engine_manager.set_resource_plan_properties(
            context, properties)

    def ego_get_consumer_list(self, context):
        return self.resource_engine_manager.get_consumer_list(context)

    def ego_get_consumer_by_id(self, context, tenant_id):
        return self.resource_engine_manager.get_consumer_by_id(
            context, tenant_id)

    def ego_create_consumer(self, context, tenant_id):
        return self.resource_engine_manager.create_consumer(tenant_id)

    def ego_delete_consumer(self, context, tenant_id):
        return self.resource_engine_manager.delete_consumer(tenant_id)

    def ego_set_consumer(self, context, properties):
        return self.resource_engine_manager.set_consumer(context, properties)

    def ego_get_plan_details(self, context):
        return self.resource_engine_manager.get_plan_details(context)

    def ego_get_plan_detail_by_consumer(self, context, tenant_id):
        return self.resource_engine_manager.get_plan_detail_by_consumer(
            context, tenant_id)

    def ego_set_plan_details(self, context, details):
        return self.resource_engine_manager.set_plan_details(context, details)

    def notify_resource_status(self, context, msg):
        return self.resource_engine_manager.notify_resource_status(context,
                                                                   msg)

    def get_services_info(self, context):
        return self.resource_engine_manager.get_services_info(context)

    @periodic_task.periodic_task(spacing=3600)
    def __sync_up_ego_consumer(self, context):
        """This task is triggered every 1 hour."""
        self.resource_engine_manager.sync_up_ego_consumer()
