# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""
Handles all requests related to the rtpolicy service.
"""

from nova.db import base
from nova.ibm.ego.rtpolicy import rpcapi as rtpolicy_rpcapi


class API(base.Base):
    """
    API for doing resource optimization policy via the
    nova-ibm-ego-resource-optimization manager.
    """

    def __init__(self, **kwargs):
        self.rtpolicy_rpcapi = rtpolicy_rpcapi.RTPolicyAPI()
        super(API, self).__init__(**kwargs)

    def get_all(self, context):
        return self.rtpolicy_rpcapi.get_all(context)

    def get(self, context, rtpolicy_id):
        return self.rtpolicy_rpcapi.get(context, rtpolicy_id)

    def delete(self, context, rtpolicy_id):
        return self.rtpolicy_rpcapi.delete(context, rtpolicy_id)

    def update(self, context, rtpolicy_id, rtpolicy, **kwargs):
        return self.rtpolicy_rpcapi.update(context, rtpolicy_id,
                                           rtpolicy, **kwargs)

    def hypervisor_list(self, context, search_opts):
        return self.rtpolicy_rpcapi.hypervisor_list(context, search_opts)

    def event_list(self, context, search_opts, marker, limit):
        return self.rtpolicy_rpcapi.event_list(context, search_opts, marker,
                                               limit)
