# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""Resource Optimization Service."""

from nova.ibm.ego import ego_gettextutils
from nova.ibm.ego.rtpolicy import policy_engine_manager
from nova import manager

from oslo_config import cfg
from oslo_log import log as logging
import oslo_messaging as messaging
from oslo_service import periodic_task
from oslo_utils import importutils

_ = ego_gettextutils._

LOG = logging.getLogger(__name__)

CONF = cfg.CONF
policy_opts = [
    cfg.StrOpt('rtpolicy_conf_path',
               default='/etc/nova/rtpolicy_conf.xml',
               help='The path to the rtpolicy configuration file')
    ]
CONF.register_opts(policy_opts)
CONF.import_opt('prs_driver', 'nova.ibm.ego.ego_flags')


class RTPolicyManager(manager.Manager):
    """Manages the policy CRUD operations and triggers the policy tasks."""

    target = messaging.Target(version="1.0")

    def __init__(self, policy_driver=None, *args, **kwargs):
        super(RTPolicyManager, self).__init__(*args, **kwargs)
        """Initialize the EGO manager"""
        self.prs_driver = importutils.import_object(CONF.prs_driver)
        rtpolicy_conf_path = CONF.rtpolicy_conf_path
        self.policy_engine = policy_engine_manager.PolicyEngineManager(
            rtpolicy_conf_path)
        LOG.debug("The policy engine manager successfully initialized "
                  "the resource optimization policy service.")

    def notify_policy_resource_status(self, context, msg):
        self.policy_engine.notify_policy_resource_status(context, msg)

    def get(self, context, rtpolicy_id):
        return self.policy_engine.get(rtpolicy_id)

    def get_all(self, context):
        return self.policy_engine.get_all()

    def update(self, context, rtpolicy_id, **kwargs):
        return self.policy_engine.update(rtpolicy_id, kwargs)

    def hypervisor_list(self, context, search_opts):
        return self.policy_engine.hypervisor_list(context, search_opts)

    def event_list(self, context, search_opts, marker, limit):
        return self.policy_engine.event_list(context, search_opts, marker,
                                             limit)

    @periodic_task.periodic_task(spacing=60)
    def __periodic_task(self, context):
        """This task is triggered every 60 seconds."""
        ctxt = self.prs_driver.get_context(ctxt=context)
        self.policy_engine.policy_function(ctxt)
