# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""
Handles all requests related to the workload resource group service.
"""

from nova.db import base
from nova.ibm.ego.wrg import rpcapi as wrg_rpcapi


class API(base.Base):
    """
    API for doing workload resource group via the
    nova-ibm-ego-workload-resource-group manager.
    """

    def __init__(self, **kwargs):
        self.wrg_rpcapi = wrg_rpcapi.WRGAPI()
        super(API, self).__init__(**kwargs)

    def create(self, context, wrg_body):
        return self.wrg_rpcapi.create(context, wrg_body)

    def get_all(self, context):
        return self.wrg_rpcapi.get_all(context)

    def get(self, context, wrg_uuid):
        return self.wrg_rpcapi.get(context, wrg_uuid)

    def delete(self, context, wrg_uuid):
        return self.wrg_rpcapi.delete(context, wrg_uuid)

    def update(self, context, wrg_uuid, wrg_body):
        return self.wrg_rpcapi.update(context, wrg_uuid, wrg_body)

    def associate_inst_with_wrg(self, context, inst_uuid, wrg_name=None,
                                inst_name=None, **kwargs):
        return self.wrg_rpcapi.associate_inst_with_wrg(context, inst_uuid,
                                                 wrg_name, inst_name, **kwargs)

    def get_inst_wrg_map(self, context):
        return self.wrg_rpcapi.get_inst_wrg_map(context)
