# ===============================================================._
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from oslo_config import cfg
from oslo_utils import importutils

from nova.ibm.ego import ego_gettextutils

_ = ego_gettextutils._

_store_client = [
    cfg.StrOpt('wrg_store_client',
             default='nova.ibm.ego.wrg.clients.etcd_client.EtcdClient',
             help='The default client which sends request of storing workload'
                  'resource group info'),
]


class BaseStoreClient(object):
    """Store base class that all Store driver client should inherit from."""

    def write(self, key, value, **kwdargs):
        raise NotImplementedError()

    def read(self, key, **kwdargs):
        raise NotImplementedError()

    def delete(self, key, **kwdargs):
        raise NotImplementedError()

    def update(self, obj):
        """
        Updates the value for a key atomically. Typical usage would be:

        c = Client()
        o = c.read("/somekey")
        o.value += 1
        c.update(o)
        """
        raise NotImplementedError()

    @classmethod
    def get_instance(cls, conf):
        raise NotImplementedError()


def get_store_client(conf):
    conf.register_opts(_store_client)
    client_class = importutils.import_class(conf.wrg_store_client)
    if issubclass(client_class, BaseStoreClient):
        return client_class.get_instance(conf)
    else:
        raise TypeError(_('%s is not a subclass of BaseStoreClient')
                          % conf.wrg_store_client)
