# =================================================================
# Licensed Materials _ Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights _ Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""Constants used in Workload Resource group mananger component."""

import copy

WRG_SERVICE_TOPIC = 'workload_resource_group'
# One of the following values must be specified to specify a velocity
# goal: fastest, fast, moderate, slow, slowest.
FASTEST_VELOCITY = 'fastest'
FAST_VELOCITY = 'fast'
MODERATE_VELOCITY = 'moderate'
SLOW_VELOCITY = 'slow'
SLOWEST_VELOCITY = 'slowest'
VELOCITY_GOAL_LIST = [FASTEST_VELOCITY, FAST_VELOCITY, MODERATE_VELOCITY,
                      SLOW_VELOCITY, SLOWEST_VELOCITY]

# One of the following values must be specified to specify the business
# importance : highest, high, medium, low, lowest.
HIGHEST_BUSINESS = 'highest'
HIGH_BUSINESS = 'high'
MEDIUM_BUSINESS = 'medium'
LOW_BUSINESS = 'low'
LOWEST_BUSINESS = 'lowest'
BUSINESS_IMPORTANCE_LIST = [HIGHEST_BUSINESS, HIGH_BUSINESS, MEDIUM_BUSINESS,
                            LOW_BUSINESS, LOWEST_BUSINESS]

WRG_NAME_MAX_LENGTH = 256
WRG_DESCRIPTION_MAX_LENGTH = 512
POLICY_NAME_MAX_LENGTH = 256
POLICY_DESCRIPTION_MAX_LENGTH = 512
SC_NAME_MAX_LENGTH = 256
SC_DESCRIPTION_MAX_LENGTH = 512

WRG_INFO = 'wrg-info'
WRG_NAME = 'name'
WRG_DESC = 'description'
WRG_CID = 'wrg-cluster-id'

PERF_POLICY = 'performance-policy'
PERF_POLICY_INFO = 'perf-policy-info'
POLICY_NAME = 'name'
POLICY_DESC = 'description'
POLICY_BUSNS_IMPTC = 'business-importance'

SVC_CLASSES = 'service-classes'
SC_NAME = 'name'
SC_DESC = 'description'
SC_CID = 'service-class-cluster-id'
SC_BUSNS_IMPTC = 'business-importance'
SC_VEL_GOAL = 'velocity-goal'
SC_CPU_CRITICAL = 'cpu-critical'
SC_VS_NAME_FILTERS = 'virtual-server-name-filters'
SC_CPU_CRITICAL_LIST = [True, False]

WRG_ATTR_LIST = [WRG_INFO, PERF_POLICY]
WRG_INFO_ATTR_LIST = [WRG_NAME, WRG_DESC, WRG_CID]
PERF_POLICY_ATTR_LIST = [PERF_POLICY_INFO, SVC_CLASSES]
POLICY_INFO_ATTR_LIST = [POLICY_NAME, POLICY_DESC, POLICY_BUSNS_IMPTC]
SC_ATTR_LIST = [SC_NAME, SC_CID, SC_DESC, SC_BUSNS_IMPTC, SC_VEL_GOAL,
                SC_CPU_CRITICAL, SC_VS_NAME_FILTERS]

DEFAULT_WRG_NAME = "PRSDefaultWorkloadResourceGroup"

DEFAULT_SVC_CLASS = {
    SC_NAME: "PRSDefaultServiceClass",
    SC_DESC: "PRS generated default service class",
    SC_BUSNS_IMPTC: MEDIUM_BUSINESS,
    SC_VEL_GOAL: MODERATE_VELOCITY,
    SC_CPU_CRITICAL: False,
    SC_VS_NAME_FILTERS: [".*"]}

DEFAULT_PERF_POLICY = {
    PERF_POLICY_INFO: {
        POLICY_NAME: "PRSDefaultPerformancePolicy",
        POLICY_DESC: "PRS Generated Default Performance Policy",
        POLICY_BUSNS_IMPTC: MEDIUM_BUSINESS},
    SVC_CLASSES: [DEFAULT_SVC_CLASS.copy()]}

DEFAULT_WRG = {
    WRG_INFO: {
        WRG_NAME: DEFAULT_WRG_NAME,
        WRG_DESC: "PRS Generated Default Workload Resource Group"},
    PERF_POLICY: copy.deepcopy(DEFAULT_PERF_POLICY)}

PRS_WRG_CREATE_START = 'prs.wrg.create.start'
PRS_WRG_CREATE_ERROR = 'prs.wrg.create.error'
PRS_WRG_CREATE_END = 'prs.wrg.create.end'

PRS_WRG_DELETE_START = 'prs.wrg.delete.start'
PRS_WRG_DELETE_ERROR = 'prs.wrg.delete.error'
PRS_WRG_DELETE_END = 'prs.wrg.delete.end'

PRS_WRG_UPDATE_START = 'prs.wrg.update.start'
PRS_WRG_UPDATE_ERROR = 'prs.wrg.update.error'
PRS_WRG_UPDATE_END = 'prs.wrg.update.end'

WRG_STORE_PREFIX = '/ego/prs/workload-resource-groups/'

VS_NAME = 'vs-name'

OLD_WRG = 'old_wrg'
VS_STORE_PREFIX = '/ego/prs/virtual-server-instance/'
WRG_NAMESPACE = 'ego::prs::wrg'
