# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from nova import exception
from nova.ibm.ego import ego_gettextutils

_ = ego_gettextutils._


class WRGNotFoundByUUID(exception.NotFound):
    msg_fmt = _('Workload resource group %(uuid)r cannot be found.')


class WRGNotFoundByName(exception.NotFound):
    msg_fmt = _('Workload resource group %(name)r cannot be found.')


class WRGNameExists(exception.NovaException):
    msg_fmt = _("Workload resource group %(name)r already exists.")


class InvalidWRGName(exception.Invalid):
    msg_fmt = _("Invalid workload resource group name: %(name)r. Names are "
                "restricted to the following set of characters: "
                "[a-z, A-Z, -, _, 0-9] and embedded whitespace.")


class InvalidBusinessImpt(exception.Invalid):
    msg_fmt = _("Invalid business importance: %(business_importance)r. "
                "Business importance "
                "must be one of the following: ['highest', 'high', 'medium', "
                "'low', 'lowest'].")


class InvalidVelocityGoal(exception.Invalid):
    msg_fmt = _("Invalid velocity goal: %(velocity_goal)r. Velocity goal "
                "must be one "
                "of the following: ['fastest', 'fast', 'moderate', 'slow', "
                "'slowest'].")


class WRGSyncFailed(exception.NovaException):
    msg_fmt = _("The workload resource group manager could not load workload "
                "resource group information from persistence.")


class InstWRGMapSyncFailed(exception.NovaException):
    msg_fmt = _("The workload resource group manager could not load virtual "
                "server instances' association with workload resource group "
                "from persistence.")


class AssociateVSWRGFailed(exception.NovaException):
    msg_fmt = _("Failed to associate workload resource group for virtual "
                "instance %(uuid)r.")


class DissociateVSWRGFailed(exception.NovaException):
    msg_fmt = _("Failed to dissociate workload resource group from virtual "
                "instance %(uuid)r.")


class WRGDefaultCreateFailed(exception.NovaException):
    msg_fmt = _("The workload resource group manager failed to create "
                "the default workload resource group.")


class WRGDeleteFailed(exception.NovaException):
    msg_fmt = _("The workload resource group manager failed to delete "
                "workload resource group(id is "
                "%(uuid)r) due to (%(reason)s).")


class WRGUpdateFailed(exception.NovaException):
    msg_fmt = _("The workload resource group manager failed to update "
                "%(uuid)r.")


class WRGStoreFailed(exception.NovaException):
    msg_fmt = _("Failed to persist workload resource group %(name)r.")


class StoreClientError(Exception):
    '''
    Base exception class for exceptions coming from store clients.
    For internal use only.
    '''
    pass


class StoreKeyNotFound(StoreClientError):
    pass
