# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from nova import exception
from nova.ibm.ego import ego_constants
from nova.ibm.ego import ego_gettextutils

_ = ego_gettextutils._


class EGOException(exception.NovaException):
    msg_fmt = _("EGO scheduler exception.")


class EGOExpOpenConnFaild(EGOException):
    msg_fmt = _("Failed to open EGO connection, reason: %(reason)s.")


class EGOExpConnLost(EGOException):
    code = ego_constants.LOST_CONN_CODE
    msg_fmt = _("Platform Resource Scheduler is temporarily disconnected.")


class EGOExpUnameFailed(EGOException):
    msg_fmt = _("Failed to get EGO cluster info, reason: %(reason)s")


class EGOExpLogonFailed(EGOException):
    msg_fmt = _("Could not log on Platform Resource Scheduler, "
                "reason: %(reason)s.")


class EGOExpLogonOrRegFailed(EGOException):
    msg_fmt = _("Could not log on or register Platform Resource Scheduler, "
                "reason: %(reason)s.")


class EGOExpGetAllAllocFailed(EGOException):
    msg_fmt = _("Could not get information of all allocations, "
                "reason: %(reason)s.")


class EGOExpSetResourceInfoFailed(EGOException):
    msg_fmt = _("Platform Resource Scheduler could not set EGO "
                "resources using resource string (%(cstr)s), "
                "reason: %(reason)s.")


class EGOExpCreateAllocFailed(EGOException):
    code = ego_constants.NOT_ENOUGH_RESOURCES_CODE
    msg_fmt = _("Platform Resource Scheduler cannot create a resource "
                "allocation due to the following reason: %(reason)s.")


class EGOExpFreeAllocFailed(EGOException):
    msg_fmt = _("Platform Resource Scheduler cannot free a resource "
                "allocation due to the following reason: %(reason)s.")


class EGOExpResizeAllocFailed(EGOException):
    code = ego_constants.NOT_ENOUGH_RESOURCES_CODE
    msg_fmt = _("Platform Resource Scheduler cannot resize allocation due to"
                " the following reason: %(reason)s")


class EGOExpSetResourcePlanFailed(EGOException):
    msg_fmt = _("Platform Resource Scheduler could not set the EGO "
                "resource plan using XML string (%(cstr)s), "
                "reason: %(reason)s.")


class EGOExpGetResourceInfoFailed(EGOException):
    msg_fmt = _("Platform Resource Scheduler could not get RPI "
                "resources using resource requirement (%(cstr)s), "
                "reason: %(reason)s.")


class EGOExpMigrateDecisionFailure(EGOException):
    msg_fmt = _("Platform Resource Scheduler cannot migrate decision "
                "(%(decid)s) due to the following reason: %(reason)s.")


class EGOExpGetPossibleHostsFailed(EGOException):
    msg_fmt = _("Platform Resource Scheduler could not find "
                "suitable hosts for decision (%(decid)s), "
                "reason: %(reason)s.")


class EGOExpNoAllocID(EGOException):
    code = ego_constants.NOT_ENOUGH_RESOURCES_CODE
    msg_fmt = _("Virtual machine (%(inst_id)s) is not allocated as a "
                "resource to any managed host. This virtual machine might "
                "be a standalone virtual machine. Platform Resource Scheduler "
                "must delete allocations associated with the virtual machine.")


class EGOExpCreateResourceGroupFailure(exception.Invalid):
    msg_fmt = _("Platform Resource Scheduler could not create "
                "EGO resource group %(group)s with resource "
                "requirement %(resreq)s.")


class EGOExpDeleteResourceGroupFailure(exception.Invalid):
    msg_fmt = _("Platform Resource Scheduler could not delete "
                "EGO resource group %(group)s.")


class EGOExpGetResourceGroupFailure(exception.Invalid):
    msg_fmt = _("Platform Resource Scheduler failed to get all "
                "EGO resource groups.")


class EGOExpGetDecisionsByFilterFailed(EGOException):
    msg_fmt = _("Platform Resource Scheduler could not find "
                "suitable decisions by filter (%(filter)s), "
                "reason: %(reason)s.")


class EGOExpUpdateDecFailed(EGOException):
    code = ego_constants.NOT_ENOUGH_RESOURCES_CODE
    msg_fmt = _("Platform Resource Scheduler cannot update decision "
                "%(decid)s due to the following reason: %(reason)s")


class DecNotFoundByInst(EGOException):
    msg_fmt = _("Platform Resource Scheduler cannot find decision "
                "for instance (%(uuid)s).")


class InvalidServerGroupPolicyInputInt(exception.Invalid):
    msg_fmt = _("The value %(value)s for server group metadata "
                "is invalid. Specify a number between "
                "%(min)d and %(max)d as the value.")


class EGOExpReleaseDecisionFailed(exception.Invalid):
    msg_fmt = _("Platform Resource Scheduler cannot release decision "
                "(%(decid)s) due to the following reason: %(reason)s.")


class InvalidServerGroupPolicyInputStr(exception.Invalid):
    msg_fmt = _("The value %(value)s for server group metadata "
                "is invalid. Specify a valid string.")


class InvalidSubdemandInput(exception.Invalid):
    msg_fmt = _("The subdemand is invalid: %(reason)s.")


class InvalidWorkloadResourceGroup(exception.Invalid):
    msg_fmt = _("The workload resource group %(name)s is invalid,"
                " due to the following reason: %(reason)s."
                " Specify a valid workload resource group.")


class EGOExpCreateConsumerFailed(EGOException):
    msg_fmt = _("Platform Resource Scheduler cannot create consumer "
                "%(name)s due to the following reason: %(reason)s")


class EGOExpAddRemoveConsumerFailed(EGOException):
    msg_fmt = _("Platform Resource Scheduler cannot add/remove consumer "
                "%(name)s due to the following reason: %(reason)s")


class EGOExpSetConsumerFailed(EGOException):
    msg_fmt = _("Platform Resource Scheduler cannot set consumer "
                "%(name)s due to the following reason: %(reason)s")


class EGOExpGetConsumerTreeFailed(EGOException):
    msg_fmt = _("Platform Resource Scheduler cannot get consumer tree "
                "due to the following reason: %(reason)s")


class EGOExpDeleteConsumerFailed(EGOException):
    msg_fmt = _("Platform Resource Scheduler cannot delete consumer "
                "%(name)s due to the following reason: %(reason)s")


class EGOExpGetServiceInstanceInfoFailed(EGOException):
    msg_fmt = _("Platform Resource Scheduler cannot get service instance "
                "info due to the following reason: %(reason)s")


class EGOExtenOperationFailed(EGOException):
    msg_fmt = _("Platform Resource Scheduler Extension operation failed "
                "due to the following reason: %(reason)s")
