# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""
Scheduler Service
"""

from nova.compute import rpcapi as compute_rpcapi
from nova.ibm.ego import ego_constants
from nova.ibm.ego import ego_gettextutils
from nova.image import glance
from nova.scheduler import manager
from nova import utils as nova_utils

from oslo_log import log as logging
from oslo_service import periodic_task

LOG = logging.getLogger(__name__)

_ = ego_gettextutils._
_LI = ego_gettextutils._LI


class SchedulerManager(manager.SchedulerManager):
    """
    Scheduler Manager that inherits from the original
    SchedulerManager
    """
    def __init__(self, *args, **kwargs):
        super(SchedulerManager, self).__init__(*args, **kwargs)
        self.image_service = glance.get_default_image_service()
        self.compute_rpcapi = compute_rpcapi.ComputeAPI()

    def notify_scheduler_resource_status(self, context, msg):
        self.driver.notify_scheduler_resource_status(context, msg)

    @periodic_task.periodic_task(spacing=3600)
    def ego_sync_up(self, context):
        LOG.info(_LI('Spawning thread for ego_sync_up periodic task.'))
        nova_utils.spawn(self.driver.ego_manager.ego_sync_up)

    # update service status(up/down) from DB
    # spacing must be no more than 30
    @periodic_task.periodic_task(spacing=30)
    def ego_sync_up_hosts(self, context):
        self.driver.ego_manager.ego_sync_up_hosts()

    def ego_alloc_create(self, ctxt, name, inter_sub_policies,
                              sub_demands):
        name = ego_constants.UNDERLINE.join([ego_constants.HEAT, name])
        return self.driver.ego_manager.ego_create_alloc_for_heat(
                            ctxt, name, inter_sub_policies, sub_demands)

    def ego_alloc_resize(self, ctxt, name, sub_demands):
        name = ego_constants.UNDERLINE.join([ego_constants.HEAT, name])
        return self.driver.ego_manager.ego_resize_alloc_for_heat(
                                            ctxt, name, sub_demands)

    def ego_alloc_free(self, ctxt, name):
        name = ego_constants.UNDERLINE.join([ego_constants.HEAT, name])
        self.driver.ego_manager.ego_free_alloc(name)

    def ego_get_release_candidates(self, ctxt, name,
                                    sub_demand_name, num_unit):
        name = ego_constants.UNDERLINE.join([ego_constants.HEAT, name])
        self.driver.ego_manager.ego_get_release_candidates(name,
                                        sub_demand_name, num_unit)

    def ego_get_possible_hosts(self, ctxt, decid, group_list, order_list,
                               filter_properties):
        return self.driver.ego_get_possible_hosts(ctxt, decid, group_list,
                                               order_list, filter_properties)

    def ego_get_resource_list(self, ctxt, cstr, state, is_attr_required):
        return self.driver.ego_manager.ego_get_resource_list(cstr, state,
                                                      is_attr_required)

    def update_aggregates(self, ctxt, aggregates):
        pass

    def delete_aggregate(self, ctxt, aggregate):
        pass

    def update_instance_info(self, context, host_name, instance_info):
        pass

    def delete_instance_info(self, context, host_name, instance_uuid):
        pass

    def sync_instance_info(self, context, host_name, instance_uuids):
        pass

    def get_instance_info(self, context, instance_uuid):
        return self.driver.ego_manager.get_instance_info(
            context, instance_uuid)
