# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import six

import nova.conf

from nova import db
from nova.scheduler.ibm.ego import filters

from oslo_log import log as logging

CONF = nova.conf.CONF

LOG = logging.getLogger(__name__)


class AggregateImagePropertiesIsolation(filters.EGOBaseFilter):
    """AggregateImagePropertiesIsolation works with image properties."""

    # Aggregate data and instance type does not change within a request
    run_filter_once_per_request = True

    def __init__(self):
        super(AggregateImagePropertiesIsolation, self).__init__()

    def parse(self, filter_properties, all_egohost_attrs):
        """Checks a host in an aggregate that metadata key/value match
        with image properties.
        """
        cfg_namespace = CONF.aggregate_image_properties_isolation_namespace
        cfg_separator = CONF.aggregate_image_properties_isolation_separator

        spec = filter_properties.get('request_spec', {})
        image_props = spec.get('image', {}).get('properties', {})
        context = filter_properties['context']

        includes = []
        excludes = []

        for ego_host in all_egohost_attrs.keys():
            metadata = db.aggregate_metadata_get_by_host(
                context, all_egohost_attrs[ego_host]['host'])

            succuss = True
            for key, options in six.iteritems(metadata):
                if (cfg_namespace and
                        not key.startswith(cfg_namespace + cfg_separator)):
                    self._exclude_hosts([ego_host])
                    continue
                prop = image_props.get(key)
                if prop and prop not in options:
                    succuss = False
                    LOG.debug("%(host_state)s fails image aggregate properties"
                              " requirements. Property %(prop)s does not "
                              "match %(options)s.",
                              {'host_state': ego_host,
                               'prop': prop,
                               'options': options})
                    break
            if succuss:
                includes.append(ego_host)
            else:
                excludes.append(ego_host)

        if includes:
            self._include_hosts(includes)
        if excludes:
            self._exclude_hosts(excludes)
