# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import six

from nova.ibm.ego import ego_gettextutils
from nova.scheduler.ibm.ego import ego_resreq_utils
from nova.scheduler.ibm.ego import filters

from oslo_log import log as logging

_ = ego_gettextutils._

LOG = logging.getLogger(__name__)

_SCOPE = 'aggregate_instance_extra_specs'


class AggregateInstanceExtraSpecsFilter(filters.EGOBaseFilter):
    # Aggregate data and instance type does not change within a request
    run_filter_once_per_request = True

    def __init__(self):
        super(AggregateInstanceExtraSpecsFilter, self).__init__()

    def parse(self, filter_properties, all_egohost_attrs):
        """
        Convert AggregateInstanceExtraSpecsFilter requirement
        to EGO res req
        """
        instance_type = filter_properties.get('instance_type')
        if 'extra_specs' not in instance_type:
            return
        extra_specs = {}
        for key, req in six.iteritems(instance_type['extra_specs']):
            # Either not scope format, or aggregate_instance_extra_specs scope
            scope = key.split(':', 1)
            if len(scope) > 1:
                if scope[0] != _SCOPE:
                    continue
                else:
                    del scope[0]
            key = scope[0]
            extra_specs[key] = req
        select_list = ego_resreq_utils.extra_specs_to_select(extra_specs)
        LOG.debug("After passing the AggregateInstanceExtraSpecsFilter "
                  "filter, the resource requirement expression "
                  "is (%s)." % select_list)
        self._select(select_list)
