# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import six

from nova.ibm.ego import ego_constants
from nova.ibm.ego import ego_gettextutils
from nova.scheduler.ibm.ego import filters

from oslo_log import log as logging

_ = ego_gettextutils._

LOG = logging.getLogger(__name__)


class AvailabilityZoneFilter(filters.EGOBaseFilter):
    # Availabilty zones do not change within a request
    run_filter_once_per_request = True

    def __init__(self):
        super(AvailabilityZoneFilter, self).__init__()
        self._set_filter_scope([ego_constants.FILTER_WORKING_SCOPE_CREATE])

    def parse(self, filter_properties, all_egohost_attrs):
        """
        Convert AvailabilityZoneFilter requirement
        to EGO res req
        """
        spec = filter_properties.get('request_spec', {})
        props = spec.get('instance_properties', {})
        availability_zone = props.get('availability_zone')
        if not availability_zone:
            return

        if isinstance(availability_zone, six.string_types):
            req_str = (
                "'" + availability_zone +
                "'" + " <in> $availability_zone")
        else:
            # availability_zone is list pass from heat resource
            # "OS::Heat::InstanceGroup" or
            # "AWS::AutoScaling::AutoScalingGroup"
            zone_str = ','.join([str(zone) for zone in availability_zone])
            req_str = (
                "'" + zone_str +
                "'" + " <in> $availability_zone")
        self._select(req_str)
        LOG.debug("After passing the AvailabilityZoneFilter filter, "
                  "the resource requirement expression "
                  "is (%s)." % req_str)
