# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import six

from nova.ibm.ego import ego_gettextutils
from nova import objects
from nova.scheduler.ibm.ego import ego_resreq_utils
from nova.scheduler.ibm.ego import filters

from oslo_log import log as logging

_ = ego_gettextutils._

LOG = logging.getLogger(__name__)


class ComputeCapabilitiesFilter(filters.EGOBaseFilter):
    # Instance type and host capabilities do not change within a request
    run_filter_once_per_request = True

    def __init__(self):
        super(ComputeCapabilitiesFilter, self).__init__()

    def parse(self, filter_properties, all_egohost_attrs):
        """
        Convert ComputeCapabilitiesFilter requirement to EGO res req

        :returns: return new resource requirement list
        """
        select_list = []
        spec = filter_properties.get('request_spec', {})
        props = spec.get('instance_type', {})
        instance_type_id = props['id']
        instance_type = objects.Flavor.get_by_id(filter_properties['context'],
                                                 instance_type_id)
        if not instance_type or 'extra_specs' not in instance_type:
            return
        extra_specs_list = instance_type['extra_specs']
        cap_list = []
        scope_flag = 0
        for key, req in six.iteritems(extra_specs_list):
            # Either not scope format, or in capabilities scope
            scope = key.split(':')
            if len(scope) > 1 and scope[0] != "capabilities":
                scope_flag = scope_flag + 1
                continue
            elif scope[0] == "capabilities":
                del scope[0]
                prev_dict = ''
                for capability in scope:
                    select_list.append("'$" + capability + "'")
                    if prev_dict:
                        select_list.append(
                            "'" + capability + "'" +
                            " <in> $" + "'" + prev_dict + "'")
                    prev_dict = capability
                cap_list.append((key, scope[-1], req))
            else:
                pass
        if scope_flag == len(instance_type['extra_specs']):
            LOG.debug("After passing the ComputeCapabilitiesFilter "
                      "filter, there are no new resource requirements.")
            return
        for cap_key, sub_key, sub_val in cap_list:
            extra_specs_list.pop(cap_key)
            extra_specs_list[sub_key] = sub_val
        select_list.extend(
            ego_resreq_utils.extra_specs_to_select(extra_specs_list))
        self._select(select_list)
        LOG.debug("After passing the ComputeCapabilitiesFilter "
                  "filter, the resource requirement expression "
                  "is (%s)." % select_list)
