# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from nova.ibm.ego import ego_constants
from nova.ibm.ego import ego_gettextutils
from nova.scheduler.ibm.ego import filters

from oslo_log import log as logging

_ = ego_gettextutils._

LOG = logging.getLogger(__name__)


class DiskFilter(filters.EGOBaseFilter):
    def __init__(self):
        super(DiskFilter, self).__init__()
        self._set_filter_extra_resreq_for_create(False)

    def parse(self, filter_properties, all_egohost_attrs):
        """
        Convert DiskFilter requirement to EGO res req
        """
        spec = filter_properties.get('request_spec', {})
        operation_type = spec.get('operation_type')
        instance_type = spec.get('instance_type')
        requested_disk = (1024 * (instance_type['root_gb'] +
                                 instance_type['ephemeral_gb']) +
                         instance_type['swap'])
        if operation_type == ego_constants.FILTER_WORKING_SCOPE_RESIZE:
            # if it's a resize, then we can reduce the total disk being
            # requested by disk's current size (this works if you're either
            # increasing or decreasing the disk's size)
            instance_properties = spec.get('instance_properties', {})
            curr_disk = 1024 * (instance_properties.get('root_gb', 0) +
                                instance_properties.get('ephemeral_gb', 0))
            requested_disk -= curr_disk
        if requested_disk < 0:
            req_str = "$usable_disk_mb>=0" + str(requested_disk)
        else:
            req_str = "$usable_disk_mb>=" + str(requested_disk)
        self._select(req_str)
        LOG.debug("After passing the DiskFilter filter, "
                  "the resource requirement expression is (%s)."
                    % req_str)


class AggregateDiskFilter(DiskFilter):
    """
    Convert AggregateDiskFilter requirement to EGO res req
    """
    pass
