# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from nova.ibm.ego import ego_gettextutils
from nova.scheduler.ibm.ego import ego_resreq_utils
from nova.scheduler.ibm.ego import filters

from oslo_log import log as logging

_ = ego_gettextutils._

LOG = logging.getLogger(__name__)


class ImagePropertiesFilter(filters.EGOBaseFilter):
    # Image properties and compute capabilities do not change within a request.
    run_filter_once_per_request = True

    def __init__(self):
        super(ImagePropertiesFilter, self).__init__()

    def parse(self, filter_properties, all_egohost_attrs):
        """
        Convert ImagePropertiesFilter requirement to EGO resreq
        """
        select_list = []
        hypervisor_data = ego_resreq_utils.get_hypervisor_data(
            filter_properties)
        image_arch = hypervisor_data['architecture']
        image_hypervisor_type = hypervisor_data['hypervisor_type']
        # For VMware ESXi case
        spec = filter_properties.get('request_spec', {})
        image_props = spec.get('image', {}).get('properties', {})
        image_prs_hypervisor_type = image_props.get('prs_hypervisor_type')
        image_vm_mode = image_props.get('vm_mode')
        checked_img_props = (image_arch, image_hypervisor_type, image_vm_mode)

        if not any(checked_img_props) or image_prs_hypervisor_type is not None:
            return select_list

        req_str = None
        if image_arch is not None:
            attr_type, res_str = ego_resreq_utils.generate_attr_str(
                image_arch, True)
            req_str = res_str
        if image_hypervisor_type is not None:
            if req_str:
                attr_type, res_str = ego_resreq_utils.generate_attr_str(
                    image_hypervisor_type, True)
                req_str = req_str + ',' + res_str
            else:
                attr_type, req_str = ego_resreq_utils.generate_attr_str(
                    image_hypervisor_type)

        if image_vm_mode is not None:
            if req_str:
                attr_type, res_str = ego_resreq_utils.generate_attr_str(
                    image_vm_mode, True)
                req_str = req_str + ',' + res_str
            else:
                attr_type, req_str = ego_resreq_utils.generate_attr_str(
                    image_vm_mode)

        req_str = req_str + ' <in> $supported_instances'

        self._select(req_str)
        LOG.debug("After passing the ImagePropertiesFilter filter, "
                  "the resource requirement expression is (%s)."
                    % req_str)
