# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import six

from nova.ibm.ego import ego_gettextutils
from nova.scheduler.ibm.ego import filters

from oslo_log import log as logging
from oslo_serialization import jsonutils

_ = ego_gettextutils._

LOG = logging.getLogger(__name__)


class JsonFilter(filters.EGOBaseFilter):
    """
    Allow simple JSON-based grammar for selecting hosts.
    """
    COMMANDS = {
        's==': 's==',
        's!=': 's!=',
        's>=': 's>=',
        's>': 's>',
        's<=': 's<=',
        's<': 's<',
        '=': '==',
        '==': '==',
        '<': "<",
        '>': ">",
        'in': '<in>',
        '<=': "<=",
        '>=': ">=",
        'not': "!",
        'or': "||",
        'and': '&&',
        '~': '~',
        '~*': '~*',
    }

    def __init__(self):
        super(JsonFilter, self).__init__()

    def __process_filter(self, query):
        """
        Recursively parse the query structure
        """
        if not query:
            return ''
        cmd = query[0]
        method = self.COMMANDS[cmd]
        cooked_args = []
        for arg in query[1:]:
            if isinstance(arg, list):
                arg = self.__process_filter(arg)
            if arg is not None:
                cooked_args.append(str(arg))

        result = "("
        index = 0
        if method == self.COMMANDS['not']:
            for arg in cooked_args[0:]:
                if index == 0:
                    result = result + "(" + method + " '" + arg + "')"
                else:
                    result = result + "&&" + "(" + method + " '" + arg + "')"
                index = index + 1
            result = result + ")"
            return result

        for arg in cooked_args[0:]:
            if index == len(cooked_args) - 1:
                result = result + "'" + arg + "'"
            else:
                result = result + "'" + arg + "'" + " " + method + " "
            index = index + 1
        result = result + ")"
        return result

    def parse(self, filter_properties, all_egohost_attrs):
        select_list = []
        # Convert JsonFilter requirement to EGO res req
        json_query = filter_properties.get('json_query', None)
        if not json_query:
            scheduler_hints = filter_properties.get('scheduler_hints') or {}
            query_list = scheduler_hints.get('query', [])
            if not query_list or len(query_list) == 0:
                return
        else:
            query_list = json_query
        if isinstance(query_list, six.string_types):
            singlejsonlist = eval(query_list)
            query = jsonutils.dumps(singlejsonlist)
            decodejson = jsonutils.loads(query)
            result = self.__process_filter(decodejson)
            result = result.replace("'(", "(").replace(")'", ")")
            select_list.append(result)
        else:
            for jsonstr in query_list:
                singlejsonlist = eval(jsonstr)
                query = jsonutils.dumps(singlejsonlist)
                decodejson = jsonutils.loads(query)
                result = self.__process_filter(decodejson)
                result = result.replace("'(", "(").replace(")'", ")")
                select_list.append(result)
        self._select(select_list)
        LOG.debug("After passing the JsonFilter filter, "
                  "the resource requirement expression is (%s)."
                    % select_list)
