# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
from oslo_config import cfg
from oslo_log import log as logging

from nova.scheduler.ibm.ego import filters
from nova.scheduler import utils

LOG = logging.getLogger(__name__)

CONF = cfg.CONF
CONF.import_opt('weight_setting',
                'nova.scheduler.weights.metrics',
                group='metrics')


class MetricsFilter(filters.EGOBaseFilter):
    """Metrics Filter

    This filter is used to filter out those hosts which don't have the
    corresponding metrics so these the metrics weigher won't fail due to
    these hosts.
    """

    def __init__(self):
        super(MetricsFilter, self).__init__()
        opts = utils.parse_options(CONF.metrics.weight_setting,
                                   sep='=',
                                   converter=float,
                                   name="metrics.weight_setting")
        self.keys = [x[0] for x in opts]

    def parse(self, filter_properties, all_egohost_attrs):
        includes = []
        excludes = []
        for ego_host in sorted(all_egohost_attrs):
            state = all_egohost_attrs[ego_host]
            unavail = [i for i in self.keys if i not in state['metrics']]
            if unavail:
                excludes.append(ego_host)
                LOG.debug("%(host_state)s does not have the following "
                          "metrics: %(metrics)s",
                          {'host_state': ego_host,
                           'metrics': ', '.join(unavail)})
            else:
                includes.append(ego_host)

        if includes:
            self._include_hosts(includes)
        if excludes:
            self._exclude_hosts(excludes)
