# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from oslo_config import cfg

from nova.scheduler import host_manager
from nova.scheduler.ibm.ego import filters
from nova.virt import hardware

CONF = cfg.CONF
CONF.import_opt('cpu_allocation_ratio', 'nova.scheduler.filters.core_filter')
CONF.import_opt('ram_allocation_ratio', 'nova.scheduler.filters.ram_filter')


class NUMATopologyFilter(filters.EGOBaseFilter):
    """Filter on requested NUMA topology."""
    numa_limits = {}

    def __init__(self):
        super(NUMATopologyFilter, self).__init__()

    @classmethod
    def get_numa_limits(cls, ego_host):
        return cls.numa_limits.get(ego_host)

    def parse(self, filter_properties, all_egohost_attrs):
        ram_ratio = CONF.ram_allocation_ratio
        cpu_ratio = CONF.cpu_allocation_ratio
        request_spec = filter_properties.get('request_spec', {})
        instance = request_spec.get('instance_properties', {})
        instance_topology = hardware.instance_topology_from_instance(instance)

        includes = []
        excludes = []

        for (ego_host, states) in all_egohost_attrs.items():
            host_state = host_manager.HostState(
                states['host'], None)
            host_state.numa_topology = states['numa_topology']
            host_topology, _fmt = hardware.host_topology_and_format_from_host(
                host_state)
            if instance_topology:
                if host_topology:
                    if not hardware.VirtNUMAHostTopology.can_fit_instances(
                            host_topology, [instance_topology]):
                        excludes.append(ego_host)
                        continue

                    limit_cells = []
                    usage_after_instance = (
                        hardware.VirtNUMAHostTopology.usage_from_instances(
                            host_topology, [instance_topology]))
                    success = True
                    for cell in usage_after_instance.cells:
                        max_cell_memory = int(cell.memory * ram_ratio)
                        max_cell_cpu = len(cell.cpuset) * cpu_ratio
                        if (cell.memory_usage > max_cell_memory or
                                cell.cpu_usage > max_cell_cpu):
                            success = False
                            break
                        limit_cells.append(
                            hardware.VirtNUMATopologyCellLimit(
                                cell.id, cell.cpuset, cell.memory,
                                max_cell_cpu, max_cell_memory))

                    if success:
                        self.__class__.numa_limits[ego_host] = (
                            hardware.VirtNUMALimitTopology(
                                cells=limit_cells).to_json())
                        includes.append(ego_host)
                    else:
                        excludes.append(ego_host)
                else:
                    excludes.append(ego_host)
                    continue
            else:
                includes.append(ego_host)

        if includes:
            self._include_hosts(includes)
        if excludes:
            self._exclude_hosts(excludes)
