# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from nova.ibm.ego import ego_constants
from nova.ibm.ego import ego_gettextutils
from nova.scheduler.ibm.ego import filters

from oslo_log import log as logging

_ = ego_gettextutils._

LOG = logging.getLogger(__name__)


class PowerMigrationLicenseFilter(filters.EGOBaseFilter):
    """
    This filter ensures that the hypervisor's license supports the migration
    operation.
    """

    def __init__(self):
        super(PowerMigrationLicenseFilter, self).__init__()
        self._set_filter_scope(
            [ego_constants.FILTER_WORKING_SCOPE_LIVE_MIGRATION,
             ego_constants.FILTER_WORKING_SCOPE_COLD_MIGRATION])

    def parse(self, filter_properties, all_egohost_attrs):
        if not self._is_verified_hypervisor(filter_properties,
                            list(ego_constants.POWERVM_HYPERVISOR_TYPES)):
            LOG.debug("Ignoring PowerMigrationLicenseFilter...")
            return
        request_spec = filter_properties.get('request_spec', {})
        op_type = request_spec.get('operation_type')
        if op_type == ego_constants.FILTER_WORKING_SCOPE_LIVE_MIGRATION:
            req_str = "$active_lpar_mobility_capable == True"
        else:
            req_str = "$inactive_lpar_mobility_capable == True"
        self._select(req_str)
        LOG.debug("After passing the PowerMigrationLicenseFilter filter, "
                  "the resource requirement expression "
                  "is (%s)." % req_str)
