# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from nova.ibm.ego import ego_gettextutils
from nova.scheduler.ibm.ego import filters

from oslo_log import log as logging

_ = ego_gettextutils._

LOG = logging.getLogger(__name__)


class TypeAffinityFilter(filters.EGOBaseFilter):
    run_filter_once_per_request = True

    def __init__(self):
        super(TypeAffinityFilter, self).__init__()

    def parse(self, filter_properties, all_egohost_attrs):
        """
        Convert TypeAffinityFilter requirement to EGO res req
        """
        instance_type = filter_properties.get('instance_type')
        req_str = ("($run_instance_type==%s)||($run_instance_type s== dynamic)"
                    % instance_type['flavorid'])
        self._select(req_str)
        LOG.debug("After passing the TypeAffinityFilter filter, "
                  "the resource requirement expression is (%s)."
                    % req_str)


class AggregateTypeAffinityFilter(filters.EGOBaseFilter):
    # Aggregate data does not change within a request
    run_filter_once_per_request = True

    def __init__(self):
        super(AggregateTypeAffinityFilter, self).__init__()

    def parse(self, filter_properties, all_egohost_attrs):
        """
        Convert AggregateTypeAffinityFilter requirement to EGO res req
        """
        instance_type = filter_properties.get('instance_type')
        req_str = ("'%s' <in> $instance_type || $instance_type s== dynamic"
                    % instance_type['name'])
        self._select(req_str)
        LOG.debug("After passing the AggregateTypeAffinityFilter filter, "
                  "the resource requirement expression "
                  "is (%s)." % req_str)
