# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import six

from nova import exception
from nova.ibm.ego import ego_constants
from nova.ibm.ego import ego_gettextutils
from nova.scheduler.ibm.ego import ego_resreq_utils
from nova.scheduler.ibm.ego import filters

from oslo_log import log as logging
from oslo_serialization import jsonutils

_ = ego_gettextutils._

LOG = logging.getLogger(__name__)


class VMwareDataCenterFilter(filters.EGOBaseFilter):

    """
    This filter ensures that data center name on the source
    equals to destination host as this is a requirement for live migration
    and cold migration
    """
    run_filter_once_per_request = True

    def __init__(self):
        super(VMwareDataCenterFilter, self).__init__()
        self._set_filter_scope(
            [ego_constants.FILTER_WORKING_SCOPE_LIVE_MIGRATION,
             ego_constants.FILTER_WORKING_SCOPE_COLD_MIGRATION])

    def parse(self, filter_properties, all_egohost_attrs):
        if not self._is_verified_hypervisor(filter_properties,
                            ego_constants.HYPERVISOR_TYPE_ESX_IMAGE):
            LOG.debug("Ignoring VMwareDataCenterFilter...")
            return
        request_spec = filter_properties.get('request_spec', {})
        instance_properties = request_spec.get('instance_properties', {})
        ego_host = instance_properties.get('ego_host')
        for hostname, host_data in six.iteritems(all_egohost_attrs):
            if ego_host == hostname:
                host_metrics = jsonutils.loads(host_data['metrics'])
                metrics_map = ego_resreq_utils.list2map(host_metrics, 'name')
                dc_name = metrics_map.get('datacenterInfo')
                if not dc_name:
                    msg = _('Data Center not found for host (%s)') % ego_host
                    raise exception.MigrationPreCheckError(msg)
                req_str = "$datacenterInfo==" + str(dc_name)
                self._select(req_str)
                LOG.debug("After passing the VMwareDataCenterFilter filter, "
                          "the resource requirement expression "
                          "is (%s)." % req_str)
                break
