# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import six

from nova.ibm.ego import ego_gettextutils
from nova.scheduler.ibm.ego import filters

from oslo_log import log as logging

_ = ego_gettextutils._
_LE = ego_gettextutils._LE

LOG = logging.getLogger(__name__)


class zHPMFilter(filters.EGOBaseFilter):
    """
    This filter ensures that:
    1) In hybrid environment, a KVM/QEMU instance
    without specifying WRG(Workload Resource Group)
    will be placed on:
        * hypervisor without zHPM installed
        * hypervisor with zHPM installed, but "cpu_mgmt" is off
    2) If a KVM/QEMU instance with WRG assigned will be
    placed on a hypervisor which has zHPM installed and
    "cpu_mgmt"/"cpu_cooperative_mode" is on.
    """
    run_filter_once_per_request = True

    def __init__(self):
        super(zHPMFilter, self).__init__()
        self._set_filter_extra_resreq_for_create(False)

    def parse(self, filter_properties, all_egohost_attrs):
        num_zhpm_hosts = 0
        for egohost, host_attrs in six.iteritems(all_egohost_attrs):
            cpu_mgmt = host_attrs['metrics'].get('cpu_mgmt')
            zhpm_cooperative_mode = host_attrs['metrics'].get(
                'zhpm_cooperative_mode')
            if cpu_mgmt == 'on' and zhpm_cooperative_mode == 'on':
                num_zhpm_hosts += 1
        request_spec = filter_properties.get('request_spec', {})
        if not request_spec.get("wrg_def"):
            req_str = "(! $cpu_mgmt)||($cpu_mgmt==off)"
            if num_zhpm_hosts == len(all_egohost_attrs.keys()):
                LOG.error(_LE("Platform Resource Scheduler detected "
                "insufficient resources due to the flavor not having a "
                "workload resource group associated but all hosts in cluster "
                "are zHPM-enabled hosts."))
        else:
            req_str = "($cpu_mgmt==on)&&($zhpm_cooperative_mode==on)"
            if num_zhpm_hosts == 0:
                LOG.error(_LE("Platform Resource Scheduler detected "
                "insufficient resources due to not having any zHPM-enabled "
                "hosts in cluster."))
        self._select(req_str)
        LOG.debug("After passing the zHPMFilter filter, "
            "the resource requirement expression is (%s)." % req_str)
