# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import binascii

from oslo_config.ibm import secret_opt_base_handler


class SecretOptHandler(secret_opt_base_handler.SecretOptHandlerBase):

    ENCODED_PREFIX = '[IBM:v1]'

    def _is_encoded(self, value):
        """Accept a value and return None if the value is not encoded.

        :param value:The text to be verified
        :returns: None if the value is not encoded,
        else return the value decoded by 'base64'.
        """
        try:
            decoded = value.decode('base64')
        except binascii.Error:
            return None
        except AttributeError:
            return None

        if decoded.startswith(self.ENCODED_PREFIX):
            return decoded
        return None

    def decode(self, opt_value):
        """Decode the opt value.

        If the opt is not encoded, return the given value.
        :param opt_value:The text to be decoded.
        :returns: The decoded value
        """

        decoded_base64 = self._is_encoded(opt_value)
        if decoded_base64 is not None:
            len_mask = len(self.ENCODED_PREFIX)
            # NOTE (yahongdu): rot13 will be unavailable in python 3.x.
            temp_value = decoded_base64[len_mask:].decode('rot13')
            # strip() is used to remove the new line character
            # added by encode or decode.
            return str(temp_value).strip()
        return opt_value
