# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import ConfigParser

from oslo_config.ibm import secret_opt_handler

import sys


def decode_opt(opt_value):

    value = decode_by_external_handler(opt_value)
    if value is None:
        value = secret_opt_handler.SecretOptHandler().decode(opt_value)
    return value


def decode_by_external_handler(opt_value):

    from oslo_config import cfg
    OSLO_CONF_PATHS = cfg.find_config_files('oslo')

    if (OSLO_CONF_PATHS is not None) and (len(OSLO_CONF_PATHS) > 0):
        OSLO_CONF_PATH = OSLO_CONF_PATHS[0]
        config = ConfigParser.RawConfigParser()
        config.read(OSLO_CONF_PATH)
        try:
            handler_path = config.get("DEFAULT", "secret_opt_handler")
            length = len(handler_path.split("."))
            if (length > 1):
                handler = ".".join(handler_path.split(".")[:(length - 1)])
                handler_class = handler_path.split(".")[length - 1]
                try:
                    __import__(handler)
                    decode_class = getattr(sys.modules[handler], handler_class)
                    value = decode_class().decode(opt_value)
                    return value
                except ImportError:
                    return None
                except AttributeError:
                    return None
        except ConfigParser.NoSectionError:
            return None
        except ConfigParser.NoOptionError:
            return None
