# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
from powervc_oslo.config import data_utils
from powervc_oslo.config import config_utils as cutils
from powervc_ceilometer import _

CEILOMETER_CONF = '/etc/ceilometer/ceilometer.conf'
SECTION_DB = 'database'
TTL_DICT = {'meter_ttl': 'metering_time_to_live',
            'event_ttl': 'event_time_to_live'}
default_config = None


def ceilometer_meter_data_expiry(args):
    global default_config
    unit = cutils.SEC
    if args.unit is not None:
        unit = args.unit
    if args.set is None:
        # reading
        value = data_utils.get_openstack_config_value(CEILOMETER_CONF,
                                                      TTL_DICT['meter_ttl'],
                                                      SECTION_DB)
        cutils.print_time_output(args, unit, value, cutils.CURRENT)
        def_val = (default_config[CEILOMETER_CONF][SECTION_DB]
                   [TTL_DICT['meter_ttl']][0])
        cutils.print_time_output(args, unit, def_val, cutils.DEFAULT)
    else:
        # writing
        cutils.check_is_negative(args.set)
        new_val = cutils.convert_time(args.set, unit, cutils.SEC)
        cutils.check_max_time(new_val)
        display_val = '%s %s' % (args.set, unit)
        cutils.safe_print(_('Setting %(key)s to %(val)s') %
                          dict(key=TTL_DICT['meter_ttl'], val=display_val))
        data_utils.set_openstack_config_value(CEILOMETER_CONF,
                                              TTL_DICT['meter_ttl'],
                                              str(new_val),
                                              SECTION_DB)


def ceilometer_event_data_expiry(args):
    global default_config
    unit = cutils.SEC
    if args.unit is not None:
        unit = args.unit
    if args.set is None:
        # reading
        value = data_utils.get_openstack_config_value(CEILOMETER_CONF,
                                                      TTL_DICT['event_ttl'],
                                                      SECTION_DB)
        cutils.print_time_output(args, unit, value, cutils.CURRENT)
        def_val = (default_config[CEILOMETER_CONF][SECTION_DB]
                   [TTL_DICT['event_ttl']][0])
        cutils.print_time_output(args, unit, def_val, cutils.DEFAULT)
    else:
        # writing
        cutils.check_is_negative(args.set)
        new_val = cutils.convert_time(args.set, unit, cutils.SEC)
        cutils.check_max_time(new_val)
        display_val = '%s %s' % (args.set, unit)
        cutils.safe_print(_('Setting %s to %s') %
                          (TTL_DICT['event_ttl'], display_val))
        data_utils.set_openstack_config_value(CEILOMETER_CONF,
                                              TTL_DICT['event_ttl'],
                                              str(new_val),
                                              SECTION_DB)


def setup_parser(metering_subparser, config):
    global default_config
    default_config = config
    # Setup parser for meter_ttl
    ceilo_meter_ttl_parser = metering_subparser.add_parser(
        'meter_ttl',
        help=_(
            'Configure the time to live interval for the meter data.'),
        description=_(
            'Run this sub-command with --unit option to view the '
            'current setting'))
    ceilo_meter_ttl_parser.add_argument(
        '--set',
        help=_('Set the time to live interval for the meter data.'),
        metavar='VALUE',
        type=int,
        default=None)
    ceilo_meter_ttl_parser.add_argument(
        '--unit',
        help=_(
            'Unit of measure for this setting. Accepted values are sec, '
            'min, hr. Default unit is seconds. '),
        choices=['sec', 'min', 'hr'],
        default=None)
    ceilo_meter_ttl_parser.set_defaults(func=ceilometer_meter_data_expiry)

    # Setup parser for event_ttl
    ceilo_event_ttl_parser = metering_subparser.add_parser(
        'event_ttl',
        help=_(
            'Configure the time to live interval for the event data.'),
        description=_(
            'Run this sub-command with --unit option to view the '
            'current setting'))
    ceilo_event_ttl_parser.add_argument(
        '--set',
        help=_('Set the time to live interval for the event data.'),
        metavar='VALUE',
        type=int,
        default=None)
    ceilo_event_ttl_parser.add_argument(
        '--unit',
        help=_(
            'Unit of measure for this setting. Accepted values are sec, '
            'min, hr. Default unit is seconds. '),
        choices=['sec', 'min', 'hr'],
        default=None)
    ceilo_event_ttl_parser.set_defaults(func=ceilometer_event_data_expiry)
