#
#
# Copyright 2015 IBM Corp.
# All Rights Reserved.
#
#    Copyright (c) 2013 OpenStack Foundation
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron.common import topics
from neutron.extensions import portbindings
from neutron.plugins.common import constants as p_constants
from neutron.plugins.ml2 import driver_api as api
from neutron.plugins.ml2.drivers.\
    openvswitch.mech_driver import mech_openvswitch as mech_ovs
from neutron.plugins.ml2 import rpc
from oslo_log import log
from powervc_neutron.plugins.ml2.drivers import mech_ibmpower
from powervc_neutron.plugins.pvm.agent import constants

LOG = log.getLogger(__name__)


class PvcSEAMechanismDriver(mech_ibmpower.IBMPowerMechanismDriver):
    """Attach to networks using PowerVC Shared Ethernet agent.

    The PvcSEAMechanismDriver integrates the ml2 plugin with the
    PowerVC Shared Ethernet Agent.
    """

    def __init__(self):
        LOG.debug('PowerVC SEA Agent mechanism driver loaded')
        # Calling super with OVS mechanism driver class so we can get down to
        # the SimpleAgentMechanismDriverBase __init__ and send in our agent
        # type, vif type, etc
        super(mech_ovs.OpenvswitchMechanismDriver, self).__init__(
            constants.AGENT_TYPE_PVC_SEA, constants.VIF_TYPE_PVC_SEA,
            {portbindings.CAP_PORT_FILTER: False})
        self.rpc_publisher = rpc.AgentNotifierApi(topics.AGENT)

        # Do not support hybrid plugs
        self.vif_details[portbindings.OVS_HYBRID_PLUG] = False

    def check_segment_for_agent(self, segment, agent):
        LOG.debug("Checking segment: %s", segment)
        network_type = segment[api.NETWORK_TYPE]
        return network_type in ['vlan']

    def try_to_bind_segment_for_agent(self, context, segment, agent):
        # When this method is called, the parent should ideally be calling
        # down to the agent to state that the port was updated.  However,
        # it appears this isn't flowing properly.  This makes sure the
        # port is passed down to the agent.
        bindable = (super(PvcSEAMechanismDriver, self).
                    try_to_bind_segment_for_agent(context, segment, agent))
        if bindable:
            self.rpc_publisher.port_update(
                context._plugin_context, context._port,
                segment[api.NETWORK_TYPE], segment[api.SEGMENTATION_ID],
                segment[api.PHYSICAL_NETWORK])
        return bindable

    def get_allowed_network_types(self, agent=None):
        return [p_constants.TYPE_VLAN]

    def get_mappings(self, agent):
        return agent['configurations'].get('interface_mappings', {})
