#
#
# Copyright 2015 IBM Corp.
# All Rights Reserved.
#
#   Copyright 2011 Nicira Networks, Inc.
#   All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import os
import shutil
import sys

from neutron.agent.linux import utils as lutils
from neutron.i18n import _
from powervc_neutron.plugins.pvm.agent import constants


def config():
    '''
    Performs the agent installation configuration.
    '''
    # Turn on sea agent for system restart
    full_args = ['systemctl', 'enable', constants.AGENT_BINARY_PVC_SEA +
                 '.service']
    lutils.execute(full_args)

    # Copy the sea_agent_startup script and set permissions
    script_dir = os.path.dirname(os.path.realpath(__file__))
    shutil.copy(script_dir + '/powervc_pvm_sea_agent_startup.py',
                '/usr/bin/' + constants.AGENT_BINARY_PVC_SEA)
    full_args = ['chmod', '755', '/usr/bin/' + constants.AGENT_BINARY_PVC_SEA]
    lutils.execute(full_args)

    # Since we're updating the neutron.conf file, we need to update the
    # permissions to allow us to modify it
    full_args = ["chmod", "660", constants.NEUTRON_CONF]
    lutils.execute(full_args)


def unconfig():
    '''
    Will unconfigure the system.  Stops the processes, removes /usr/bin script
    '''
    # Stop the agent
    full_args = ['systemctl', 'stop', constants.AGENT_BINARY_PVC_SEA +
                 '.service']
    lutils.execute(full_args)

    if os.path.isfile('/usr/bin/' + constants.AGENT_BINARY_PVC_SEA):
        os.remove('/usr/bin/' + constants.AGENT_BINARY_PVC_SEA)


if __name__ == '__main__':
    if len(sys.argv) <= 1:
        print _("Must specify 'configure' or 'unconfigure' as input")
    else:
        option = sys.argv[1]
        if option == 'configure':
            config()
        elif option == 'unconfigure':
            unconfig()
        else:
            print _("Must specify 'configure' or 'unconfigure' as input")
    sys.exit(0)
