#
#
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014, 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import functools

from stevedore import extension
from nova.api.openstack.compute.legacy_v2.contrib import standard_extensions
from oslo_log import log as logging
from powervc_nova import _
from oslo_config import cfg

LOG = logging.getLogger(__name__)
CONF = cfg.CONF

# NOTE(tpeoples): This import is needed in order to load the monkey patches
# within powervc_nova.conductor.manager for the nova-api process(es).
# TODO(tpeoples): Once these specific monkey patches are converted to changes
# within the PowerVC OpenStack git repository, this can be removed.
__import__('powervc_nova.conductor.manager')


def api_extensions_v2(ext_mgr):
    """Factory to load all Nova v2 API Extensions (PowerVC + Core)"""
    # First load all the standard extensions from the Nova Contrib directory
    standard_extensions(ext_mgr)
    # Now we can load the PowerVC extensions specified in the Extension Point
    v2mgr = extension.ExtensionManager('nova.api.v2.extensions')
    for ext in v2mgr.extensions:
        try:
            ext_mgr.load_extension(ext.plugin)
        except Exception as ex:
            LOG.warn(_('Failed to load extension %(name)s: %(ex)s'),
                     dict(name=ext.name, ex=ex))
            LOG.exception(ex)


def api_extensions_v3(ext_mgr):
    """Factory to load all Nova v3 API Extensions (PowerVC + Core)"""
    # Now we can load the PowerVC extensions specified in the Extension Point
    v3mgr = extension.ExtensionManager('nova.api.v3.extensions')
    for ext in v3mgr.extensions:
        try:
            ext_mgr.load_extension(ext.plugin)
        except Exception as ex:
            LOG.warn(_('Failed to load extension %(name)s: %(ex)s'),
                     dict(name=ext.name, ex=ex))
            LOG.exception(ex)
