#
#
# Copyright 2014, 2015 IBM Corp.
# All Rights Reserved.
# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import math

from nova.compute.resources import vcpu as base_vcpu
from oslo_log import log as logging

LOG = logging.getLogger(__name__)


class VCPU(base_vcpu.VCPU):
    def __init__(self):
        super(VCPU, self).__init__()

    def _get_proc_units(self, usage):
        proc_units = 0
        power_specs = usage.get('power_specs', None)
        if power_specs is not None:
            proc_units = power_specs.get('proc_units', 0)
            if proc_units is None:
                proc_units = 0
        return proc_units

    def add_instance(self, usage):
        requested = int(usage.get('vcpus', 0))
        proc_units = self._get_proc_units(usage)
        requested = proc_units if proc_units != 0 else requested
        self._used += requested

    def remove_instance(self, usage):
        requested = int(usage.get('vcpus', 0))
        proc_units = self._get_proc_units(usage)
        requested = proc_units if proc_units != 0 else requested
        self._used -= requested

    def write(self, resources):
        resources['vcpus'] = self._total
        resources['vcpus_used'] = int(math.ceil(float(self._used)))
