# =================================================================
#
# Copyright 2013, 2015 IBM Corp.
# All Rights Reserved.
# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import oslo_config.cfg

# Importing full names to not pollute the namespace and cause possible
# collisions with use of 'from nova.network import <foo>' elsewhere.
from oslo_utils import importutils

import nova.network
import nova.network.security_group.openstack_driver


def placement_module():
    host_type = oslo_config.cfg.CONF.host_type.lower()
    if 'powervm' in host_type:
        placement_class = 'powervc_nova.network.ibmpowervm.placement_pvm.' +\
            'IBMPowerVMNetworkPlacementPVM'
    elif 'kvm' in host_type:
        placement_class = 'powervc_nova.network.powerkvm.placement_kvm.' +\
            'IBMPowerVMNetworkPlacementKVM'
    cls = importutils.import_class(placement_class)
    return cls()


# The OpenStack code has deprecated the ability to extend the Network API
# class and with that setup it works now only for Nova Networks, not for
# Neutron Networks, so we are overriding to allow it to continue to work
def _powervc_is_neutron():
    """Override to say that we are using Neutron for Networking"""
    return True


def _powervc_API(skip_policy_check=False):
    """Override to allow the Network API Class to still be extended"""
    cls = importutils.import_class(oslo_config.cfg.CONF.network_api_class)
    return cls(skip_policy_check=skip_policy_check)


def _powervc_is_neutron_security_groups():
    """Override to still use the Nova Networks Security Groups default"""
    return False

nova.network.is_neutron = _powervc_is_neutron
nova.network.API = _powervc_API
nova.network.security_group.openstack_driver.\
    is_neutron_security_groups = _powervc_is_neutron_security_groups
