# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from webob import exc
from nova import exception
import powervc_oslo

from powervc_nova import _


class IBMPowerException(exception.NovaException):
    def format_translated_message(self, locale_context):
        msg_fmt = powervc_oslo.translate_message(locale_context,
                                                 self.msg_fmt)
        return msg_fmt % self.kwargs


class IBMPowerHTTPNotImplemented(exc.HTTPNotImplemented):
    def format_translated_message(self, locale_context):
        return powervc_oslo.translate_message(locale_context,
                                              self.explanation)


class IBMPowerInvalid(exception.Invalid):
    def format_translated_message(self, locale_context):
        msg_fmt = powervc_oslo.translate_message(locale_context,
                                                 self.msg_fmt)
        return msg_fmt % self.kwargs


class IBMPowerVMNetworkAPIUnsupported(IBMPowerException):
    msg_fmt = _('This API is not supported in this environment.')


class IBMPowerVMNetworkAPIUnsupportedOutsidePowerVM(
        IBMPowerHTTPNotImplemented):
    explanation = _('The API is not supported outside a PowerVM environment.')


class IBMPowerVMNetworkAPIUnsupportedOutsidePowerKVM(
        IBMPowerHTTPNotImplemented):
    explanation = _('The API is not supported outside a PowerKVM environment.')


class IBMPowerVMInvalidVlan(IBMPowerInvalid):
    msg_fmt = _("The specified VLAN '%(vlan)s' is invalid.  A valid VLAN is "
                "an integer, typically in the range 1-4094 inclusive.")


class IBMPowerKVMVswitchNotFound(IBMPowerInvalid):
    msg_fmt = _("Virtual Switch not found: %(ovs_name)s.")


class IBMPowerKVMVswitchUpdateException(IBMPowerException):
    msg_fmt = _("An exception occurred updating the switch.  Check "
                "/var/log/nova/nova-api.log for details.")


class IBMPowerKVMOneHostAtATime(IBMPowerInvalid):
    msg_fmt = _("This API can only run on one host at a time.")


class IBMPowerKVMHostNotFound(IBMPowerInvalid):
    msg_fmt = _("The specified host name %(host_name)s was not found.")
