# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""
    The purpose of this file is to delete network associations
    periodically for the deleted networks
"""
from oslo_log import log as logging
from nova import network as net_root

from powervc_nova.objects.network import dom as network_dom

LOG = logging.getLogger(__name__)


def cleanup_network_associations(context, host_name, transaction=None):
    """
    Removes network associations for deleted networks
    periodically
    :param context: The context for building the data
    :param host_name: The name of the host
    """

    LOG.debug('Entry: network associations cleanup task')
    netafact = network_dom.NetworkAssociationFactory.get_factory()
    if not transaction:
        transaction = netafact.get_resource_transaction()

    nova_net_api = net_root.API()
    neutron_network_ids = set()
    net_assn_neutron_ids = set()
    net_assn_map = dict()
    # retrieve existing quantum networks
    existing_neutron_networks = nova_net_api.get_all(context)
    for neutron_network in existing_neutron_networks:
        neutron_network_ids.add(neutron_network.get("uuid"))

    with transaction.begin(subtransactions=True):
        # retrieve unique network ids from existing network associations
        existing_association_networks = netafact.\
            find_all_network_assocs_by_host(
                context, host_name, transaction)

        for net_assn in existing_association_networks:
            net_assn_neutron_ids.add(net_assn.neutron_net_id)
            net_assn_map[net_assn.neutron_net_id] = \
                net_assn_map.get(net_assn.neutron_net_id, [])
            net_assn_map[net_assn.neutron_net_id].append(net_assn)

        deleted_network_ids = net_assn_neutron_ids - \
            neutron_network_ids

        # delete associations of deleted networks
        for networkid in deleted_network_ids:
            LOG.debug('network associations cleanup'
                      ' for networkid ' + str(networkid))

            for net_association in net_assn_map[networkid]:
                net_association.delete()
    LOG.debug('Exit: network associations cleanup task')
