#
#
# Copyright 2015 IBM Corp.
# All Rights Reserved.
#
#    Copyright 2013 Red Hat Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from powervc_nova.db import api as db_api
from nova.objects import block_device as blk_dev
from nova.objects import base

_BLOCK_DEVICE_OPTIONAL_JOINED_FIELD = ['instance']
BLOCK_DEVICE_OPTIONAL_ATTRS = _BLOCK_DEVICE_OPTIONAL_JOINED_FIELD


def _expected_cols(expected_attrs):
    return [attr for attr in expected_attrs
            if attr in _BLOCK_DEVICE_OPTIONAL_JOINED_FIELD]


class PowerVCBlockDeviceMappingList(blk_dev.BlockDeviceMappingList):

    @base.remotable_classmethod
    def get_by_volume_id(cls, context, volume_id, expected_attrs=None):
        if expected_attrs is None:
            expected_attrs = []
        db_bdms = db_api.block_device_mapping_get_all_by_volume_id(
            context, volume_id,
            _expected_cols(expected_attrs))
        return base.obj_make_list(
            context, cls(), blk_dev.BlockDeviceMapping, db_bdms or [])
