# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from nova.ibm.ego import ego_constants

"""PowerVC PRS constants."""

DRO_ENABLED_KEY = 'dro_enabled'

# Aggregate default initial placement settings.
DEFAULT_INITIAL_POLICY_ID = '1'

# Aggregate default HA settings.
DEFAULT_HA_ID = '1'
DEFAULT_HA_RUN_INTERVAL = '1'
DEFAULT_HA_STABILIZATION = '1'

# DRO Functions
DRO_FUNCTION_HOST_OPTIMIZATION = 'host_optimization'
DRO_FUNCTIONS = [DRO_FUNCTION_HOST_OPTIMIZATION]

# DRO Scheduling Keys
DRO_SCHEDULES = 'dro_schedules'
DRO_SCHEDULES_ENABLED_KEY = 'schedule_enabled'
DRO_SCHEDULES_RULES_KEY = 'rules'
DRO_SCHEDULES_TIME_MASK = '%H:%M UTC'

# Aggregate default DRO settings.
DEFAULT_DRO_ENABLED = 'False'
DEFAULT_RT_POLICY_ID = '5'
DEFAULT_RT_RUN_INTERVAL = '5'
DEFAULT_RT_STABILIZATION = '2'
DEFAULT_RT_THRESHOLD = '70'
DEFAULT_RT_MAX_PARALLEL = '10'
DEFAULT_RT_ACTION = 'migrate_vm_advise_only'

#Prefixes for DRO event types
_DRO_EVENTS_PREFIX = 'powervc.dro.'
_DRO_SCHED_EVENTS_PREFIX = _DRO_EVENTS_PREFIX + "schedule."

# DRO scheduling event types
DRO_SCHED_HOST_OPTIMIZATION_START = \
    'powervc.dro.schedule.host_optimization.start'
DRO_SCHED_HOST_OPTIMIZATION_END = \
    'powervc.dro.schedule.host_optimization.end'

# Event types for which CPU utilization is added to payload.
DRO_ADD_CPU_UTIL_EVENT_TYPES = [
    ego_constants.PRS_RTPOLICY_ACTION_START,
    ego_constants.PRS_RTPOLICY_ACTION_END,
    ego_constants.PRS_RTPOLICY_HYPERVISOR_VIOLATED,
    ego_constants.PRS_RTPOLICY_HYPERVISOR_RECOVERED]
DRO_DB_AGGREGATE_EVENT_TYPES = [
    DRO_SCHED_HOST_OPTIMIZATION_START,
    DRO_SCHED_HOST_OPTIMIZATION_END]
DRO_DB_HYPERVISOR_EVENT_TYPES = [
    ego_constants.PRS_RTPOLICY_ACTION_START,
    ego_constants.PRS_RTPOLICY_ACTION_END,
    ego_constants.PRS_RTPOLICY_HYPERVISOR_ALERT,
    ego_constants.PRS_RTPOLICY_HYPERVISOR_VIOLATED,
    ego_constants.PRS_RTPOLICY_HYPERVISOR_RECOVERED]
# Event types to store in the database.
DRO_DB_EVENT_TYPES = (DRO_DB_AGGREGATE_EVENT_TYPES +
                      DRO_DB_HYPERVISOR_EVENT_TYPES)

DRO_EVENT_TABLE_EVENT_TIME = 'event_time'
DRO_EVENT_TABLE_EVENT_TYPE = 'event_type'
DRO_EVENT_TABLE_HYPERVISOR_HOSTNAME = 'hypervisor_hostname'
DRO_EVENT_TABLE_AGGREGATE_ID = 'aggregate_id'
DRO_EVENT_TABLE_PAYLOAD = 'payload'

CPU_COD_RT_ACTION = 'cpu_cod'
CPU_COD_ADVISE_RT_ACTION = 'cpu_cod_advise'
CPU_COD_MIGRATE_VM_RT_ACTION = 'cpu_cod_then_migrate'
CPU_COD_MIGRATE_VM_ADVISE_RT_ACTION = 'cpu_cod_then_migrate_advise'

COD_POLICY_ACTIONS = (CPU_COD_RT_ACTION,
                      CPU_COD_ADVISE_RT_ACTION,
                      CPU_COD_MIGRATE_VM_RT_ACTION,
                      CPU_COD_MIGRATE_VM_ADVISE_RT_ACTION)

COD_ADVISE_POLICY_ACTIONS = (CPU_COD_ADVISE_RT_ACTION,
                             CPU_COD_MIGRATE_VM_ADVISE_RT_ACTION)

COD_MIGRATE_POLICY_ACTIONS = (CPU_COD_MIGRATE_VM_RT_ACTION,
                              CPU_COD_MIGRATE_VM_ADVISE_RT_ACTION)

# This flag is used to determine whether the number of mobile cores to add to
# a host per optimization cycle should be restricted to a fixed number
COD_RESTRICT_CORE_ACTIVATIONS_PER_CYCLE = False
# Maximum number of mobile cores that should be moved to a host per
# optimization cycle
COD_MAX_CORE_ACTIVATIONS_PER_CYCLE = 1
