# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import six

from oslo_log import log as logging

from nova.ibm.ego import ego_constants

from powervc_nova.prs import utils as prs_utils
from powervc_nova.scheduler.filters import PowerVCBaseFilter
from powervc_nova import logcall

LOG = logging.getLogger(__name__)


class PowerVCDROFilter(PowerVCBaseFilter):
    """
    This EGO scheduler filter is responsible for filtering out destination
    hosts that have DRO disabled; this filter is only applicable when
    live migrations are initiated by the DRO engine.
    """

    def __init__(self):
        super(PowerVCDROFilter, self).__init__()
        self._set_filter_scope(
            [ego_constants.FILTER_WORKING_SCOPE_LIVE_MIGRATION])

    @logcall
    def parse(self, filter_properties, all_host_attrs):
        context = filter_properties.get('context')
        request_spec = filter_properties.get('request_spec', {})
        invoked_by_dro = request_spec.get('invoked_by_rtpolicy', False)

        if not invoked_by_dro:
            # A non-DRO live migration was invoked, so no filtering is required
            return

        include_hosts = []

        # Iterate over all the hosts and check whether they have DRO disabled
        for hostname, host_data in six.iteritems(all_host_attrs):
            compute_node = prs_utils.get_compute_node(context, hostname)
            dro_enabled = prs_utils.get_dro_enabled_for_host(
                context, compute_node, check_host_groups=True)
            if dro_enabled:
                include_hosts.append(hostname)

        LOG.debug("Setting included hosts in %(filter)s (%(hosts)s)."
                  % dict(filter=self.__class__.__name__,
                         hosts=six.text_type(include_hosts)))
        self._include_hosts(include_hosts)
