# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from oslo_log import log as logging

from nova import objects
from nova.ibm.ego import ego_constants

from powervc_nova.scheduler.filters import PowerVCBaseFilter
from powervc_nova import logcall
from powervc_nova import _

LOG = logging.getLogger(__name__)


class PowerVCResizeSameHostFilter(PowerVCBaseFilter):
    """
    This EGO scheduler filter is responsible for keeping the VM on the same
    host system on which it currently resides.
    """

    def __init__(self):
        super(PowerVCResizeSameHostFilter, self).__init__()
        self._set_filter_scope([ego_constants.FILTER_WORKING_SCOPE_RESIZE])
        self._set_error_message(_("The resize operation failed because "
                                  "the instance does not have its 'host' "
                                  "set. Please retry your request in a few "
                                  "minutes."))

    @logcall
    def parse(self, filter_properties, all_host_attrs):
        request_spec = filter_properties.get('request_spec', {})
        instance_props = request_spec.get('instance_properties', {})
        inst = objects.Instance.get_by_uuid(filter_properties.get('context'),
                                            instance_props.get('uuid'))

        if not inst.host:
            LOG.error(_("Could not determine host for %(inst)s; %(filter)s "
                        "returning no valid hosts.") % dict(
                      inst=inst.uuid, filter=self.__class__.__name__))
            self._include_hosts([])
            return

        req_str = "$host == '%s'" % inst.host

        self._select(req_str)
        LOG.debug("After passing the %(filter)s, the resource requirement "
                  "expression is (%(exp)s)." % dict(
                      filter=self.__class__.__name__, exp=req_str))
