# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from nova import utils
from oslo_log import log as logging

from nova.ibm.ego import ego_constants

from powervc_nova.scheduler.filters import PowerVCBaseFilter
from powervc_nova.virt.ibmpowervm.common.constants import IBMI_OS_DISTRO
from powervc_nova import logcall

LOG = logging.getLogger(__name__)


class PowerVMIBMiMobilityFilter(PowerVCBaseFilter):
    """
    This EGO scheduler filter applies to IBM i virtual machines only and
    ensures that target hosts also support IBM i mobility operations.
    """

    def __init__(self):
        super(PowerVMIBMiMobilityFilter, self).__init__()
        self._set_filter_scope(
            [ego_constants.FILTER_WORKING_SCOPE_LIVE_MIGRATION,
             ego_constants.FILTER_WORKING_SCOPE_COLD_MIGRATION])

    @logcall
    def parse(self, filter_properties, all_host_attrs):
        if not self._allow_oper_for_powervm(filter_properties, all_host_attrs):
            LOG.debug("Ignoring %s..." % self.__class__.__name__)
            return

        request_spec = filter_properties.get('request_spec', {})
        instance_props = request_spec.get('instance_properties', {})

        system_metadata = utils.instance_sys_meta(instance_props)
        os_type = system_metadata.get('image_os_distro')

        if IBMI_OS_DISTRO == os_type:
            req_str = "$ibmi_lpar_mobility_capable == True"
            self._select(req_str)
            LOG.debug("After passing the %(filter)s, the resource requirement "
                      "expression is (%(exp)s)." % dict(
                          filter=self.__class__.__name__, exp=req_str))
        else:
            LOG.debug("Ignoring %s since the virtual machine is not IBM i." %
                      self.__class__.__name__)
