# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from oslo_log import log as logging

from nova.ibm.ego import ego_constants

from powervc_nova.scheduler.filters import PowerVCBaseFilter
from powervc_nova import logcall

LOG = logging.getLogger(__name__)


class PowerVMMigrationCountFilter(PowerVCBaseFilter):
    """
    This EGO scheduler filter ensures that the number of ongoing migrations
    doesn't exceed the max number of migrations supported by PowerVM.  It
    also checks that candidate destinations aren't currently serving as a
    source of another migration operation (i.e., a PowerVM restriction).
    """

    def __init__(self):
        super(PowerVMMigrationCountFilter, self).__init__()
        self._set_filter_scope(
            [ego_constants.FILTER_WORKING_SCOPE_LIVE_MIGRATION,
             ego_constants.FILTER_WORKING_SCOPE_COLD_MIGRATION])

    @logcall
    def parse(self, filter_properties, all_host_attrs):
        if not self._allow_oper_for_powervm(filter_properties, all_host_attrs):
            LOG.debug("Ignoring %s..." % self.__class__.__name__)
            return

        request_spec = filter_properties.get('request_spec', {})

        operation_type = request_spec.get('operation_type')
        is_live_mig = (operation_type ==
                       ego_constants.FILTER_WORKING_SCOPE_LIVE_MIGRATION)

        if is_live_mig:
            req_str = ("$active_migrations_in_progress "
                       "< $active_migrations_supported")
        else:
            req_str = ("$inactive_migrations_in_progress "
                       "< $inactive_migrations_supported")

        req_str = "($source_host_for_migration == False) && (%s)" % req_str
        self._select(req_str)
        LOG.debug("After passing the %(filter)s, the resource requirement "
                  "expression is (%(exp)s)." % dict(
                      filter=self.__class__.__name__, exp=req_str))
