# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from nova.compute import power_state, hv_type
from nova.ibm.ego.hapolicy import constants as prs_ha_const

# Power State information
NOSTATE = ''
RUNNING = 'Running'
STARTING = 'Starting'
SHUTDOWN = 'Not Activated'
MIGRATING_RUNNING = 'Migrating - Running'
MIGRATING_RUNNING_K2 = 'migrating running'
POWER_STATE = {
    NOSTATE: power_state.NOSTATE,
    RUNNING: power_state.RUNNING,
    SHUTDOWN: power_state.SHUTDOWN,
    STARTING: power_state.RUNNING,
    MIGRATING_RUNNING: power_state.RUNNING,
    MIGRATING_RUNNING_K2: power_state.RUNNING
}

LE_GUEST_CAPABLE = 'little_endian_guest_capable'
AIX_CAPABLE = 'aix_capable'
IBMI_CAPABLE = 'ibmi_capable'
LINUX_CAPABLE = 'linux_capable'
SHR_PROC_POOL_CAPABLE = 'shared_processor_pool_capable'

NOVALINK_PLATFORM = 'NovaLink'
HMC_PLATFORM = 'HMC'

HMC_VERSION = 'hmc_version'

# remote restart related stats
PRS_MAX_PARA_REBUILDS = prs_ha_const.PARALLEL_REBUILD
PRS_MAX_PARA_DEST_REBUILDS = prs_ha_const.PARALLEL_DEST_REBUILD
NUM_RR_IN_PROGRESS = 'remote_restarts_in_progress'
# below 2 values need to be cleared once compute service is back up or
# the host is in operating state.
# host level remote restart stats
# indicate whether host is dead and ready for remote restart
HOST_DEAD = 'host_dead'
# target host for host level remote restart
REMOTE_RESTART_TARGET_HOST = 'host_dead_target_host'

PROC_UNITS = 'proc_units'

COD_ENABLED = 'cod_enabled'
COD_INACTIVE_CORES = 'cod_inactive_cores'
COD_MOBILE_CORES = 'cod_mobile_cores'
COD_UNRETURNED_MOBILE_CORES = 'cod_unreturned_mobile_cores'
EPOOL_OUT_OF_COMPLIANCE = 'OutOfCompliance'
COD_MOBILE_CORES_RESOURCE_TYPE = 'cores'
COD_MOBILE_MEM_RESOURCE_TYPE = 'memory'

PLATFORMS_SUPPORTING_EPOOLS = (HMC_PLATFORM, NOVALINK_PLATFORM)

SOURCE_HOST_FOR_MIGRATION = 'source_host_for_migration'

#
# Only these values should be returned in the 'stats' value of the
# Host.get_host_stats table.
#
COMPUTE_STATS = [
    'active_migrations_in_progress',
    'active_migrations_supported',
    'prs:max_parallel_migrations',
    'compatibility_modes',
    'disk_available',
    'disk_total',
    'disk_used',
    'host_memory_free',
    'host_memory_reserved',
    'host_memory_total',
    'ibmi_lpar_mobility_capable',
    'ibmi_restrictedio_capable',
    LE_GUEST_CAPABLE,
    'inactive_migrations_in_progress',
    'inactive_migrations_supported',
    'lmb_size',
    'maintenance_mode_needed',
    'memory_mb_used',
    'num_vios_virtual_slots_in_use',
    'num_vios_virtual_slots_reserved',
    'preferred_active_migrations_supported',
    'preferred_inactive_migrations_supported',
    PROC_UNITS,
    'proc_units_reserved',
    'proc_units_used',
    SOURCE_HOST_FOR_MIGRATION,
    'vios_max_virtual_slots',
    'max_vcpus_per_aix_linux_lpar',
    'max_procs_per_aix_linux_lpar',
    COD_ENABLED,
    COD_INACTIVE_CORES,
    COD_MOBILE_CORES,
    COD_UNRETURNED_MOBILE_CORES,
    AIX_CAPABLE,
    IBMI_CAPABLE,
    LINUX_CAPABLE,
    PRS_MAX_PARA_REBUILDS,
    PRS_MAX_PARA_DEST_REBUILDS,
    NUM_RR_IN_PROGRESS,
    HOST_DEAD,
    REMOTE_RESTART_TARGET_HOST,
    SHR_PROC_POOL_CAPABLE
]

ACTIVE_LPAR_MOBILITY_CAPABLE = 'active_lpar_mobility_capable'
ACTIVE_MEMORY_EXPANSION_CAPABLE = 'active_memory_expansion_capable'
INACTIVE_LPAR_MOBILITY_CAPABLE = 'inactive_lpar_mobility_capable'
PVM_SIMP_REMOTE_RESTART_CAPABLE = \
    'powervm_lpar_simplified_remote_restart_capable'
IBMi_LPAR_MOBILITY_CAPABLE = 'ibmi_lpar_mobility_capable'
IBMi_RESTRICTED_CAPABLE = 'ibmi_restrictedio_capable'
CUSTOM_MAC_ADDR_CAPABLE = 'custom_mac_addr_capable'
HYPERVISOR_STATE = 'hypervisor_state'
MANAGED_BY = 'managed_by'


def enum(**enums):
    return type('Enum', (), enums)

# enum type returned by getSystemType
PowerSystemType = enum(POWER5="POWER5", POWER6="POWER6", POWER7="POWER7",
                       POWER8="POWER8")

POWER8_MODE = 'POWER8'
POWER7_MODE = 'POWER7'
POWER6_PLUS_MODE = 'POWER6_Plus'
POWER6_MODE = 'POWER6'
POWER5_MODE = 'POWER5'

# Power VM hypervisor info
IBM_POWERVM_HYPERVISOR_TYPE = hv_type.PHYP
# Tuple containing acceptable "PowerVM" hypervisor type values.
IBM_POWERVM_HYPERVISOR_TYPES = ('powervm', 'phyp')
IBM_POWERVM_HYPERVISOR_VERSION = 8

# The types of LPARS that are supported.
POWERVM_SUPPORTED_INSTANCES = [['ppc64', hv_type.PHYP, 'hvm']]

# The supported storage connectivity types
CONNECTION_TYPE_NPIV = 'npiv'       # fibre channel NPIV
CONNECTION_TYPE_SSP = 'ibm_ssp'     # shared storage pool vSCSI
CONNECTION_TYPE_VSCSI = "pv_vscsi"  # physical volume vSCSI
# Connectivity category to check for
CHK_BOOT = 'boot'
CHK_DATA = 'data'
CHK_ALL = 'all'

# IBM i os_distro and os_type
IBMI_OS_DISTRO = 'ibmi'
IBMI_OS_TYPE = 'OS400'

# list of hypervisor_state that will allow remote restart
RR_HYP_STATES = ['error', 'error - dump in progress', 'power off',
                 'no connection']

# default processor pool name
DEFAULT_PROC_POOL = 'DefaultPool'

# max concurrent remote restart support for 840 HMC
MAX_PARA_RR_SRC_840 = 50
MAX_PARA_RR_DEST_840 = 16

# MIGRATING_STATES are the HMC states which correspond to an active migration
# from schema/uom/inc/Enumerations.xsd
MIGRATING_STATES = ["migrating running", "migrating not active"]

# Fabric metadata keys
WWPN_SYSTEM_METADATA_KEY = 'npiv_adpt_wwpns'
FABRIC_STATE_METADATA_KEY = 'fabric_state'
FS_UNMAPPED = 'unmapped'
FS_MIGRATING = 'migrating'
FS_INST_MAPPED = 'inst_mapped'
