# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from nova import exception

from powervc_nova import _


COD_MOBILE_CORES_RESOURCE_TYPE = 'cores'
COD_MOBILE_MEM_RESOURCE_TYPE = 'memory'


class IBMPowerVMMigrationFailed(exception.NovaException):
    msg_fmt = _("The migration task failed. %(error)s")


class IBMPowerVMRemoteRestartFailed(exception.NovaException):
    msg_fmt = _("The remote restart task failed. %(error)s")


class IBMPowerVMMigrationInProgress(exception.NovaException):
    msg_fmt = _("Migration of %(lpar)s is already in progress.")


class IBMPowerVMRemoteRestartInProgress(exception.NovaException):
    msg_fmt = _("Remote restart of %(lpar)s is already in progress.")


class IBMPowerVMProgressUpdateError(exception.NovaException):
    msg_fmt = _("Unable to update progress for virtual machine '%(uuid)s'.")


class IBMPowerVMMediaRepOutOfSpace(exception.NovaException):
    msg_fmt = _("The media library is too small for the ISO image. The "
                "media library requires %(size)s MB of space but only "
                "%(free)s MB is available.")


class IBMPowerVMEmptyK2Entry(exception.NovaException):
    msg_fmt = _("The K2 entry is None.")


class IBMPowerVMLPARIsRunningDuringCapture(exception.NovaException):
    msg_fmt = _("Virtual machine '%(instance_name)s' is running during "
                "capture. Virtual machines must be stopped before they "
                "can be captured.")


class IBMPowerVMLPARInstanceNotFound(exception.NovaException):
    msg_fmt = _("Unable to find virtual machine '%(instance_name)s'.")


class IBMPowerVMInsufficientResources(exception.NovaException):
    msg_fmt = _("Insufficient resources")


class IBMPowerVMMemoryBelowMin(IBMPowerVMInsufficientResources):
    msg_fmt = _("The requested memory (%(mem_requested)d MB) is below "
                "the minimum required value (%(mem_min)d MB)")


class IBMPowerVMDiskBelowActual(IBMPowerVMInsufficientResources):
    msg_fmt = _("The requested disk (%(new_disk_size)d GB) is lesser than "
                "the existing disk (%(disk_size)d GB)")


class IBMPowerVMCPUsBelowMin(IBMPowerVMInsufficientResources):
    msg_fmt = _("The requested CPUs (%(cpus_requested)d) are below "
                "the minimum required value (%(cpus_min)d) ")


class IBMPowerVMProcUnitsBelowMin(IBMPowerVMInsufficientResources):
    msg_fmt = _("The requested processing units (%(units_requested)s) are "
                "below the minimum required value (%(units_min)s) ")


class IBMPowerVMInsufficientFreeMemory(IBMPowerVMInsufficientResources):
    msg_fmt = _("Insufficient free memory: (%(mem_requested)d MB "
                "requested, %(mem_avail)d MB free)")


class IBMPowerVMInsufficientCPU(IBMPowerVMInsufficientResources):
    msg_fmt = _("Insufficient available CPUs: (%(cpus_requested)d "
                "requested, %(cpus_avail)d available)")


class IBMPowerVMInsufficientProcUnits(IBMPowerVMInsufficientResources):
    msg_fmt = _("Insufficient available processing units: "
                "(%(units_requested)s requested, "
                "%(units_avail)s available)")


class IBMPowerVMJobRequestFailed(exception.NovaException):
    msg_fmt = _("The '%(operation_name)s' operation failed. %(error)s")


class IBMPowerVMJobRequestTimedOut(IBMPowerVMJobRequestFailed):
    msg_fmt = _("The '%(operation_name)s' operation failed. "
                "Failed to complete the task in %(seconds)s seconds.")


class IBMPowerVMLPARInstanceCleanupFailed(exception.NovaException):
    msg_fmt = _("Virtual machine '%(instance_name)s' cleanup failed. "
                "Reason: %(reason)s")


class IBMPowerVMInstanceTerminationFailure(exception.
                                           InstanceTerminationFailure):
    msg_fmt = _("Failed to delete virtual machine. %(reason)s")


class IBMPowerVMIBMiDiskResizeNotSupported(exception.NovaException):
    msg_fmt = _("Unable to change disk size for virtual machine "
                "%(instance_name)s because disk size change is not supported "
                "for IBM i.")


class IBMPowerVMInvalidImageObject(exception.NovaException):
    msg_fmt = _("Unable to save the captured image meta data for virtual "
                "machine %(instance_name)s because the image object is "
                "invalid.")


class IBMPowerVMInvalidCaptureVolume(exception.NovaException):
    msg_fmt = _("Unable to capture volume %(volume)s attached to "
                "%(instance_name)s because it is in the '%(state)s' state.")


class IBMPowerVMInvalidRootDevice(exception.NovaException):
    msg_fmt = _("The root device of %(instance_name)s is not a volume.")


class IBMPowerVMMissingRootBDM(exception.NovaException):
    msg_fmt = _("Instance %(instance_name)s is missing a root "
                "disk specification.")


class IBMPowerVMRootDeviceMismatch(exception.NovaException):
    msg_fmt = _("The root device %(instance_root)s for "
                "instance %(instance_name)s, does not match the root device"
                " name specified by the attached volume: %(bdm_root_dev)s.")


class IBMPowerVMVolumesNotAttached(exception.NovaException):
    msg_fmt = _("Unable to capture volumes %(volumes)s because they are not "
                "attached to instance %(instance_name)s.")


class IBMPowerVMImageVolumeCloneFailure(exception.NovaException):
    msg_fmt = _("Image volume creation failure for instance %(instance_name)s."
                " Volume error: %(volume_error)s. Volume health status: "
                "%(hstatus)s.")


class IBMPowerVCK2APIError(exception.NovaException):
    msg_fmt = _("An error occurred while working with objects returned "
                "from the K2 API.")


class IBMPowerVMNoOSForVM(exception.NovaException):
    msg_fmt = _("Virtual machine '%(instance_name)s' does not have an"
                " operating system identified.  The process for bringing"
                " the virtual machine under management by PowerVC must "
                "be completed.")


class IBMPowerVMUnsupportedOSForVM(exception.NovaException):
    msg_fmt = _("The operating system %(os_distro)s for virtual machine "
                "'%(instance_name)s' is not supported. Supported operating "
                "systems include the following: %(options)s")


class IBMPowerVMErrorExtendingVolume(exception.NovaException):
    msg_fmt = _("An error occurred when extending volume %(volume_id)s")


class IBMPowerVMNoLicenseSRC(exception.NovaException):

    def __init__(self, message=None, **kwargs):
        message = _("Unable to deploy virtual machine '%(instance_name)s' "
                    "because the host does not support the operating system"
                    " installed on this image") % kwargs
        super(IBMPowerVMNoLicenseSRC, self).__init__(message)


class IBMPowerVMUnSupportedOSVersion(exception.NovaException):

    def __init__(self, message=None, **kwargs):
        message = _("Unable to deploy virtual machine '%(instance_name)s' "
                    "because the host does not support the operating system"
                    "version installed on this image") % kwargs
        super(IBMPowerVMUnSupportedOSVersion, self).__init__(message)


class IBMPowerVMLUADiscoveryFailed(exception.NovaException):
    msg_fmt = _("Failed to discover the volume error code %(status)s")


class IBMPowerVMLUADiscoveryITLError(exception.NovaException):
    msg_fmt = _("LUA Discovery Completed with few ITL Error:"
                "Device %(dev)s Discovered, Status: %(status)s"
                "Reason: %(msg)s")


class IBMPowerVMAllocStgForMinVIOSFailed(exception.NovaException):
    msg_fmt = _("Unable to configure storage adapters for virtual machine "
                "%(instance_name)s on the required number of "
                "Virtual I/O Servers for the volume. "
                "Minimum number of Virtual I/O Servers required per volume: "
                "%(min_vios)s. %(reasons)s")


class IBMPowerVMHostInMultiplePools(exception.NovaException):
    msg_fmt = _("The host %(host)s was found to be a member of more than one "
                "Capacity on Demand enterprise pool. It should only be in at "
                "most one enterprise pool. It was found in the following "
                "pools: %(pools)s.")


class IBMPowerVMSetMobileResourcesFailed(exception.NovaException):
    msg_fmt = _('Failed to set the mobile %(resource_type)s on host %(host)s '
                'to %(num_mobile_resources)d.')


class IBMPowerVMInvalidInputMobileResourcesValue(exception.NovaException):
    msg_fmt = _('The desired mobile %(resource_type)s value '
                '(%(desired_mobile_resources)s) is not zero or a positive '
                'integer value.')


class IBMPowerVMHostNotEnoughResources(exception.NovaException):
    msg_fmt = _('The number of mobile %(resource_type)s to add '
                '(%(increase_amount)s) to host %(host)s exceeds the number of '
                'available %(resource_type)s (%(available)s) on the host and '
                'the number of unreturned %(resource_type)s (%(unreturned)s).')


class IBMPowerVMPoolNotEnoughMobileResources(exception.NovaException):
    msg_fmt = _('The number of mobile %(resource_type)s to add '
                '(%(increase_amount)s) to host %(host)s exceeds the number of '
                'available mobile %(resource_type)s (%(available)s) in the '
                'enterprise pool (%(pool_name)s, %(pool_uuid)s).')


class IBMPowerVMPoolNotCompliantTooManyResources(exception.NovaException):
    msg_fmt = _('The enterprise pool is out of compliance and the host '
                '%(host)s has %(unreturned_resources)d unreturned mobile '
                '%(resource_type)s. Therefore only up to '
                '%(unreturned_resources)d mobile %(resource_type)s can be '
                'assigned to the host.')
