# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from nova import exception
from powervc_nova import _


class IBMPowerVMHSCLError(exception.NovaException):
    msg_fmt = _("Encountered unexpected error from the HMC: "
                "%(err_message)s.")


class IBMPowerVMInvalidVIOConfig(exception.InvalidParameterValue):
    msg_fmt = _("The virtual adapter configuration is invalid.")


class IBMPowerVMAttrRequired(exception.NovaException):
    msg_fmt = _("%(attr)s is a required attribute for virtual machine "
                " '%(instance_name)s'.")


class IBMPowerVMAttrValueInvalid(exception.NovaException):
    msg_fmt = _("'%(value)s' is not a valid value for %(attr)s for "
                "virtual machine '%(instance_name)s'.")


class IBMPowerVMDesiredLTMin(exception.NovaException):
    msg_fmt = _("The value for minimum %(attr)s (%(value)s) cannot be greater"
                " than the desired %(attr)s (%(des_value)s) for virtual"
                " machine '%(instance_name)s'.")


class IBMPowerVMDesiredGTMax(exception.NovaException):
    msg_fmt = _("The value for maximum %(attr)s (%(value)s) cannot be less "
                " than the desired %(attr)s (%(des_value)s) for virtual"
                " machine '%(instance_name)s'.")


class IBMPowerVMBothMinMax(exception.NovaException):
    msg_fmt = _("If you specify a minimum or maximum value for %(attr)s, "
                "you must specify both minimum and maximum %(attr)s for "
                "virtual machine '%(instance_name)s'.")


class IBMPowerVMNotLMBMultiple(exception.NovaException):
    msg_fmt = _("%(attr)s (%(value)d MB) is not a multiple of the region "
                "memory size (%(lmb)d MB) for virtual machine "
                "'%(instance_name)s'. Specify a value that is a multiple "
                "of the region memory size.")


class IBMPowerVMInvalidAvailPriority(exception.NovaException):
    msg_fmt = _("'%(availability_priority)s' is not a valid value for"
                " availability priority for virtual machine "
                "'%(instance_name)s'. Specify a value between 0 and 255.")


class IBMPowerVMInvalidProcCompat(exception.NovaException):
    msg_fmt = _("'%(processor_compatibility)s' is not a valid option for"
                " processor compatibility for virtual machine"
                " '%(instance_name)s'. Valid options include the "
                "following: '%(valid_values)s'.")


class IBMPowerVMInvalidProcCompatIBMi(exception.NovaException):
    msg_fmt = _("Changing '%(processor_compatibility)s' "
                "is not supported because for an IBM i "
                "virtual machine '%(instance_name)s', its "
                "processor compatibility mode can only be "
                "set as 'default' on Power6 host.")


class IBMPowerVMInvalidBoolean(exception.NovaException):
    msg_fmt = _("'%(value)s' is not valid for virtual machine "
                "'%(instance_name)s'. %(option)s must either be "
                "true or false. ")


class IBMPowerVMInvalidExpFactor(exception.NovaException):
    msg_fmt = _("'%(value)s' is not valid for virtual machine "
                "'%(instance_name)s'. %(option)s must be greater than or "
                "equal to 1.0 and less than or equal to 10.0.")


class IBMPowerVMInvalidDedShareMode(exception.NovaException):
    msg_fmt = _("'%(ded_share_mode)s' is not a valid option for sharing mode"
                " for dedicated processors for virtual machine "
                "'%(instance_name)s'. Valid options include the following:"
                " '%(valid_values)s'.")


class IBMPowerVMInvalidAttrWithDed(exception.NovaException):
    msg_fmt = _("'%(option)s' is not a valid option for dedicated processor"
                " mode for virtual machine '%(instance_name)s'.")


class IBMPowerVMInvalidAttrWithShared(exception.NovaException):
    msg_fmt = _("'%(option)s' is not a valid option for shared processor"
                " mode for virtual machine '%(instance_name)s'.")


class IBMPowerVMInvalidAttrWithCapped(exception.NovaException):
    msg_fmt = _("Sharing weight is not a valid option for capped sharing"
                " mode for virtual machine '%(instance_name)s'.")


class IBMPowerVMInvalidSharedWeight(exception.NovaException):
    msg_fmt = _("'%(shared_weight)s' is not a valid value for shared"
                " processor weight for virtual machine '%(instance_name)s'."
                " Specify a value between 0 and 255.")


class IBMPowerVMMustSpecifyVCPURange(exception.NovaException):
    msg_fmt = _("If you specify a minimum and maximum value for processing "
                "units, you must also specify a minimum and maximum value for"
                " virtual CPUs for virtual machine '%(instance_name)s'.")


class IBMPowerVMMustSpecifyProcUnits(exception.NovaException):
    msg_fmt = _("If you specify a minimum and maximum value for processing "
                "units, you must also specify a value for desired processing"
                " units for virtual machine'%(instance_name)s'.")


class IBMPowerVMInvalidProcUnitsPerVCPU(exception.NovaException):
    msg_fmt = _("There must be a minimum of %(proc_units_per_vcpu)s processing"
                " units per virtual CPU. The requested %(level)s processing"
                " units were '%(proc_units)s' and the requested %(level)s"
                " virtual CPUs were '%(vcpus)d' for virtual machine"
                " '%(instance_name)s'.")


class IBMPowerVMProcUnitsGTVCPUs(exception.NovaException):
    msg_fmt = _("The number of processing units cannot be a larger value than"
                " the number of virtual CPUs. The requested %(level)s "
                "processing units were '%(proc_units)s' and the requested "
                "%(level)s virtual CPUs were '%(vcpus)d' for virtual machine"
                " '%(instance_name)s'.")


class IBMPowerVMVCPUsAboveMaxAllowed(exception.NovaException):
    msg_fmt = _("The desired processors (%(vcpus)d) cannot be above the"
                " maximum processors allowed per partition (%(max_allowed)d)"
                " for virtual machine '%(instance_name)s'.")


class IBMPowerVMMaxVCPUsAboveMaxAllowed(exception.NovaException):
    msg_fmt = _("The maximum processors (%(vcpus)d) cannot be above the"
                " maximum system capacity processor limit %(max_allowed)d)"
                " for virtual machine '%(instance_name)s'.")


class IBMPowerVMInsufficientCPU(exception.NovaException):
    msg_fmt = _("Insufficient CPUs available on host for "
                "virtual machine '%(instance_name)s' (%(cpus_requested)s "
                "requested and %(cpus_avail)s available)")


class IBMPowerVMNoRMCConnectivity(exception.NovaException):
    msg_fmt = _("Unable to resize virtual machine '%(instance_name)s'. "
                "Resizing an active virtual machine requires RMC "
                "connectivity.")


class IBMPowerVMVolumeAttachFailed(exception.NovaException):
    msg_fmt = _("Unable to attach storage %(backing_dev)s to virtual machine "
                "%(instance_name)s. %(reason)s")


class IBMPowerVMVolumeDetachFailed(exception.NovaException):
    msg_fmt = _("Unable to detach storage %(backing_dev)s from virtual"
                " machine %(instance_name)s. %(reason)s")


class IBMPowerVMVolumeLookupFailed(exception.NovaException):
    msg_fmt = _("Unable to determine that the volume with id %(uuid)s has"
                " been discovered or mapped on a Virtual I/O Server")


class IBMPowerVMVIOSNotFound(exception.NovaException):
    msg_fmt = _("Unable to locate the Virtual I/O Server for this operation")


class IBMPowerVMStorageTypeNotSupported(exception.NovaException):
    msg_fmt = _("The storage type specified '%(storage_type)s' is "
                "not supported for this operation")


class IBMPowerVMDesiredOutsideCurrentRange(exception.NovaException):
    msg_fmt = _("Desired %(attr)s requested (%(desired)s) is outside the"
                " currently assigned minimum (%(min)s) and maximum (%(max)s)"
                " values for virtual machine '%(instance_name)s'. Either "
                "specify a valid value for desired %(attr)s or power off the"
                " virtual machine before resizing.")


class IBMPowerVMProcPoolNameModNotAllowed(exception.NovaException):
    msg_fmt = _("Changing the specified shared processor pool for "
                "virtual machine %(instance_name)s is not supported."
                "The current processor pool is: %(curr_proc_pool)s. "
                "The requested processor pool is: %(proc_pool)s.")


class IBMPowerVMProcPoolNameNotFound(exception.NovaException):
    msg_fmt = _("Unable to locate the shared processor pool %(pool_name)s "
                "on the managed host %(host)s.")


class IBMPowerVMProcPoolIDChanged(exception.NovaException):
    msg_fmt = _("The shared processor pool ID was changed for shared "
                "processor pool %(proc_pool)s. The shared processor pool ID "
                "is changed from %(curr_proc_pool_id)d to %(proc_pool_id)d.")


class IBMPowerVMShrProcPoolNotSupported(exception.NovaException):
    msg_fmt = _("The managed host %(host)s does not support multiple "
                "shared processor pools.")


class IBMPowerVMShrProcPoolNotConfigured(exception.NovaException):
    msg_fmt = _("The shared processor pool %(pool_name)s is not configured. "
                "Configure the shared processor pool and retry the operation.")


class IBMPowerVMUnsupportedInDLPAR(exception.NovaException):
    msg_fmt = _("Changing %(attr)s is not supported while virtual machine"
                " '%(instance_name)s' is running. The virtual machine must"
                " be powered off first.")


class IBMPowerVMUpdateFailed(exception.NovaException):
    msg_fmt = _("Failed to update object properties for %(uuid)s because "
                "the object information is out of date.")


class IBMPowerVMSNoStorageConnectivityFound(exception.NovaException):
    msg_fmt = _("Unable to find a supported storage connection type in the"
                " list of storage connectivity groups: %(scg_conn_types)s")


class IBMPowerVMAdapterCreateFailed(exception.NovaException):
    msg_fmt = _("Unable to create virtual adapter for attaching storage "
                "to virtual machine %(instance_name)s. Reason: %(reason)s")


class IBMPowerVMInvalidStorageConnectivityInfo(exception.NovaException):
    msg_fmt = _("Unable to perform this operation with the storage "
                "connectivity group provided for virtual machine"
                " %(instance_name)s.")


class IBMPowerVMUpdateInfoIncomplete(exception.NovaException):
    msg_fmt = _("Failed to update object properties for %(uuid)s because "
                "the object information is incomplete.")


class IBMPowerVMNoHMCConsolesForHost(exception.NovaException):
    msg_fmt = _("There are no HMC consoles configured to manage host "
                "%(host)s.")


class IBMPowerVMHMCSessionHasLoggedOff(exception.NovaException):
    msg_fmt = _("The HMC session for host %(host)s has been logged off "
                "as part of the unmanage operation.")


class IBMPowerVMNoRMCConnectivityForStorage(exception.NovaException):
    msg_fmt = _("Unable to configure storage for virtual machine "
                "'%(instance_name)s'. Configuring storage for a virtual "
                "machine in Active state requires RMC connectivity.")


class IBMPowerVMInvalidExtraSpec(exception.NovaException):
    msg_fmt = _("Invalid attribute name '%(key)s' was passed in as part of "
                "the flavor extra specs for virtual machine "
                "'%(instance_name)s'.")


class IBMPowerVMHostBusy(exception.NovaException):
    msg_fmt = _("The operation cannot complete because the system is busy. "
                "Wait a few minutes and then try the operation again.")


class IBMPowerVMVGMissing(exception.NovaException):
    msg_fmt = _("Unable to find volume group "
                "'%(volume_group_name)s' on Virtual I/O Server '%(uuid)s'.")


class IBMPowerVMIncompleteHostStatistics(exception.NovaException):
    msg_fmt = _("Unable to retrieve host statistics for managed host "
                "%(host)s.")


class IBMPowerVMDelReqProfFailed(exception.NovaException):
    msg_fmt = _("Failed to modify the required virtual adapter properties in "
                "the profile for Virtual I/O Server %(uuid)s.")


class IBMPowerVMRmReqAdapter(exception.NovaException):
    msg_fmt = _("Unable to remove the virtual adapter from Virtual I/O Server "
                "%(uuid)s because it is a required adapter.")


class IBMPowerVMCreateAdapter(exception.NovaException):
    msg_fmt = _("Unable to create virtual adapters on Virtual I/O Server "
                "'%(uuid)s'. Run the Verify Environment tool. "
                "You can either click the Verify Environment button on the "
                "GUI Home page or type the powervc-validate command on the "
                "command line. Verify that a sufficient number of virtual "
                "resources are configured for the Virtual I/O Server. Then, "
                "try the operation again.")


class IBMPowerVIOSLicenseNotAccepted(exception.NovaException):
    msg_fmt = _("The license for Virtual I/O Server %(vios_name)s "
                "has not been accepted.")


class IBMPowerVIOSIsNotRunning(exception.NovaException):
    msg_fmt = _("Virtual I/O Server %(vios_name)s is not running. "
                "The current state is: %(vios_state)s")


class IBMPowerVIOSRMCNotAcitve(exception.NovaException):
    msg_fmt = _("Virtual I/O Server %(vios_name)s does not have an active "
                "Resource Monitoring Control state.")


class IBMPowerVIOSIsTooBusy(exception.NovaException):
    msg_fmt = _("Virtual I/O Server %(vios_name)s is currently "
                "too busy to process update operations.")


class IBMPowerVIOSMissingPhysicalPort(exception.NovaException):
    msg_fmt = _("Virtual I/O Server %(vios_name)s does not have any "
                " fibre channel ports listed in the storage connectivity "
                "group.")


class IBMPowerVMInsufficientVIOSForStg(exception.NovaException):
    msg_fmt = _("There are not enough Virtual I/O Servers with the necessary "
                "connectivity within the storage connectivity group for "
                "virtual machine '%(instance_name)s' volume configuration: "
                "(Requested: %(min_vios)s Available: %(avail_vios)s). "
                "%(reasons)s")


class IBMPowerVMAllocStgForDualFabricFailed(exception.NovaException):
    msg_fmt = _("Unable to configure storage adapters for virtual machine "
                "%(instance_name)s on available physical Fibre Channel ports "
                "that satisfy the NPIV switch fabric requirement of the "
                "storage connectivity group. %(reasons)s")


class IBMPowerMACNotSupported(exception.NovaException):
    msg_fmt = _("The virtual machine %(name)s cannot use the specified "
                "port because it would require MAC address %(mac)s to be set. "
                "The host does not support custom MAC addresses.")


class IBMPowerVMHostNotOperating(exception.NovaException):
    msg_fmt = _("The operation failed because it is only allowed when the "
                "managed host is in the Standby or Operating state.")


class IBMPowerVMOSRmMemFailed(exception.NovaException):
    msg_fmt = _("Unable to resize the virtual machine %(instance_name)s "
                "to the specified memory. This is likely because some of the "
                "memory to be removed is being used by the operating system. "
                "Try the resize again with a larger memory. To determine "
                "the amount of memory that operating system will not remove, "
                "see the nova compute log file %(file_name)s.")


class IBMPowerVMHostUnsupportedCapability(exception.NovaException):
    msg_fmt = _("The operation failed because the managed host does not"
                " support the %(attr)s.")


class IBMPowerVIOSError(exception.NovaException):
    msg_fmt = _("Error getting information from the Virtual I/O Servers. "
                "This could be because it is down or too busy to process "
                "the request.")


class IBMPowerVMCannotCloseVTerm(exception.NovaException):
    msg_fmt = _('The virtual machine %(name)s cannot be deleted because '
                'it has an open virtual terminal connection. Close the '
                'virtual terminal connection and try again. Disable Serial '
                'Over LAN (SOL) if the managed system is a Power Blade or '
                'Power ITE.')
