# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
from oslo_utils import encodeutils


def contains_msg_in_k2_err(k2err, msg, k2resp=None):
    """
    Check whether the K2Error contains the specified msg.
    :param k2err: K2Error
    :param msg: error message
    :param k2resp: k2response from K2Error. If k2respse is not None, we will
                   use k2resp to extract message instead.
    :return Boolean indicating whether the K2Error contains the message
    """
    k2e_resp = None
    if k2resp is not None:
        k2e_resp = k2resp
    elif hasattr(k2err, 'k2response'):
        k2e_resp = k2err.k2response
    if (k2e_resp is not None and k2e_resp.body is not None
            and msg in
            encodeutils.safe_encode(k2e_resp.body.__str__(),
                                    errors="ignore")):
        return True
    return False


def contains_msgs_in_k2_err(k2err, msgs, k2resp=None):
    """
    Check whether the K2Error contains the one of the messages.
    :param k2err: K2Error
    :param msg: error message
    :param k2resp: k2response from K2Error. If k2respse is not None, we will
                   use k2resp to extract message instead.
    :return Boolean indicating whether the K2Error contains the message
    """
    k2e_resp = None
    if k2resp is not None:
        k2e_resp = k2resp
    elif hasattr(k2err, 'k2response'):
        k2e_resp = k2err.k2response
    if (k2e_resp is not None and k2e_resp.body is not None
            and any(msg in encodeutils.safe_encode
                    (k2e_resp.body.__str__(),
                     errors="ignore")
                    for msg in msgs)):
        return True
    return False
