# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import re

from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper import EntryWrapper
from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper import DELIM
from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper import ROOT

from oslo_log import log as logging

LOG = logging.getLogger(__name__)

BASE_VERSION = ROOT + DELIM + 'BaseVersion'
VER_PATTERN = "v([0-9]+)r([0-9]+)\.([0-9]+)\.([0-9]+)"


class HMCVersion():
    def __init__(self, ver_str):
        self._base_ver_string = ver_str.lower() if ver_str is not None \
            else None
        # parse the ver_str based on VER_PATTERN
        if self._base_ver_string:
            match = re.search(VER_PATTERN, self._base_ver_string)
        else:
            match = None
        if match:
            self._version = int(match.group(1))
            self._release = int(match.group(2))
            self._maint = int(match.group(3))
            self._serv_pack = int(match.group(4))
        else:
            self._version = 0
            self._release = 0
            self._maint = 0
            self._serv_pack = 0

    def __eq__(self, other):
        if isinstance(other, HMCVersion):
            return self._base_ver_string == other._base_ver_string
        else:
            return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def __gt__(self, other):
        if self._version > other._version:
            return True
        elif self._version < other._version:
            return False
        # _version is the same
        if self._release > other._release:
            return True
        elif self._release < other._release:
            return False
        # _version and _release are the same
        if self._maint > other._maint:
            return True
        elif self._maint < other._maint:
            return False
        # _version, _release, maintenance are the same
        if self._serv_pack > other._serv_pack:
            return True
        return False

    def __ge__(self, other):
        return self.__eq__(other) or self.__gt__(other)

    def __lt__(self, other):
        return not (self.__eq__(other) or self.__gt__(other))

    def __le__(self, other):
        return self.__eq__(other) or self.__lt__(other)


class MgmtConsoleWrapper(EntryWrapper):
    """
    Wrapper object for k2 ManagementConsole schema
    """

    def get_hmc_base_version(self):
        return self.get_parm_value(BASE_VERSION)
