# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from powervc_nova import _

import powervc_nova.virt.ibmpowervm.common.constants as commonconsts
from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper import EntryWrapper
from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper \
    import add_deprecated_property_name
from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper import DELIM
from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper import ROOT

from oslo_log import log as logging

from nova.ibm.ego.hapolicy import constants as ha_constants

LOG = logging.getLogger(__name__)

# Managed System Constants
CAPABILITIES = ROOT + 'AssociatedSystemCapabilities'
MEM_CONFIG = ROOT + 'AssociatedSystemMemoryConfiguration'
PROC_CONFIG = ROOT + 'AssociatedSystemProcessorConfiguration'

ACTIVE_LPM_CAP = CAPABILITIES + DELIM + 'ActiveLogicalPartitionMobilityCapable'
AME_CAP = CAPABILITIES + DELIM + 'ActiveMemoryExpansionCapable'
CURR_LMB_SIZE = MEM_CONFIG + DELIM + 'CurrentLogicalMemoryBlockSize'
INACTIVE_LPM_CAP = CAPABILITIES + DELIM + \
    'InactiveLogicalPartitionMobilityCapable'
SIMP_REMOTE_RESTART_CAP = CAPABILITIES + DELIM + \
    'PowerVMLogicalPartitionSimplifiedRemoteRestartCapable'
VETH_MAC_ADDR_CAP = CAPABILITIES + DELIM + \
    'VirtualEthernetCustomMACAddressCapable'
IBMi_LPM_CAP = CAPABILITIES + DELIM + 'IBMiLogicalPartitionMobilityCapable'
IBMi_RESTRICTEDIO_CAP = CAPABILITIES + DELIM + 'IBMiRestrictedIOModeCapable'
AIX_CAP = CAPABILITIES + DELIM + 'AIXCapable'
IBMI_CAP = CAPABILITIES + DELIM + 'IBMiCapable'
LINUX_CAP = CAPABILITIES + DELIM + 'LinuxCapable'
SHR_PROC_POOL_CAP = CAPABILITIES + DELIM + 'SharedProcessorPoolCapable'

# Migration Constants
PROC_COMPAT_MODES = PROC_CONFIG + DELIM + \
    'SupportedPartitionProcessorCompatibilityModes'
MIGR_INFO = ROOT + 'SystemMigrationInformation'
MAX_ACTIVE_MIGR = MIGR_INFO + DELIM + 'MaximumActiveMigrations'
MAX_INACTIVE_MIGR = MIGR_INFO + DELIM + 'MaximumInactiveMigrations'
ACTIVE_MIGR_RUNNING = MIGR_INFO + DELIM + 'NumberOfActiveMigrationsInProgress'
INACTIVE_MIGR_RUNNING = MIGR_INFO + DELIM + \
    'NumberOfInactiveMigrationsInProgress'
MAX_FIRMWARE_MIGR = MIGR_INFO + DELIM + 'MaximumFirmwareActiveMigrations'

SYSTEM_NAME = ROOT + 'SystemName'
STATE = 'State'

PROC_UNITS_INSTALLED = PROC_CONFIG + DELIM + \
    'InstalledSystemProcessorUnits'

PROC_UNITS_AVAIL = PROC_CONFIG + DELIM + \
    'CurrentAvailableSystemProcessorUnits'

PROC_UNITS_CONFIGURABLE = PROC_CONFIG + DELIM + \
    'ConfigurableSystemProcessorUnits'

MAX_PROCS_PER_PARTITION = PROC_CONFIG + DELIM + \
    'CurrentMaximumAllowedProcessorsPerPartition'

MAX_PROCS_PER_PARTITION_1339A_0929 = PROC_CONFIG + DELIM + \
    'CurrentMaximumProcessorsPerPartition'  # Deprecated in 1339A_0929_1701Z

add_deprecated_property_name(MAX_PROCS_PER_PARTITION,
                             MAX_PROCS_PER_PARTITION_1339A_0929)

MAX_PROCS_PER_AIX_LINUX_PARTITION = PROC_CONFIG + DELIM + \
    'CurrentMaximumProcessorsPerAIXOrLinuxPartition'

MAX_VCPUS_PER_PARTITION = PROC_CONFIG + DELIM + \
    'MaximumAllowedVirtualProcessorsPerPartition'

MAX_VCPUS_PER_PARTITION_1339A_0929 = PROC_CONFIG + DELIM + \
    'MaximumVirtualProcessorsPerPartition'  # Deprecated in 1339A_0929_1701Z

add_deprecated_property_name(MAX_VCPUS_PER_PARTITION,
                             MAX_VCPUS_PER_PARTITION_1339A_0929)

MAX_VCPUS_PER_AIX_LINUX_PARTITION = PROC_CONFIG + DELIM + \
    'CurrentMaximumVirtualProcessorsPerAIXOrLinuxPartition'

MEMORY_INSTALLED = MEM_CONFIG + DELIM + \
    'InstalledSystemMemory'

MEMORY_AVAIL = MEM_CONFIG + DELIM + \
    'CurrentAvailableSystemMemory'

MEMORY_CONFIGURABLE = MEM_CONFIG + DELIM + \
    'ConfigurableSystemMemory'

MEMORY_REGION_SIZE = MEM_CONFIG + DELIM + \
    'MemoryRegionSize'

SYS_FIRMWARE_MEM = MEM_CONFIG + DELIM + \
    'MemoryUsedByHypervisor'

HOST_IP_ADDRESS = ROOT + 'PrimaryIPAddress'

# VIOS
VIOS_LINK = ROOT + "AssociatedVirtualIOServers" + DELIM + "link"


class ManagedSystemEntryWrapper(EntryWrapper):

    def get_system_name(self):
        new_value = self.get_parm_value(SYSTEM_NAME)
        if not hasattr(self, '_hostname'):
            self._hostname = new_value
        elif new_value != self._hostname:
            LOG.error(_('Hostname has changed from %(old)s to %(new)s. '
                        'A restart is required for the change to take effect.')
                      % {'old': self._hostname, 'new': new_value})
        return self._hostname

    def get_system_state(self):
        return self.get_parm_value(STATE, 'unknown')

    def get_proc_units(self):
        k2_property = PROC_UNITS_INSTALLED
        k2_value = self.get_parm_value(k2_property, 0)

        return k2_value

    def get_proc_units_configurable(self):
        k2_property = PROC_UNITS_CONFIGURABLE
        k2_value = self.get_parm_value(k2_property, 0)

        return k2_value

    def get_proc_units_avail(self):
        k2_property = PROC_UNITS_AVAIL
        k2_value = self.get_parm_value(k2_property, 0)

        return k2_value

    def get_max_sys_procs_limit(self):
        k2_property = MAX_PROCS_PER_PARTITION
        k2_value = self.get_parm_value_int(k2_property, 0)

        return k2_value

    def get_max_procs_per_aix_linux_lpar(self):
        k2_property = MAX_PROCS_PER_AIX_LINUX_PARTITION
        k2_value = self.get_parm_value_int(k2_property, 0)
        # Some systems will not have maximum procs per lpar based on
        # partition type. In that case, use system max procs per partition.
        if k2_value == 0:
            k2_value = self.get_max_sys_procs_limit()

        return k2_value

    def get_max_sys_vcpus_limit(self):
        k2_property = MAX_VCPUS_PER_PARTITION
        k2_value = self.get_parm_value_int(k2_property, 0)

        return k2_value

    def get_max_vcpus_per_aix_linux_lpar(self):
        k2_property = MAX_VCPUS_PER_AIX_LINUX_PARTITION
        k2_value = self.get_parm_value_int(k2_property, 0)
        # Some systems will not have maximum vcpus per lpar based on
        # partition type. In that case, use system max vcpus per partition.
        if k2_value == 0:
            k2_value = self.get_max_sys_vcpus_limit()

        return k2_value

    def get_memory_total(self):
        k2_property = MEMORY_INSTALLED
        k2_value = self.get_parm_value_int(k2_property, 0)

        return k2_value

    def get_memory_free(self):
        k2_property = MEMORY_AVAIL
        k2_value = self.get_parm_value_int(k2_property, 0)

        return k2_value

    def get_memory_configurable(self):
        k2_property = MEMORY_CONFIGURABLE
        k2_value = self.get_parm_value_int(k2_property, 0)

        return k2_value

    def get_memory_region_size(self):
        k2_property = MEMORY_REGION_SIZE
        k2_value = self.get_parm_value_int(k2_property, 0)

        return k2_value

    def get_firmware_memory(self):
        k2_property = SYS_FIRMWARE_MEM
        k2_value = self.get_parm_value_int(k2_property, 0)

        return k2_value

    def get_host_ip_address(self):
        k2_property = HOST_IP_ADDRESS
        k2_value = self.get_parm_value(k2_property)

        return k2_value

    def get_aix_capable(self):
        cap = self.get_parm_value(AIX_CAP)
        # we can get 'unavailable' if PHYP interface is running an older
        # level and doesn't support query of this information
        if cap is not None and cap.lower() == 'inactive':
            return False
        # Default to true since that's what we always
        # assume prior to this capability
        return True

    def get_capabilities(self):
        """
        returns: The system capabilities from K2
        """
        # VirtualEthernetCustomMACAddressCapable (custom_mac_addr_capable) will
        # default to True, which is the correct setting for POWER7 servers.
        cap_data = {'active_lpar_mobility_capable':
                    self.get_parm_value_bool(ACTIVE_LPM_CAP),
                    'active_memory_expansion_capable':
                    self.get_parm_value_bool(AME_CAP),
                    'inactive_lpar_mobility_capable':
                    self.get_parm_value_bool(INACTIVE_LPM_CAP),
                    'powervm_lpar_simplified_remote_restart_capable':
                    self.get_parm_value_bool(SIMP_REMOTE_RESTART_CAP),
                    'ibmi_lpar_mobility_capable':
                    self.get_parm_value_bool(IBMi_LPM_CAP, False),
                    'custom_mac_addr_capable':
                    self.get_parm_value_bool(VETH_MAC_ADDR_CAP, True),
                    'ibmi_restrictedio_capable':
                    self.get_parm_value_bool(IBMi_RESTRICTEDIO_CAP, False),
                    commonconsts.AIX_CAPABLE:
                    self.get_aix_capable(),
                    commonconsts.IBMI_CAPABLE:
                    self.get_parm_value_bool(IBMI_CAP, True),
                    commonconsts.LINUX_CAPABLE:
                    self.get_parm_value_bool(LINUX_CAP, True),
                    commonconsts.SHR_PROC_POOL_CAPABLE:
                    self.get_parm_value_bool(SHR_PROC_POOL_CAP, False),
                    }
        return cap_data

    def get_proc_compat_modes(self):
        """
        returns: A list of strings containing the processor
                 compatibility modes
        """
        modes = self.get_parm_values(PROC_COMPAT_MODES)
        # Format the proc compat modes from K2 strings to PowerVC strings
        modes = [mode.replace('_Plus', '+') for mode in modes]
        return modes

    def get_migration_data(self):
        """
        returns: The migration properties from K2
        """

        max_migr_sup = self.get_parm_value_int(MAX_FIRMWARE_MIGR)
        act_migr_sup = self.get_parm_value_int(MAX_ACTIVE_MIGR)
        inact_migr_sup = self.get_parm_value_int(MAX_INACTIVE_MIGR)
        pref_act_migr_sup = act_migr_sup
        pref_inact_migr_sup = inact_migr_sup
        act_migr_prog = self.get_parm_value_int(ACTIVE_MIGR_RUNNING)
        inact_migr_prog = self.get_parm_value_int(INACTIVE_MIGR_RUNNING)

        migr_data = {'max_migration_ops_supported': max_migr_sup,
                     'active_migrations_supported': act_migr_sup,
                     'inactive_migrations_supported': inact_migr_sup,
                     'preferred_active_migrations_supported':
                     pref_act_migr_sup,
                     'preferred_inactive_migrations_supported':
                     pref_inact_migr_sup,
                     'active_migrations_in_progress': act_migr_prog,
                     'inactive_migrations_in_progress': inact_migr_prog,
                     ha_constants.PARALLEL_MIGRATE: pref_act_migr_sup
                     }
        return migr_data

    def get_vios_links(self):
        '''
        This method traverses the AssociatedVirtualIOServers list and returns
        all the hrefs found therein.
        '''
        ret_links = []
        vios_links = self._k2entry.element.findall(VIOS_LINK)

        # If we found some VIOSes
        if vios_links:
            for link in vios_links:
                ret_links.append(link.getattrib()['href'])

        return ret_links
