# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper import EntryWrapper
from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper import DELIM
from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper import ROOT
from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper \
    import hmc_sharing_mode_to_nova

from oslo_log import log as logging

LOG = logging.getLogger(__name__)


MEM_CONFIG = 'ProfileMemory'
PROC_CONFIG = 'ProcessorAttributes'
PROC_COMP_MODE = 'DesiredProcessorCompatibilityMode'

USE_DED_PROCS = PROC_CONFIG + DELIM + 'HasDedicatedProcessors'
DED_PROC_CONFIG = PROC_CONFIG + DELIM + 'DedicatedProcessorConfiguration'
SHARED_PROC_CONFIG = PROC_CONFIG + DELIM + 'SharedProcessorConfiguration'
SHARING_MODE = PROC_CONFIG + DELIM + 'SharingMode'

DESIRED_PROCS = DED_PROC_CONFIG + DELIM + 'DesiredProcessors'
MAX_PROCS = DED_PROC_CONFIG + DELIM + 'MaximumProcessors'
MIN_PROCS = DED_PROC_CONFIG + DELIM + 'MinimumProcessors'

DESIRED_VCPU = SHARED_PROC_CONFIG + DELIM + 'DesiredVirtualProcessors'
MAX_VCPU = SHARED_PROC_CONFIG + DELIM + 'MaximumVirtualProcessors'
MIN_VCPU = SHARED_PROC_CONFIG + DELIM + 'MinimumVirtualProcessors'
DESIRED_PROC_UNITS = SHARED_PROC_CONFIG + DELIM + 'DesiredProcessingUnits'
MAX_PROC_UNITS = SHARED_PROC_CONFIG + DELIM + 'MaximumProcessingUnits'
MIN_PROC_UNITS = SHARED_PROC_CONFIG + DELIM + 'MinimumProcessingUnits'
UNCAPPED_WEIGHT = SHARED_PROC_CONFIG + DELIM + 'UncappedWeight'

DESIRED_MEM = MEM_CONFIG + DELIM + 'DesiredMemory'
MAX_MEM = MEM_CONFIG + DELIM + 'MaximumMemory'
MIN_MEM = MEM_CONFIG + DELIM + 'MinimumMemory'

DEFAULT_SHARING_MODE = 'share_idle_procs'
DEFAULT_PROC_COMP_MODE = 'default'
FALSE = 'false'
ZERO = '0'

VIO_ADAPTERS = ('IOConfigurationInstance' + DELIM +
                'ProfileVirtualIOAdapters' + DELIM +
                'ProfileVirtualIOAdapterSubclass')

PROF_VSCSI_CLIENT = 'ProfileVirtualSCSIClientAdapter'
PROF_VSCSI_SERVER = 'ProfileVirtualSCSIServerAdapter'
PROF_VFC_CLIENT = 'ProfileVirtualFibreChannelClientAdapter'
PROF_VFC_SERVER = 'ProfileVirtualFibreChannelServerAdapter'
REMOTE_LPAR_ID = 'RemotePartitionID'
CONN_LPAR_ID = 'ConnectingPartitionID'
LOCAL_LPAR_ID = 'LocalPartitionID'
IS_REQUIRED = 'IsRequired'


class LPARProfileEntryWrapper(EntryWrapper):
    """
    Wrapper object for k2 LogicalPartitionProfile schema
    """

    def get_mem(self):
        k2_property = ROOT + DESIRED_MEM
        return self.get_parm_value(k2_property, ZERO)

    def get_max_mem(self):
        k2_property = ROOT + MAX_MEM
        return self.get_parm_value(k2_property, ZERO)

    def get_min_mem(self):
        k2_property = ROOT + MIN_MEM
        return self.get_parm_value(k2_property, ZERO)

    def get_proc_compatibility_mode(self):
        k2_property = ROOT + PROC_COMP_MODE
        return self.get_parm_value(k2_property, DEFAULT_PROC_COMP_MODE)

    def get_sharing_mode(self):
        k2_property = ROOT + SHARING_MODE
        mode = self.get_parm_value(k2_property, DEFAULT_SHARING_MODE)
        return hmc_sharing_mode_to_nova.get(mode)

    def get_proc_mode(self):
        k2_property = ROOT + USE_DED_PROCS
        return self.get_parm_value(k2_property, FALSE).lower()

    def get_procs(self):
        k2_property = ROOT + DESIRED_PROCS
        return self.get_parm_value(k2_property, ZERO)

    def get_max_procs(self):
        k2_property = ROOT + MAX_PROCS
        return self.get_parm_value(k2_property, ZERO)

    def get_min_procs(self):
        k2_property = ROOT + MIN_PROCS
        return self.get_parm_value(k2_property, ZERO)

    def get_vcpus(self):
        k2_property = ROOT + DESIRED_VCPU
        return self.get_parm_value(k2_property, ZERO)

    def get_max_vcpus(self):
        k2_property = ROOT + MAX_VCPU
        return self.get_parm_value(k2_property, ZERO)

    def get_min_vcpus(self):
        k2_property = ROOT + MIN_VCPU
        return self.get_parm_value(k2_property, ZERO)

    def get_proc_units(self):
        k2_property = ROOT + DESIRED_PROC_UNITS
        return self.get_parm_value(k2_property, ZERO)

    def get_max_proc_units(self):
        k2_property = ROOT + MAX_PROC_UNITS
        return self.get_parm_value(k2_property, ZERO)

    def get_min_proc_units(self):
        k2_property = ROOT + MIN_PROC_UNITS
        return self.get_parm_value(k2_property, ZERO)

    def get_uncapped_weight(self):
        k2_property = ROOT + UNCAPPED_WEIGHT
        return self.get_parm_value(k2_property, ZERO)

    def undo_req_vfc_vscsi_slots(self, lpar_id=None):
        """
        Check to see whether there are any vfc or vscsi slots marked as
        required. If so, remove the required settings for a given lpar_id.
        :param lpar_id: If not specific, all required setting will be unset.
                        Otherwise, it will only unset vslots with
                        specific lpar_id as the remote or connecting lpar id.
        :return Boolean indicating whether _k2entry has been modified.
        """
        # find the virtual io section
        k2_property = ROOT + VIO_ADAPTERS
        vio_subclass_list = self._k2entry.element.findall(k2_property)
        mod = False
        if vio_subclass_list:
            for vio in vio_subclass_list:
                for elem in vio.iter():
                    tag = elem.gettag()
                    # only inspect either ProfileVirtualSCSIClientAdapter,
                    # ProfileVirtualSCSIServerAdapter,
                    # ProfileVirtualFibreChannelClientAdapter, or
                    # ProfileVirtualFibreChannelServerAdapter
                    if tag in (PROF_VSCSI_CLIENT, PROF_VSCSI_SERVER,
                               PROF_VFC_CLIENT, PROF_VFC_SERVER):
                        # whether we found a vio matching to lpar_id
                        found_match = False
                        # extract the Local and remote partition id
                        if lpar_id is not None:
                            lpar_id_int = int(lpar_id)
                            remote_lpar_id = elem.findtext(REMOTE_LPAR_ID)
                            if not remote_lpar_id:
                                remote_lpar_id = elem.findtext(CONN_LPAR_ID)
                            local_lpar_id = elem.findtext(LOCAL_LPAR_ID)
                            if ((remote_lpar_id and
                                 int(remote_lpar_id) == lpar_id_int) or
                                (local_lpar_id and
                                 int(local_lpar_id) == lpar_id_int)):
                                found_match = True
                        else:
                            found_match = True
                        if found_match:
                            # check whether it's required or not
                            is_req = elem.find(ROOT + IS_REQUIRED)
                            if (is_req is not None and
                                    is_req.gettext() == 'true'):
                                is_req.settext('false')
                                mod = True
        return mod
