# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from oslo_log import log as logging
from powervc_oslo.k2.k2operator import K2ConnectionError
from powervc_oslo.k2.k2operator import K2TimeoutError
from powervc_oslo.k2.k2operator import K2SSLError
from powervc_nova import _

LOG = logging.getLogger(__name__)


class K2OperGuard(object):

    def __init__(self, host=None, hmc=None):
        if hmc is not None:
            self._hmc = hmc
        else:
            if host is None:
                raise Exception("This is a code error.")
            # Now we have a host but not a specific HMC
            self._host = host
            self._hmc = None

    def __enter__(self):
        # Assign an HMC if one was not already selected
        if self._hmc is None:
            self._hmc = self._host.select_active_hmc()
            # TODO: what if no hmcs are active?  Wait?

        if self._hmc is not None:
            self._opr = self._hmc.get_k2oper()
            return self._opr
        else:
            # This is an error
            # TODO:  This could be because no HMCs were available
            return None

    def __exit__(self, type, value, traceback):
        # Following K2Errors indicate communication problems with HMC
        if (isinstance(value, K2ConnectionError) or
                isinstance(value, K2TimeoutError) or
                isinstance(value, K2SSLError)):
            LOG.error(_('Communication problem encountered %(type)s '
                        'with HMC %(hmc)s.') %
                      {'type': type, 'hmc': self._hmc.ipaddr})
            # TODO: Failover the back up HMC
