# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import copy

from nova_powervm.virt.powervm import live_migration as lpm
from oslo_log import log as logging
from oslo_serialization import jsonutils
from powervc_nova.objects import migrate_data as mig_obj
from pypowervm.tasks import migration as mig

LOG = logging.getLogger(__name__)


class PowerVCLiveMigrationDest(lpm.LiveMigrationDest):
    def __init__(self, drvr, instance):
        super(lpm.LiveMigrationDest, self).__init__(
            drvr, instance, mig_obj.PowerVCLiveMigrateData())


class PowerVCLiveMigrationSrc(lpm.LiveMigrationSrc):
    def __init__(self, drvr, instance, dest_data):
        super(PowerVCLiveMigrationSrc, self).__init__(
            drvr, instance, dest_data)

    def live_migration(self, context, migrate_data):
        """Start the live migration.

        :param context: security context
        :param migrate_data: a PowerVMLiveMigrateData object
        """
        LOG.debug("Starting migration.", instance=self.instance)
        logged_mig_data = copy.deepcopy(migrate_data)
        logged_mig_data.dest_cred = '********'
        LOG.debug("Migrate data: %s" % logged_mig_data)

        # The passed in mig data has more info (dest data added), so replace
        self.mig_data = migrate_data
        # Get the vFC and vSCSI live migration mappings
        vol_data = migrate_data.vol_data
        vfc_mappings = vol_data.get('vfc_lpm_mappings')
        if vfc_mappings is not None:
            vfc_mappings = jsonutils.loads(vfc_mappings)
        vscsi_mappings = vol_data.get('vscsi_lpm_mappings')
        if vscsi_mappings is not None:
            vscsi_mappings = jsonutils.loads(vscsi_mappings)

        dest_spp_id = None
        if hasattr(migrate_data, 'dest_spp_id'):
            # For default shared processor pool case,
            # the shared processor pool id passed to migrate_lpar
            # should be None, as opposed to '0'.
            # spp_id, if not None, must be an integer
            # greater than or equal to 1 with neo-rest.
            if migrate_data.dest_spp_id != '0':
                dest_spp_id = migrate_data.dest_spp_id

        try:
            # Migrate the LPAR!
            mig.migrate_lpar(self.lpar_w, self.mig_data.dest_sys_name,
                             validate_only=False,
                             tgt_mgmt_svr=self.mig_data.dest_ip,
                             tgt_mgmt_usr=self.mig_data.dest_user_id,
                             virtual_fc_mappings=vfc_mappings,
                             virtual_scsi_mappings=vscsi_mappings,
                             spp_id=dest_spp_id)

        except Exception:
            LOG.error(_("Live migration failed."), instance=self.instance)
            raise
        finally:
            LOG.debug("Finished migration.", instance=self.instance)
