# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
from oslo_config import cfg
from powervc_nova.network.powerkvm.agent.commandlet import CommandExecutor \
    as commands
from powervc_nova.network.powerkvm import ovs_tap_cleanup

pydevd_opts = [
    cfg.StrOpt('pydevd_remote_debug',
               default='',
               help='Turns on the pydev remote debugger'),
    cfg.StrOpt('pydevd_remote_debug_host',
               default='',
               help='Remote debug server'),
    cfg.IntOpt('pydevd_remote_debug_port',
               default=5678,
               help='Remote debug server port')
]

powerkvm_opts = [
    cfg.StrOpt('hw_video_model',
               default='vga',
               help='video model should be vga on power'),
    cfg.BoolOpt('serial_console_log_enabled',
                default=False,
                help='flag to configure whether to redirect in file or not'),
    cfg.BoolOpt('enable_kvm_ovs_auto_config',
                default=True,
                help='config default bridge automatically on the first boot'),
    cfg.BoolOpt('open_libvirt_tls_firewall_port',
                default=True,
                help='Opens the libvirt-tls port if closed to allow '
                'migration'),
    cfg.IntOpt('number_of_exceptions_needed_for_missing_inst',
               default=6,
               help='Number of instance not found exceptions must be received'
               'before reporting the instance as missing'),
    cfg.IntOpt('prs_max_parallel_rebuilds',
               default=10,
               help='Integer value that indicates the max number of '
               'concurrent remote restarts that will occur for a given '
               'source host. Required stat that applies to PowerVM '
               'and PowerKVM.'),
    cfg.IntOpt('prs_max_parallel_dest_rebuilds',
               default=10,
               help='Integer value that indicates the max number of '
               'concurrent remote restarts that will occur for a given '
               'target host. Required stat that applies to PowerVM '
               'and PowerKVM.'),
]

CONF = cfg.CONF
CONF.register_opts(pydevd_opts)
CONF.register_opts(powerkvm_opts)


def update_hw_inv():
    ovs_tap_cleanup.cleanup_ovs_tap_ports()
    commands.set_lshw_results(commands.send_lshw_command())
