# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from nova import exception
from powervc_nova import _


class PowerKVMInvalidSMT(exception.NovaException):
    msg_fmt = _("The value %(smt)s is not a valid value for SMT. If you "
                "would like to enable SMT on virtual machine "
                "'%(instance_name)s', please specify one of the "
                "following values: %(values)s.")


class PowerKVMVCPUSNotValidWithSMT(exception.NovaException):
    msg_fmt = _("The value for vcpus is not a multiple of the value for SMT "
                "for instance '%(instance_name)s'. "
                "(vcpus=%(vcpus)d, SMT=%(smt)d)")


class PowerKVMSMTGreaterThanMax(exception.NovaException):
    msg_fmt = _("The value %(smt)d is not a supported value for SMT on "
                "the host for instance '%(instance_name)s'. SMT must "
                "be less than or equal to the host's max_smt_per_guest "
                "value of %(max_smt)d.")


class PowerKVMCoresGreaterThanMax(exception.NovaException):
    msg_fmt = _("The value %(cores)d is not a supported value for cores on "
                "the host for instance '%(instance_name)s'. Cores must "
                "be less than or equal to %(max_cores)d.")


class PowerKVMInvalidCoreValue(exception.NovaException):
    msg_fmt = _("The value %(cores)s is an invalid value for instance "
                "'%(instance_name)s'. Cores must be an integer")


class PowerKVMSocketsGreaterThanMax(exception.NovaException):
    msg_fmt = _("The value %(sockets)d is not a supported value for socket on "
                "the host for instance '%(instance_name)s'. Sockets must "
                "be less than or equal to %(max_sockets)d.")


class PowerKVMInvalidSocketValue(exception.NovaException):
    msg_fmt = _("The value %(sockets)s is an invalid value for instance "
                "'%(instance_name)s'. Sockets must be an integer")


class PowerKVMInValidCPUConfiguration(exception.NovaException):
    msg_fmt = _("The value %(sockets)d %(cores)d %(threads)d is not valid "
                "value for sockets cores and threads for vcpu %(vcpus)d "
                " for instance %(instance_name)s")


class PowerKVMInValidThreadConfiguration(exception.NovaException):
    msg_fmt = _("The value %(threads)d is not valid value for threads "
                "for instance '%(instance_name)s'. Threads must "
                "be less than %(max_threads)d and power of 2.")


class PowerKVMInvalidThreadValue(exception.NovaException):
    msg_fmt = _("The value %(threads)s is an invalid value for instance "
                "'%(instance_name)s'. Threads must be an integer")


class PowerKVMBaseDiskFileDoesnotExist(exception.NovaException):
    msg_fmt = _("The root disk for %(inst)s is not accessible. "
                "Please check for the existence and permissions of "
                "%(disk_path)s on KVM node.")


class PowerKVMVifCallbackFailed(exception.NovaException):
    msg_fmt = _("PowerKVM Virtual Interface Creation Failed")


class PowerKVMMigrationFailed(exception.NovaException):
    msg_fmt = _("The migration task failed. %(error)s")


class PowerKVMOnboardPrepFailed(exception.NovaException):
    msg_fmt = _("Preparation was attempted on virtual machine "
                "%(name)s, but it was determined that this "
                "virtual machine is not a candidate for management "
                "for the following reason(s): %(reasons)s")


class PowerKVMOnboardDiskPrepFailed(exception.NovaException):
    msg_fmt = _("The virtual machine preparation for disk "
                "'%(path)s' failed.")
