#
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================


class ImportVMNetwork(object):
    """
    This class is an 'interface' that will define how a compute nodes
    Virtual Machines should be imported.  There will be multiple
    implementations for each type of managed host.
    """
    def list_ports(self, vm_id_or_name):
        """
        Will query the endpoint and determine the network configuration of the
        VM.

        :param vm_id_or_name: Depending on the implementation, this parameter
            will be either the id of the vm or the name of the vm
        :returns: A list of dictionary objects which represents the networking
                  information.  Each dictionary will contain a 'mac_address'
                  and a 'provider:segmentation_id' - which represents the VLAN
                  along with other properties.
                  NOTE: A segmentation_id of -1 indicates a flat network.
        """

        raise NotImplementedError()

    def interrogate_vm(self, vm_id_or_name):
        """
        Will perform validation of the VM/LPAR to determine whether or not it
        can be imported.

        :param vm_id_or_name: Depending on the implementation, this parameter
            will be either the id of the vm or the name of the vm
        :returns: 1.A list of network prep steps including interfaces to move
                    This will be empty for PowerVM
                  2.A list of reasons why the VM/LPAR may not be imported.
        """
        raise NotImplementedError()
