# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import sys
import signal
from powervc_oslo import _
from powervc_oslo import enable_command_translation
from powervc_oslo import get_parser_class
from powervc_oslo.config import data_mgr as mgr
from powervc_oslo.config import data_utils
from powervc_oslo.config import general_config as gconfig


def _signal_handler(signal, frame):
    """Prints a single line to the console when a signal interrupt such as
     ctrl + c is pressed instead of dumping the exception to the console.
     :param signal: the signal interrupt number
     :param frame: the stack frame"""
    print('')
    sys.exit(0)


def remove_console_exceptions():
    """Block exceptions from being printed to the console when a signal
    interrupt is triggered."""
    signal.signal(signal.SIGINT, _signal_handler)


def _parse_arguments():
    """Parses the arguments from the command line input."""
    _ParserClass = get_parser_class()

    parser = _ParserClass(description=_('Manage PowerVC configuration '
                                        'settings'),
                          epilog=_('Run "powervc-config <sub-command> -h" '
                                   'for help on a specific sub-command.'))
    subparser = parser.add_subparsers(title=_('powervc-config sub-commands'),
                                      help=_('List of sub-commands available'
                                             ' under powervc-config'))
    gconfig.setup_parser(subparser)
    mgrs = mgr.get_registered_managers()
    mgr._multi_manager_execute(mgrs, 'invoke_config_parser', subparser)
    return parser.parse_args()


def parse_and_execute_arguments():
    """Using the parsed arguments from the command line, execute the functions
     associated with that subparser"""
    args = _parse_arguments()
    args.func(args)


def main():
    """Main invocation for the powervc-config command"""
    data_utils.setup_logging()
    remove_console_exceptions()
    enable_command_translation()
    parse_and_execute_arguments()
