# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015, 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import ConfigParser
import json
import logging
import os
import pexpect
import pwd
import commands
import time
import httplib

from novaclient.client import Client
from keystoneclient.auth.identity import v3 as auth_v3
from keystoneclient import session

from powervc_ras.cli.diag_utils import utils
from powervc_ras.cli.diag_utils import constants
from powervc_ras.cli.diag_utils import security
from powervc_ras import _

LOG = logging.getLogger(__name__)
debug = False
if os.getenv('POWERVCDIAG_DEBUG'):
    debug = True


def novainfo(dinfo, session):
    """Collect nova information.  Use dinfo_nova as the base directory
    for any nova command output.  The username, token, projectid globals
    are set and available for use.
    """

    dinfo_nova = "{0}/nova".format(dinfo)
    utils.createdirs(dinfo_nova)
    if debug:
        stime = time.time()

    # EGO information
    fprofileego = "/opt/ibm/ego/kernel/conf/profile.ego"
    if os.path.exists(fprofileego):
        fname = dinfo_nova + '/ego-allocview.info'

        ego_user = utils.get_value(
            '/etc/nova/nova.conf', 'DEFAULT', 'ego_user')
        ego_encrypted_passwd = utils.get_value(
            '/etc/nova/nova.conf', 'DEFAULT', 'ego_password')
        ego_decrypted_passwd = utils.decrypt(ego_encrypted_passwd)

        cmd = ('/usr/bin/bash -c "source %(ego_profile_file)s && '
               '%(egosh_location)s user logon && '
               '%(egosh_location)s alloc view -i > %(fname)s"' %
               {'ego_profile_file': fprofileego,
                'egosh_location': '/opt/ibm/ego/*/linux*-glibc*/bin/egosh',
                'fname': fname})
        child = None

        try:
            child = pexpect.spawn(cmd)
            child.expect('.*ser.*')
            child.sendline(ego_user)
            child.expect('.*assword.*')
            child.sendline(ego_decrypted_passwd)
            child.expect('Logged on successfully')
        except Exception as e:
            LOG.error(_('ERROR: Unable to retrieve ego-allocview info '
                        'as nova services are down'))
            LOG.exception(e)
        finally:
            if child is not None:
                child.close()

        cmd = "source {0}; /opt/ibm/ego/*/linux*-glibc*/"\
              "bin/rpigrpstat".format(fprofileego)
        stsOK, cout, cerr = utils.runcmd(cmd, wtime=utils.maxwaittime)
        if stsOK is False:
            LOG.error(_('ERROR: Unable to retrieve rpigrpstat info'))

        if stsOK and cout is not None:
            fname = dinfo_nova + '/ego-rpigrpstat.info'
            with open(fname, "w") as finfo:
                finfo.write(cout)
        cmd = "source {0}; rpistat".format(fprofileego)
        stsOK, cout, cerr = utils.runcmd(cmd, wtime=utils.maxwaittime)
        if stsOK is False:
            LOG.error(_('ERROR: Unable to retrieve ego-rpigrpstat info '))

        if stsOK and cout is not None:
            fname = dinfo_nova + '/ego-rpistatall.info'
            with open(fname, "w") as finfo:
                finfo.write(cout)
        cmd = 'source {0}; ' \
            'rpistat -R "select(activeCompute==1)"'.format(fprofileego)
        stsOK, cout, cerr = utils.runcmd(cmd, wtime=utils.maxwaittime)
        if stsOK is False:
            LOG.error(_('ERROR: Unable to retrieve ego-rpistatall info'))

        if stsOK and cout is not None:
            fname_rpistattmp = dinfo_nova + '/ego-rpistattmp.info'
            with open(fname_rpistattmp, "w") as finfo:
                finfo.write(cout)
            cmd = "/bin/egrep 'RES|reserved|proc|vcpu|memory|Consumed|active" \
                "|vms' {0}".format(fname_rpistattmp)
            stsOK, cout, cerr = utils.runcmd(cmd, ignoreerrors=True,
                                             wtime=utils.maxwaittime)
            if stsOK is False:
                LOG.error(_('ERROR: Unable to retrieve ego-rpistattmp info'))

            if stsOK and cout is not None:
                fname = dinfo_nova + '/ego-rpistatfiltered.info'
                with open(fname, "w") as finfo:
                    finfo.write(cout)
            os.remove(fname_rpistattmp)

    # nova information
    cmd = "/usr/bin/nova-manage service list"
    cout = commands.getoutput(cmd)
    # wait time of 10 secs for cmd to execute
    time.sleep(10)
    if cout is not None:
        fname = dinfo_nova + '/manageservicelist.info'
        with open(fname, "w") as finfo:
            finfo.write(cout)
    else:
        print _("ERROR: The command \"{0}\" failed to run.").format(cmd)

    token = session.get_token()
    projectid = session.get_project_id()
    # the following commands require authentication
    if token:
        cmd = '/usr/bin/curl -s -H "X-Auth-Token:' \
            '{0}" http://localhost:8774/v2.1/{1}/flavors/detail | ' \
            '/usr/bin/python -mjson.tool'.format(token, projectid)
        stsOK, cout, cerr = utils.runcmd(
            cmd, ignoreerrors=True, wtime=utils.maxwaittime)
        if stsOK and cout is not None:
            fname = dinfo_nova + '/flavorlist.info'
            with open(fname, "w") as finfo:
                finfo.write(cout)
        elif stsOK is False:
            print _('ERROR: Unable to retrieve flavor details')

        cmd = '/usr/bin/curl -s -H "X-Auth-Token:' \
            '{0}" http://localhost:8774/v2.1/{1}/os-hypervisors/statistics | '\
            '/usr/bin/python -mjson.tool'.format(token, projectid)
        stsOK, cout, cerr = utils.runcmd(
            cmd, ignoreerrors=True, wtime=utils.maxwaittime)
        if stsOK and cout is not None:
            fname = dinfo_nova + '/hypervisorstats.info'
            with open(fname, "w") as finfo:
                finfo.write(cout)
        elif stsOK is False:
            print _('ERROR: Unable to retrieve hypervisor statistics details')

        cmd = '/usr/bin/curl -s -H "X-Auth-Token:' \
            '{0}" http://localhost:8774/v2.1/{1}/ego/prs/hypervisor_status | '\
            '/usr/bin/python -mjson.tool'.format(token, projectid)
        stsOK, cout, cerr = utils.runcmd(
            cmd, ignoreerrors=True, wtime=utils.maxwaittime)
        if stsOK and cout is not None:
            fname = dinfo_nova + '/hypervisorstatuslist.info'
            with open(fname, "w") as finfo:
                finfo.write(cout)
        elif stsOK is False:
            print _('ERROR: Unable to retrieve hypervisor status detail')

        cmd = '/usr/bin/curl -s -H "X-Auth-Token:' \
            '{0}" http://localhost:8774/v2.1/{1}/extensions | '\
            '/usr/bin/python -mjson.tool'.format(token, projectid)
        stsOK, cout, cerr = utils.runcmd(
            cmd, ignoreerrors=True, wtime=utils.maxwaittime)
        if stsOK and cout is not None:
            fname = dinfo_nova + '/extensions.info'
            with open(fname, "w") as finfo:
                finfo.write(cout)
        elif stsOK is False:
            print _('ERROR: Unable to retrieve extensions details')

        # Storage Connectivity Group (SCG) Information
        url = '/storage-connectivity-groups/detail?include_ports=true'
        resp = session.get(url, endpoint_filter={'service_type': 'compute',
                                                 'interface': 'internal'})
        if resp and resp.status_code == httplib.OK:
            fname = dinfo_nova + '/storage-connectivity-groups.info'
            with open(fname, "w") as finfo:
                finfo.write(resp.text)
        else:
            print _('ERROR: Unable to retrieve storage connectivity group '
                    'details')

        # Host Storage Topologies Information
        cmd = '/usr/bin/curl -s -H "X-Auth-Token:' \
            '{0}" http://localhost:8774/v2.1/{1}/host-storage-topologies | '\
            '/usr/bin/python -mjson.tool'.format(token, projectid)
        stsOK, cout, cerr = utils.runcmd(
            cmd, ignoreerrors=True, wtime=utils.maxwaittime)
        if stsOK and cout is not None:
            fname = dinfo_nova + '/host-storage-topologies.info'
            with open(fname, "w") as finfo:
                finfo.write(cout)
        elif stsOK is False:
            print _('ERROR: Unable to retrieve host storage topology '
                    'details')

        # Hypervisors' HA policy information
        cmd = '/usr/bin/curl -s -H "X-Auth-Token:' \
            '{0}" http://localhost:8774/v2.1/{1}/ego/policy/ha/' \
            'hapolicy_hypervisor_list | '\
            '/usr/bin/python -mjson.tool'.format(token, projectid)
        stsOK, cout, cerr = utils.runcmd(
            cmd, ignoreerrors=True, wtime=utils.maxwaittime)
        if stsOK and cout is not None:
            fname = dinfo_nova + '/hypervisor-HApolicy.info'
            with open(fname, "w") as finfo:
                finfo.write(cout)
        elif stsOK is False:
            print _('ERROR: Unable to retrieve hypervisor HA policy '
                    'details')

        # Hypervisors' runtime policy status
        cmd = '/usr/bin/curl -s -H "X-Auth-Token:' \
            '{0}" http://localhost:8774/v2.1/{1}/ego/policy/placement/' \
            'rtpolicy_hypervisor_list | '\
            '/usr/bin/python -mjson.tool'.format(token, projectid)
        stsOK, cout, cerr = utils.runcmd(
            cmd, ignoreerrors=True, wtime=utils.maxwaittime)
        if stsOK and cout is not None:
            fname = dinfo_nova + '/hypervisor-runtime-policy-status.info'
            with open(fname, "w") as finfo:
                finfo.write(cout)
        elif stsOK is False:
            print _('ERROR: Unable to retrieve hypervisor runtime policy '
                    'status details')

        # Collect nova aggregate list and details
        nova_aggregare_list(dinfo_nova)

        # Collect GMR for nova. The output is redirected to /var/log/messages
        get_nova_gmr_report()
    if debug:
        print _("DEBUG: Collected nova (compute) information "
                "in {0} seconds.").format(str(round(time.time() - stime, 3)))


# Task 38475
def nova_aggregare_list(dinfo_nova):
    fname = dinfo_nova + '/aggregatelist.info'

    try:
        # Retrieve the auth_url
        base_conf_handle = ConfigParser.SafeConfigParser(allow_no_value=True)
        base_conf_handle.read('/etc/keystone/keystone.conf')
        auth_url = base_conf_handle.get('DEFAULT', 'public_endpoint', {})
        auth_url = auth_url + '/v3'

        # Retrieve the user and password
        service_user = 'nova'
        project = 'service'
        password = get_service_password(service_user)

        # Create the client to talk to nova
        auth = auth_v3.Password(auth_url=auth_url,
                                user_domain_name='Service',
                                username=service_user,
                                password=password,
                                project_domain_name='Service',
                                project_name=project)
        sess = session.Session(auth=auth,
                               verify='/etc/pki/tls/certs/powervc.crt')
        nova = Client(version='2', session=sess)

        # Get and parse the aggregates
        aggregates = nova.aggregates.list()
        if aggregates is not None:
            with open(fname, "w") as finfo:
                for aggregate in aggregates:
                    aggregate_detail = nova.aggregates.get_details(aggregate)
                    finfo.write(json.dumps(aggregate_detail.to_dict(),
                                           sort_keys=True,
                                           indent=4, separators=(',', ': ')))
    except:
        print _("ERROR: Unable to retrieve nova aggregate list or \
        write details to the file...")


def get_service_password(user):
    password = utils.get_value('/etc/nova/nova.conf', 'keystone_authtoken',
                               'password')
    password = utils.decrypt(password)
    return password

# Task 41218


def get_nova_gmr_report():
    """Collect GMR report for all the nova processes running on powervc server
    """

    print _("INFO: Triggering GMR report for nova...")
    cmd = '/usr/bin/pgrep nova'
    stsOK, cout, cerr = utils.runcmd(
        cmd, ignoreerrors=True, wtime=utils.maxwaittime)
    if stsOK and cout is not None:
        for pid in cout.rstrip().split('\n'):
            gmrcmd = '/usr/bin/kill -USR1 {0}'.format(pid)
            utils.startcmd(gmrcmd)
            time.sleep(1)
    else:
        print _("ERROR: Unable to get nova process id's")


def serviceinfo(service, dinfo, session):
    """Collect service information.  Use dinfo_service as the base directory
    for any service command output.  The username, token, projectid globals
    are set and available for use.
    """
    if service not in ("glance", "nova"):
        return
    dinfo_service = "{0}/{1}".format(dinfo, service)
    utils.createdirs(dinfo_service)
    print (_("INFO: Collecting %s (%s) information...")
           % (service, constants.service_map[service]))
    if debug:
        stime = time.time()
        print (_("DEBUG: Collected %s (%s) information "
                 "in {0} seconds.").format(str(round(time.time() - stime, 3)) %
                                           (service, constants.service_map[
                                           service])))

    service_data = service + 'info'
    globals()[service_data](dinfo, session)


def glanceinfo(dinfo, session):
    """Collect glance information.  Use dinfo_glance as the base directory
    for any glance command output.  The username, token, projectid globals
    are set and available for use.
    """

    dinfo_glance = "{0}/glance".format(dinfo)
    utils.createdirs(dinfo_glance)
    if debug:
        stime = time.time()

    # the following commands require authentication
    token = session.get_token()
    if token:
        cmd = '/usr/bin/curl -s -H "X-Auth-Token:' \
            '{0}" http://localhost:9292/v2/images | ' \
            '/usr/bin/python -mjson.tool'.format(token)
        stsOK, cout, cerr = utils.runcmd(
            cmd, ignoreerrors=True, wtime=utils.maxwaittime)
        if stsOK and cout is not None:
            fname = dinfo_glance + '/imagedetails.info'
            with open(fname, "w") as finfo:
                finfo.write(cout)
        elif stsOK is False:
            print _('ERROR: Unable to retrieve images details as '
                    'glance services are down')

    if debug:
        print _("DEBUG: Collected glance (image) information "
                "in {0} seconds.").format(str(round(time.time() - stime, 3)))
