# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import argparse
from powervc_ras.cli.diag_utils import utils
from powervc_ras import _


def parsecmd():
    """Parse the command invocation using argparse.
    """
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=_('A utility to collect diagnostic data.  The default '
                      'action is to collect all diagnostic data.'))
    parser.add_argument(
        '-v',
        '--validate',
        action='store_true',
        required=False,
        help=_(
            '  -v, --validate'
            '    Run the environment validator.'
            '    If you do not specify this option, the output from the '
            '    last validation is collected.'))
    parser.add_argument(
        '-u',
        '--user',
        nargs=1,
        required=False,
        help=_(
            '  -u USER, --user USER'
            '    Specify the user that should be used for authentication'
            ' to capture '
            '    certain diagnostics.'
            '    If you do not specify this option, the user name is '
            'obtained'
            '    from the OS_USERNAME environment variable or assumed from'
            '    whoami.'))
    parser.add_argument(
        '-o',
        '--output-dir',
        nargs=1,
        required=False,
        help=_(
            '  -o OUTPUT_DIR, --output-dir OUTPUT_DIR'
            '    Specifies the output directory to use for the diagnostic'
            '    data that is collected.'
            '    If you do not specify a directory, the default is the'
            '    directory used for the TMPDIR environment variable.  If'
            '    TMPDIR or the directory specified for TMPDIR does not '
            'exist,'
            '    then /tmp is used.'))
    parser.add_argument(
        '-f',
        '--archive-fname',
        nargs=1,
        required=False,
        help=_(
            '  -f ARCHIVE_FNAME, --archive-fname ARCHIVE_FNAME'
            '    Specifies the name of the archive file into which the'
            '    the collected diagnostic data will be added.'
            '    The .tgz file extension is added to the name you provide.'))
    parser.add_argument(
        '-n',
        '--noprompt',
        action='store_true',
        required=False,
        help=_(
            '  -n, --noprompt'
            '    Specifies that you do not want to be prompted for user '
            'input'
            '    or confirmations.  You will always be prompted for '
            'passwords'
            '    and security prompts whether or not this option is'
            '    specified.'))
    parser.add_argument(
        '-w',
        '--maxwait',
        nargs=1,
        required=False,
        help=_(
            '  -w, --maxwait'
            '    Specifies the maximum amount of time to wait for '
            'individual'
            '    diagnostic commands run by powervc-diag.'
            '    The amount of time should be specified as seconds.  The'
            '    default wait time is 300 seconds (5 minutes).'))
    parser.add_argument(
        '-r',
        '--project',
        nargs=1,
        required=False,
        help=_(
            '  -r, --project'
            '    Specifies the keystone project name to use when'
            '    authenticating the specified --user.  If not'
            '    specified, "ibm-default" will be used.'))

    if utils.ispowerKVM():
        parser.add_argument(
            '-c',
            '--kvminfo',
            nargs='?',
            const='all',
            required=False,
            help=_(
                '  -c [KVMNAME], --kvminfo [KVMNAME]'
                '    Collects KVM host information '
                '    i.e., libvirt logs, kernel logs, PowerKVM logs, etc.'
                '    and generate a SOSREPORT '
                '    If you do not specify a host, then data will be '
                '    collected for all registered hosts'))
        # TODO - Deprecate option
        parser.add_argument(
            '-p',
            '--runparallel',
            action='store_true',
            required=False,
            help=_(
                '  -p, --runparallel'
                '   Collects host information in parallel for all '
                '   registered hosts, or for the number of hosts '
                '   provided '))

    if utils.isofferingHMC():
        parser.add_argument(
            '-c',
            '--hmcinfo',
            nargs='?',
            const='all',
            required=False,
            help=_(
                '  -c [HMCNAME], --hmcinfo [HMCNAME]'
                '    Collects HMC log information.  Requires password '
                '    for HMC service id <hscpe>. It is also recommended to'
                '    increase the -w option as this is a long running '
                '    operation'
                '    If you do not specify a host, then data will be '
                '    collected for all registered hosts'))
        parser.add_argument(
            '-p',
            '--runparallel',
            action='store_true',
            required=False,
            help=_(
                '  -p, --runparallel'
                '   Collects host information in parallel for all '
                '   registered hosts, or for the number of hosts '
                '   provided '))
        # TODO- Replace nwo with novalink
        parser.add_argument(
            '-novalink',
            '--novalinkinfo',
            nargs='?',
            const='all',
            required=False,
            help=_(
                '  -novalink [NEONAME], --novalinkinfo [NEONAME]'
                '    Collects Novalink log information. '
                '    If you do not specify a host, then data will be '
                '    collected for all registered hosts'))

    return parser
