# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015, 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import os
import paramiko
import binascii
import getpass
import logging
import requests
import sys
import time
import pwd
from powervc_oslo.cmd import auth
from powervc_ras.cli.diag_utils import utils
from powervc_ras.cli.diag_utils import constants
from powervc_ras.cli.powervc_list_roles import ValidateUsersRole
from powervc_ras import _
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
debug = False
if os.getenv('POWERVCDIAG_DEBUG') is not None:
    debug = True

LOG = logging.getLogger(__name__)


def ssh_hmc(hmc_hosts):
    remove_host_list = []
    if hmc_hosts:
        # here load the clients for all the hmc in the dict
        for hmc_name in hmc_hosts.iterkeys():
            hmc = hmc_hosts[hmc_name]
            try:
                ip = hmc['ip']
                password = hmc['password']
                client_hmc = paramiko.SSHClient()
                client_hmc.load_host_keys(constants.known_hosts_file)
                client_hmc.WarningPolicy = CustomWarningPolicy
                client_hmc.set_missing_host_key_policy(CustomWarningPolicy())
                client_hmc.connect(ip, username='hscpe',
                                   password=password)
                hmc_hosts[hmc_name]['client_hmc'] = client_hmc
            except Exception as e:
                print _("Error:Failed to connect to HMC \'{0}\'."
                        " {1}").format(hmc_name, e)
                remove_host_list.append(hmc_name)

        for host in remove_host_list:
            if host in hmc_hosts.keys():
                del hmc_hosts[host]


def collect_powervc_user(dinfo):
        # collect powervc-user-list-roles
    print _("INFO: Collecting user roles and group information...")
    vuserrole = ValidateUsersRole()
    missing_users, missing_groups, users_dict = vuserrole.get_user_map()
    fname = dinfo + '/powervcuser.info'
    if vuserrole.isLdapEnabled():
        with open(fname, "w") as finfo:
            finfo.write('\nThis is a LDAP environment.\n')
            finfo.write('User and Group information will '
                        'not be recorded in logs.\n')
        return
    elif users_dict:
        assgn_key = 'Assignments'
        with open(fname, "w") as finfo:
            finfo.write('\nDetails of PowerVC Assignments : \n')
            user_assgn = users_dict.pop(assgn_key, None)
            if user_assgn:
                for assgn in user_assgn:
                    assgn_keys = assgn.keys()
                    assgn_keys.sort()
                    for key1 in assgn_keys:
                        finfo.write('\n %s : ' % key1)
                        finfo.write(str(assgn.get(key1)))
                    finfo.write("\n===============================\n")
            else:
                finfo.write("\nThere were no assignments found\n")

            finfo.write('\nSummary of Missing Users/Groups : \n')
            finfo.write('\n Missing Users : %s' % str(missing_users))
            finfo.write('\n Missing Groups : %s ' % str(missing_groups))

            for project, project_dict in users_dict.iteritems():
                finfo.write("\n===============================\n")

                finfo.write('\nSummary of Users for each Role in Project \'' +
                            project + '\' : \n')
                if not project_dict:
                    finfo.write('\n None')
                else:
                    for key, value in project_dict.iteritems():
                        if key != assgn_key:
                            finfo.write('\n %s : ' % key)
                            finfo.write(str(value))
                finfo.write('\n')


def validate_user():
        # command must be run with root authority
    euid = os.geteuid()
    if euid != 0:
        LOG.error(_("ERROR: The powervc-diag command must be run with root "
                    "authority.  Log in as root and rerun the command or use "
                    "sudo to invoke the command."))
        utils.cmdexit(
            "ERROR: The powervc-diag command must be run with root "
            "authority.  Log in as root and rerun the command or use "
            "sudo to invoke the command.", 1)


def get_user(args):
    if args.user:
        return args.user[0]
    return None


def get_project(args):
    if args.project:
        return args.project[0]
    return None


def generate_token(username, project):

    # Retrieve a token with the specified username and password
    try:
        sess = auth.get_session(user=username, project=project,
                                user_agent='powervc-diag')
        token = sess.get_token()
        projectid = sess.get_project_id()
    except Exception:
        # Authentication failed, clear out token and projectid
        token = None
        projectid = None
        sess = None

    if not token or not projectid:
        print _("WARNING: Authentication failed. Commands that require "
                "authentication will fail.")
        token = None
        projectid = None
    return sess


class CustomWarningPolicy(paramiko.client.MissingHostKeyPolicy):

    def missing_host_key(self, client, hostname, key):
        if key is not None and os.path.exists(constants.known_hosts_file):
            custom_hosts = paramiko.hostkeys.HostKeys(
                constants.known_hosts_file)
            all_keys = [entry.key for entry in custom_hosts._entries]
            if key in all_keys:
                return
        choice = "Y"
        fingerprint = binascii.hexlify(key.get_fingerprint())
        fingerprint = ':'.join(a + b for a, b in zip(fingerprint[::2],
                                                     fingerprint[1::2]))
        while choice != "y" and choice != "n":

            sys.stdout.write(_("The authenticity of host '%s' can't be"
                               " established.\n") % hostname)
            sys.stdout.write(_("RSA key fingerprint is %s\n") % fingerprint)
            sys.stdout.write(_("Host key will be saved in"
                               " the known_hosts file."
                               " Are you sure you want to continue"
                               " connecting (y/n) : "))
            choice = raw_input().lower()
            if choice == "y":
                client._host_keys.add(hostname, key.get_name(), key)
                client.save_host_keys(constants.known_hosts_file)

            if choice != "y" and choice != "n":
                print _("'{0}' is not a valid response.").format(choice)
        if choice == "n":
            LOG.exception(_('User cancelled connection attempt'))
            raise Exception(_('User cancelled connection attempt'))
