# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014, 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import argparse
import sys
import logging
import datetime
import os
import commands
import glob
import socket
import paramiko
import re
from powervc_ras import _
from powervc_oslo import enable_command_translation
enable_command_translation()
from powervc_keystone.encrypthandler import EncryptHandler

'''
LogManager will be responsible for reporting and configuring
logrotate configuration files based on user options.
'''
POWERVC_INSTALL_DIR = '/opt/ibm/powervc'
LOG_FILE_BASENAME = '%s/log/powervc-log-management' % POWERVC_INSTALL_DIR
HOST_TYPE = ['kvmhost',
             'novalinkhost']
LOG = logging.getLogger(__name__)

CONF_PATH = '/etc/logrotate.d'
FILE_LIST = ['openstack-ceilometer', 'openstack-cinder', 'openstack-glance',
             'openstack-keystone', 'openstack-neutron', 'openstack-nova',
             'powervc-bumblebee', 'powervc-ttv', 'openstack-swift']

version_details_map = {'kvm': {'ip': 'kvm_host_ip',
                               'user': 'kvm_host_user',
                               'passwd': 'kvm_host_passwd'},
                       'novalink': {'ip': 'powervm_host_ip',
                                    'user': 'powervm_host_user',
                                    'passwd': 'powervm_host_passwd'}}
known_hosts_file = "/opt/ibm/powervc/data/known_hosts"


class LogManager():

    def __init__(self):
        self.return_dict = {}
        self.return_dict_management = {}
        # self.return_dict_kvm = {}
        self.return_dict_remote_hosts = {}

    def set_remote_ip(self, host_type, argument_dict):
        if host_type is "kvm":
            remote_ip = argument_dict["kvmhost"]['ip']
        elif host_type is "novalink":
            remote_ip = argument_dict["novalinkhost"]['ip']
        return remote_ip

    def connect_remote_host(self, remote_ip, host_details, argument_dict):
        if host_details is not None:
            # connect to host to read the values
            try:
                client = paramiko.SSHClient()
                client.load_host_keys(known_hosts_file)
                client.set_missing_host_key_policy(paramiko.RejectPolicy())
                client.connect(remote_ip,
                               username=host_details['host_user'],
                               password=host_details['host_passwd'])
                if "novalinkhost" in argument_dict.keys():
                    view_host_cmd = "sudo -S /opt/ibm/powervc/bin/powervc"\
                        "-log-management-pvm view"
                    stdin, stdout, stderr = client.exec_command(
                        view_host_cmd)
                    password = host_details['host_passwd']
                    stdin.write(password + '\n')
                    stdin.flush()
                    data = stdout.read()
                    return data
                elif "kvmhost" in argument_dict.keys():
                    view_host_cmd = "/opt/ibm/powervc/bin/powervc"\
                        "-log-management-kvm view"
                    stdin, stdout, stderr = client.exec_command(
                        view_host_cmd)
                    data = stdout.read()
                    return data
            except Exception as e:
                LOG.error(_("Unable to connect to remote"
                            " host: {0} {1}").format(remote_ip, str(e)))

                cmdexit("ERROR: {0}".format(e), 1)
            LOG.debug("Collecting configuration data from :%s" % remote_ip)
        else:
            LOG.debug("Host(%s) is not managed by PowerVC"
                      " instance." % remote_ip)
            LOG.error(_("Unable to get host: %s details."
                        " Check log for more details.") % remote_ip)
            cmdexit("ERROR: {0}".format(e), 1)

    def parse_stdout(self, remote_ip, host_type, data, return_dict):
        # data = stdout.read()
        list_result = data.split('\n')
        conf_file = ""
        conf_file_data = []
        for line in list_result:
            if "Configuration file: " in line:
                if conf_file != "":
                    self.return_dict_remote_hosts[conf_file] = '\n'.\
                        join(conf_file_data)
                    conf_file = line.replace("Configuration file: \
                                ", "")
                    conf_file_data = []
                else:
                    line = line.replace("Configuration file: ", "")
                    conf_file = line
            elif ".log" in line:
                conf_file_data.append(line)
            elif "size" in line:
                conf_file_data.append(line)
            elif "rotate" in line:
                conf_file_data.append(line)
            elif "}" in line:
                conf_file_data.append(line)
        if conf_file_data and conf_file != "":
            self.return_dict_remote_hosts[conf_file] = '\n'.\
                join(conf_file_data)
            self.return_dict_remote_hosts['ip'] = remote_ip
            self.return_dict[host_type] = self.return_dict_remote_hosts

        return return_dict

    def viewConfiguration(self, host_type, argument_dict, kvm=False):
        remote_ip = ''
        for filename in FILE_LIST:
            conf_file = "%s/%s" % (CONF_PATH, filename)
            cmd = "/bin/cat %s" % conf_file
            ret_status, ret_output = commands.getstatusoutput(cmd)
            if ret_status == 0:
                self.return_dict_management[filename] = ret_output
                LOG.debug("Setting the output for file %s : %s"
                          % (conf_file, ret_output))
            else:
                LOG.error(_("Error while reading file contents."))
                LOG.error(_("filename: {0}, return output:"
                            " {1}").format(conf_file, ret_output))
        self.return_dict['management_node'] = self.return_dict_management

        if "kvmhost" in argument_dict.keys() or \
                "novalinkhost" in argument_dict.keys():
            remote_ip = self.set_remote_ip(host_type, argument_dict)

            remote_hosts = Utility.load_remote_hosts(host_type)
            host_details = remote_hosts.get(remote_ip)

            # prints error if the invalid/ host not managed by PowerVC
            if host_details is None:
                LOG.error(_("ERROR: Host %s is not managed by PowerVC"
                            " instance.") % remote_ip)
                return self.return_dict

            data = self.connect_remote_host(
                remote_ip,
                host_details,
                argument_dict)
            self.return_dict = self.parse_stdout(
                remote_ip,
                host_type,
                data,
                self.return_dict)
        return self.return_dict

    def modifyConfiguration(self, host_type, argument_dict, kvm=False):
        self.return_dict['error'] = False
        new_retain_value = None
        new_size_value = None
        cmd_list = []
        modified_attributes = None
        modified_attributes_value = None
        if argument_dict['retain'] is not None:
            new_retain_value = argument_dict['retain']
        if argument_dict['size'] is not None:
            new_size_value = argument_dict['size']
        for filename in FILE_LIST:
            if "kvmhost" in argument_dict.keys() \
                    or "novalinkhost" in argument_dict.keys():
                break
            conf_file = "%s/%s" % (CONF_PATH, filename)
            if new_size_value is not None and new_retain_value is not None:
                cmd1 = "/bin/sed -i -e \"s/^ .*size.*/    size %s/\""\
                    "  %s" % (new_size_value, conf_file)
                cmd2 = "/bin/sed -i -e \"s/^ *.\<rotate.*\>/    rotate %s/\""\
                    "  %s" % (new_retain_value, conf_file)
                modified_attributes = "size and rotate"
                modified_attributes_value = "%s and %s" % (new_size_value,
                                                           new_retain_value)
                cmd_list.append(cmd1)
                cmd_list.append(cmd2)
            elif new_size_value is not None:
                cmd = "/bin/sed -i -e \"s/^ .*size.*/    size %s/\""\
                    "  %s" % (new_size_value, conf_file)
                modified_attributes = "size"
                modified_attributes_value = new_size_value
                cmd_list.append(cmd)
            elif new_retain_value is not None:
                cmd = "/bin/sed -i -e \"s/^ *.\<rotate.*\>/    rotate %s/\""\
                    "  %s" % (new_retain_value, conf_file)
                modified_attributes = "rotate"
                modified_attributes_value = new_retain_value
                cmd_list.append(cmd)
            for cmd in cmd_list:
                ret_status, ret_output = commands.getstatusoutput(cmd)
                if ret_status == 0:
                    LOG.debug("Modified the %s attribute for file %s to %s"
                              % (modified_attributes, conf_file,
                                 modified_attributes_value))
                else:
                    LOG.error(_("Error while modifying configuration file"
                                " settings."))
                    LOG.error(_("filename: {0}, return output:"
                                " {1}").format(conf_file, ret_output))
        if "kvmhost" in argument_dict.keys() \
                or "novalinkhost" in argument_dict.keys():
            remote_ip = self.set_remote_ip(host_type, argument_dict)
            remote_host_dict = Utility.load_remote_hosts(host_type)
            host_details = remote_host_dict.get(remote_ip)
            remote_cmd = None
            if host_details is not None:
                # connect to host to read the values
                try:
                    client = paramiko.SSHClient()
                    client.load_host_keys(known_hosts_file)
                    client.set_missing_host_key_policy(paramiko.RejectPolicy())
                    client.connect(remote_ip,
                                   username=host_details['host_user'],
                                   password=host_details['host_passwd'])
                except Exception as e:
                    LOG.error(_("Unable to connect to remote"
                                " host: {0} {1}").format(remote_ip, str(e)))
                    self.return_dict['error'] = True
                    return self.return_dict
                LOG.debug("Collecting configuration data from :%s" % remote_ip)
                if "novalinkhost" in argument_dict.keys():
                    if new_size_value is not None and \
                            new_retain_value is not None:
                        remote_cmd = "sudo -S /opt/ibm/powervc/bin/powervc"\
                            "-log-management-pvm configure size"\
                            " %s retain "\
                            "%s" % (new_size_value, new_retain_value)
                    elif new_size_value is not None:
                        remote_cmd = "sudo -S /opt/ibm/powervc/bin/powervc"\
                            "-log-management-pvm configure size"\
                            " %s" % new_size_value
                    elif new_retain_value is not None:
                        remote_cmd = "sudo -S /opt/ibm/powervc/bin/powervc"\
                            "-log-management-pvm configure retain"\
                            " %s" % new_retain_value
                    stdin, stdout, stderr = client.exec_command(remote_cmd)
                    password = host_details['host_passwd']
                    stdin.write(password + '\n')
                    stdin.flush()
                elif "kvmhost" in argument_dict.keys():
                    if new_size_value is not None and \
                            new_retain_value is not None:
                        remote_cmd = "/opt/ibm/powervc/bin/powervc"\
                            "-log-management-kvm configure size"\
                            " %s retain "\
                            "%s" % (new_size_value, new_retain_value)
                    elif new_size_value is not None:
                        remote_cmd = "/opt/ibm/powervc/bin/powervc"\
                            "-log-management-kvm configure size"\
                            " %s" % new_size_value
                    elif new_retain_value is not None:
                        remote_cmd = "/opt/ibm/powervc/bin/powervc"\
                            "-log-management-kvm configure retain"\
                            " %s" % new_retain_value
                    stdin, stdout, stderr = client.exec_command(remote_cmd)
                return_data = stdout.read()
                LOG.debug("Output from the remote command: %s is "
                          "%s" % (remote_cmd, return_data))
            else:
                LOG.debug("Host(%s) is not managed by PowerVC"
                          " instance." % remote_ip)
                LOG.error(_("Unable to get host: %s details."
                            " Check log for more details.") % remote_ip)
                self.return_dict['error'] = True

        return self.return_dict


class Utility:

    @staticmethod
    def open_func(name):
        return open(name, "r")

    @staticmethod
    def isPowerKVM(version_file=None, open_func=open_func):
        try:
            if version_file is None:
                version_file = "/opt/ibm/powervc/version.properties"
            with Utility.open_func(version_file) as f:
                lines = f.read()
                value = Utility.getValue('offering', lines)
                if value is not None and value == 'powerkvm':
                    powerkvm = True
                else:
                    powerkvm = False
        except:
            LOG.error(_('The file /opt/ibm/powervc/version.properties'
                        ' either does not exist or has been deleted.'))
            sys.exit(1)

        return powerkvm

    @staticmethod
    def getValue(key, context):
        """ helper method to fnd the value of key in buffer"""
        match = None
        for line in context.splitlines():
            if key in line:
                match = line
                break
        if match is not None:
            return match[match.find("=") + 2:].strip()
        else:
            return None

    @staticmethod
    def checkCloudEnabled():
        """ Helper method to check if cloud enabled """
        cloud_enabled = False
        try:
            with open("/opt/ibm/powervc/version.properties", "r") as f:
                lines = f.read()
                if 'cloud_enabled' in lines:
                    cloud_enabled = True
                else:
                    cloud_enabled = False
        except:
            LOG.error(_('Error: The file /opt/ibm/powervc/version.properties'
                        ' either does not exist or has been deleted.'))
            cmdexit(_('Error: The file /opt/ibm/powervc/version.properties'
                      ' either does not exist or has been deleted.'), 1)
        return cloud_enabled

    @staticmethod
    def load_remote_hosts(host_type):
        """ helper method to load remote hosts information \
        in to a mapping table"""
        confile_path = "/etc/nova"
        remote_host_list = {}
        conffiles = glob.glob("{0}/nova-*.conf".format(confile_path))
        for conffile in conffiles:
            with open(conffile, "r") as f:
                lines = f.read()
            remote_host_ip = Utility.getValue(version_details_map[
                host_type]['ip'], lines)
            if remote_host_ip is not None:
                hostdetails = {}
                hostdetails['host_display_name'] = Utility.getValue(
                    'host_display_name',
                    lines)
                hostdetails['host_user'] = Utility.getValue(
                    version_details_map[
                        host_type]['user'], lines)
                hostdetails['host_passwd'] = Utility.decrypt(
                    Utility.getValue(version_details_map[
                        host_type]['passwd'], lines))
                remote_host_list[remote_host_ip] = hostdetails
        return remote_host_list

    @staticmethod
    def decrypt(text):
        """Convert an encrypted string to a clear text one. Clear text will be
        unchanged"""
        if text is None:
            return None
        encrypt_handler = EncryptHandler()
        return encrypt_handler.decode(text)

    @staticmethod
    def validateIPAddress(ip):
        """ Validate if the ip address passed if of valid ipv4 format or a
        valid ipv6 format
        """
        if ip.count('.') == 3:
            if not Utility.is_valid_ipv4_address(ip):
                raise Exception(_("Ip address passed: %s is not of valid"
                                  " ipv4 format") % ip)
        else:
            if not Utility.is_valid_ipv6_address(ip):
                raise Exception(_("Ip address passed: %s is not of valid"
                                  " ipv6 format") % ip)

    @staticmethod
    def is_valid_ipv4_address(address):
        try:
            socket.inet_pton(socket.AF_INET, address)
        except AttributeError:  # no inet_pton here, sorry
            try:
                socket.inet_aton(address)
            except socket.error:
                return False
            return address.count('.') == 3
        except socket.error:  # not a valid address
            return False
        return True

    @staticmethod
    def is_valid_ipv6_address(address):
        try:
            socket.inet_pton(socket.AF_INET6, address)
        except AttributeError:  # no inet_pton here, sorry
            try:
                socket.inet_aton(address)
            except socket.error:
                return False
            return True
        except socket.error:  # not a valid address
            return False
        return True


class Parser():

    def __init__(self, kvm=False):
        """
        Initialize the options and the help message
        """
        self.opts = _('\nargument information:\n'
                      '  -c , --configure \n'
                      '    Configure new settings for the logrotate'
                      ' configuration files\n'
                      '  -v , --view \n'
                      '    View the current setting for logrotate '
                      'configuration files.This will display a '
                      'consoliatated view of the setting '
                      'across the configuration files\n'
                      '  -s, --size\n'
                      '    Specify the maximum size after which the'
                      ' log files will be rotated. \n'
                      '    Unit of measure : K(KiloBytes),M(MegaBytes),'
                      'G(GigaBytes).Default unit is M \n'
                      '  -r , --retain \n'
                      '    Specifies the number of log archives to'
                      ' be retained\n')
        ex_view = sys.argv[0] + ' -v'
        ex_config_size = sys.argv[0] + ' -c -s 10M'
        ex_config_r = sys.argv[0] + ' -c -r 10'
        ex_config_s_r = sys.argv[0] + ' -c -s 10M -r 10'
        self.examples = _('\n\nnotes:'
                          '\n If arguments are not specified then the current '
                          'settings will be displayed.'
                          '\n\nexamples:'
                          '\n  {0}'
                          '\n    - Displays consolidated view of logrotate '
                          'configuration settings'
                          '\n  {1}'
                          '\n    - Configures maximum size of a file before it'
                          ' is archived to be 10 MB'
                          '\n  {2}'
                          '\n    - Configures maximum number of archive files'
                          ' retained to be 10'
                          '\n  {3}'
                          '\n    - Configures maximum size of a file before '
                          'it is archived to be 10 MB '
                          'and maximum number of archive files retained'
                          ' to be 10\n').format(ex_view, ex_config_size,
                                                ex_config_r, ex_config_s_r)
        self.kvmoption = _('\n  -k [kvm host], --kvmhost [kvm host]\n'
                           '    Collects KVM host information for logrotate '
                           'configuration files. Accepts only '
                           'ipaddress as input\n')
        self.novalinkoption = _('\n  -n [novalink host], --novalinkhost '
                                '[novalink host]\n'
                                '    Collects Novalink host information '
                                'for logrotate '
                                'configuration files. Accepts only '
                                'ipaddress as input\n')
        if kvm:
            self.kvm = True
            self.details = self.opts + self.kvmoption + self.examples
        else:
            self.details = self.opts + self.novalinkoption + self.examples
            self.kvm = False

    def parse(self, kvm=False):
        return_dict = {'configure': False, 'view': True,
                       'size': None, 'retain': None}
        self.parser = argparse.ArgumentParser(
            formatter_class=argparse.RawDescriptionHelpFormatter,
            description=_('A utility to view/modify the settings'
                          ' for log management. The default action'
                          ' is to view the current settings.'),
            epilog=self.details)
        self.parser.add_argument('-v', '--view', action='store_true',
                                 required=False)
        self.parser.add_argument('-c', '--configure', action='store_true',
                                 required=False)
        self.parser.add_argument('-s', '--size', nargs=1, required=False)
        self.parser.add_argument('-r', '--retain', nargs=1, required=False)

        if self.kvm:
            self.parser.add_argument('-k', '--kvmhost', nargs=1,
                                     required=False)
        else:
            self.parser.add_argument(
                '-n', '--novalinkhost', nargs=1, required=False)

        args = self.parser.parse_args()
        if args.view:
            return_dict['view'] = True
        if args.configure:
            if args.view:
                raise Exception(_("Options: configure and view"
                                  " cannot be used together"))
            if not args.size and not args.retain:
                raise Exception(_("configure option should be used in"
                                  " conjunction with the size or"
                                  " retain option."))
            return_dict['configure'] = True
            return_dict['view'] = False
        if args.size:
            if return_dict['configure'] is False:
                raise Exception(_("size option needs to be paired"
                                  " with the configure option"))
            else:
                if re.match('^[0-9]+[mMkKgG]?$', str(args.size[0])):
                    return_dict['size'] = str(args.size[0])
                else:
                    raise Exception(_("size value: %s is not "
                                      "correct.") % str(args.size[0]))
        if args.retain:
            if return_dict['configure'] is False:
                raise Exception(_("retain option needs to be paired"
                                  " with the configure option"))
            else:
                if re.match('^[0-9]+$', str(args.retain[0])):
                    return_dict['retain'] = str(args.retain[0])
                else:
                    raise Exception(_("retain value: %s is not "
                                      "correct.") % str(args.retain[0]))

        if self.kvm:
            if args.kvmhost:
                kvmhost = str(args.kvmhost[0])
                # check valid ip details
                try:
                    Utility.validateIPAddress(ip=kvmhost)
                    return_dict['kvmhost'] = {'ip': kvmhost}
                except Exception as e:
                    raise Exception(str(e))
        else:
            if args.novalinkhost:
                novalinkhost = str(args.novalinkhost[0])
                # check valid ip details
                try:
                    Utility.validateIPAddress(ip=novalinkhost)
                    return_dict['novalinkhost'] = {'ip': novalinkhost}
                except Exception as e:
                    raise Exception(str(e))
        return return_dict


def main():
    if os.geteuid() != 0:
        print (_("ERROR: You do not have root privileges"
                 " to run this command."))
        sys.exit(1)
    initialize_logger()
    try:
        isKvm = Utility.isPowerKVM()
        parser = Parser(kvm=isKvm)
        return_dict = parser.parse()
        LOG.debug("Dictionary of parsed arguments")
        LOG.debug(return_dict)
        manager = LogManager()
        if Utility.isPowerKVM():
            host_type = 'kvm'
        else:
            host_type = 'novalink'
        check_cloud_enabled = Utility.checkCloudEnabled()
        if check_cloud_enabled:
            FILE_LIST.append('powervc-clerk')
        if return_dict['view'] is True:
            result = manager.viewConfiguration(host_type,
                                               argument_dict=return_dict,
                                               kvm=isKvm)
            LOG.info(_("\n\nOutput from the view option is:\n"))
            print_console(result)
            LOG.debug(result)
        elif return_dict['configure'] is True:
            result = manager.modifyConfiguration(host_type,
                                                 argument_dict=return_dict,
                                                 kvm=isKvm)
            if result['error'] is False:
                LOG.info(_("\nSuccessfully modified "
                           "the settings for logrotate"))
                LOG.info(_("You may run the command again with the"
                           " -v|--view option to see the modified settings."))
    except Exception as e:
        cmdexit("ERROR: {0}".format(e), 1)


def print_console(result_dict):
    result_management_node = result_dict['management_node']
    print (_("Current settings on the management node"))
    for key in result_management_node.keys():
        print "--------------------------------------------------------"
        print (_("Configuration file: %s") % key)
        print "--------------------------------------------------------"
        print (_("Current values for size and rotate:\n"))
        for f in result_management_node[key].split('\n'):
            if 'postrotate ' in f:
                continue
            elif 'size ' in f or 'rotate ' in f or '.log' in f or '}' in f:
                print f
    # Prints the current settings for kvm/novalink host
    if 'kvm' in result_dict or 'novalink' in result_dict:
        if 'kvm' in result_dict:
            result_kvm = result_dict['kvm']
        elif 'novalink' in result_dict:
            result_kvm = result_dict['novalink']
        print (_("\n\nCurrent settings on the host %s") % result_kvm['ip'])
        for key in result_kvm.keys():
            if key == 'ip':
                continue
            print "--------------------------------------------------------"
            print (_("Configuration file: %s") % key)
            print "--------------------------------------------------------"
            print (_("Current values for size and rotate:\n"))
            for f in result_kvm[key].split('\n'):
                if 'size ' in f or 'rotate ' in f or '.log' in f or '}' in f:
                    print f


def initialize_logger():
    """Initialize the log object.
    Log filename and the entries will have timestamp
    """
    global timestamp
    dt = datetime.datetime.now()
    # Configure Logging
    if not os.path.exists(LOG_FILE_BASENAME):
        os.makedirs(LOG_FILE_BASENAME)
    global log_filename
    timestamp = "%s%s%s%s%s%s%s" % (dt.year,
                                    dt.month,
                                    dt.day,
                                    dt.hour,
                                    dt.minute,
                                    dt.second,
                                    dt.microsecond)
    log_filename = "%s/powervc-log-management_%s.log" % (LOG_FILE_BASENAME,
                                                         timestamp)

    loglevel_number = getattr(logging, "DEBUG", None)

    logging.basicConfig(level=loglevel_number,
                        format='%(asctime)s %(name)-12s '
                        '%(levelname)-8s %(message)s',
                        datefmt='%m-%d %H:%M',
                        filename=log_filename,
                        filemode='w')
    # define a Handler which writes INFO messages or higher to sys.stderr
    console = logging.StreamHandler()
    console.setLevel(logging.INFO)
    # set a format which is simpler for console use
    formatter = logging.Formatter('%(message)s')
    # tell the handler to use this format
    console.setFormatter(formatter)
    # add the handler to the root logger
    logging.getLogger(__name__).addHandler(console)
    LOG.debug("log filename = %s" % log_filename)


def cmdexit(msg, status):
    """Exit command with specified message and exit code.
    @param: msg - Exit message
    @param: status - Exit code (0 is successful, non-zero indicates failure)
    """
    LOG.error(msg)
    sys.exit(status)


if __name__ == '__main__':
    main()
