# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================


from sos.plugins import Plugin, RedHatPlugin
import os


class PowervcKVM(Plugin, RedHatPlugin):

    """Neutron/Nova/Ceilometer related information
    """

    plugin_name = 'powervcKVM'

    def get_vsctl_name(self, vsctl_out):

        out = []
        for line in vsctl_out[1].splitlines():
            out.append(line.strip())
        return out

    def getdumpflows(self):
        vsctl_out = self.call_ext_prog("ovs-vsctl list-br")
        if vsctl_out:
            for br_name in self.get_vsctl_name(vsctl_out):
                self.add_cmd_output("ovs-ofctl dump-flows " + br_name)

    def get_vm_ids(self, vmdetails):
        out = []
        for line in vmdetails[1].splitlines():
            line = line.strip()
            # Skipping header/dotted lines/spaces from output
            if 'Id' not in line and '----' not in line and len(line) != 0:
                out.append(line)
        return out

    def getvmdumps(self):
        vmdetails = self.call_ext_prog("virsh list --all|awk '{print $1}'")
        if vmdetails:
            for vmid in self.get_vm_ids(vmdetails):
                self.add_cmd_output("virsh dumpxml " + vmid)

    def setup(self):
        self.add_cmd_output("ovs-vsctl show")
        self.add_cmd_output("ip link show")
        self.add_cmd_output("brctl show")
        self.add_cmd_output("lshw -class network")
        self.add_copy_specs(
            ["/etc/neutron/plugins/openvswitch/*.ini",
             "/etc/nova/*",
             "/etc/ceilometer/*",
             "/etc/neutron/*",
             "/etc/sysconfig/network-scripts/*"])

        if os.path.exists("/proc/net/bonding/"):
            self.add_copy_specs(["/proc/net/bonding/*", ])
        self.getdumpflows()
        self.getvmdumps()
