# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

"""Overrides the Conductor API for additional PowerVC DB Access:
    1. Override compute_node_delete to not allow OS to erroneously delete
"""

from oslo_config import cfg
from oslo_log import log as logging

from nova import utils
from nova.conductor import api
from nova.conductor import rpcapi

from powervc_nova.conductor import manager
from powervc_nova import _

LOG = logging.getLogger(__name__)


class PowerVCConductorAPI(api.API):
    """ Extends the base Conductor API class with PowerVC capabilities """

    def __init__(self):
        """Constructor for the PowerVC extension to the Conductor API"""
        super(PowerVCConductorAPI, self).__init__()
        if cfg.CONF.conductor.use_local:
            LOG.info(_("Using local access to the PowerVCConductorManager"))
            self._manager = utils.ExceptionHelper(
                manager.PowerVCConductorManager())
        else:
            LOG.info(_("Using RPC access to the PowerVCConductorManager"))
            self._manager = PowerVCConductorRPCAPI()

    def unmanage_instances(self, context, host, inst_uuids):
        """Utility method to unmanage all of the vm instances specified"""
        return self._manager.unmanage_instances(context, host, inst_uuids)


class PowerVCConductorRPCAPI(rpcapi.ConductorAPI):
    """ Extends the base Conductor RPC API class with PowerVC capabilities """

    def unmanage_instances(self, context, host, inst_uuids):
        """Utility method to unmanage all of the vm instances specified"""
        cctxt = self.client.prepare()
        return cctxt.call(context, 'unmanage_instances',
                          host=host, inst_uuids=inst_uuids)
