#
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
"""Table Creation/Deletion for Schema changes in the PowerVC 1.2 Release"""

from sqlalchemy import MetaData
from oslo_log import log as logging
from powervc_nova.db.sqlalchemy import models

LOG = logging.getLogger(__name__)


def upgrade(migration_engine):
    """ Upgrades the Nova DB Tables to include those for PowerVC 1.2 """
    metadata = MetaData(migration_engine)
    metadata.reflect(migration_engine)
    metadata.bind = migration_engine
    # Loop through all of the DTO's that we added new in PowerVC 1.2
    # creating the tables off of the definition defined in the DTO
    for dto in models.POWERVC_V1R2_DTOS:
        table = dto.__table__.tometadata(metadata, None)
        table.create(checkfirst=True)


def downgrade(migration_engine):
    """ Downgrades the Nova DB Tables to remove those for PowerVC 1.2 """
    metadata = MetaData(migration_engine)
    metadata.reflect(migration_engine)
    metadata.bind = migration_engine
    # Loop through all of the DTO's that we added new in PowerVC 1.2
    # dropping the tables off of the definition defined in the DTO
    for dto in reversed(models.POWERVC_V1R2_DTOS):
        dto.__table__.metadata = metadata
        dto.__table__.drop(checkfirst=True)
