# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015, 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
"""Table Creation/Deletion for Schema changes in the PowerVC 1.3.0.2 Release"""

from sqlalchemy import String, Table, MetaData, Column, Integer, Float
from oslo_log import log as logging

LOG = logging.getLogger(__name__)

POWER_SPEC_COLUMNS = [
    Column('ame_expansion_factor', Float)
]

TABLE_COLUMN_UPDATES = [('instance_power_specs', POWER_SPEC_COLUMNS)]


def upgrade(migration_engine):
    """Upgrades the Nova DB Tables to include those for PowerVC 1.3.0.2"""
    metadata = MetaData(migration_engine)
    metadata.reflect(migration_engine)
    metadata.bind = migration_engine
    # Loop through each of the tables we need to add columns to
    for column_update in TABLE_COLUMN_UPDATES:
        table_name, table_columns = column_update
        # We need to add the additional columns to the given table
        atable = Table(table_name, metadata, autoload=True)
        for column in table_columns:
            # Check first to see if the Column exists on the Table already
            if atable._columns.get(column.name) is None:
                atable.create_column(column.copy())


def downgrade(migration_engine):
    """Downgrades the Nova DB Tables to remove those for PowerVC 1.3.0.2"""
    pass
