#
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2014, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from oslo_config import cfg

from oslo_log import log as logging

from powervc_nova import _
from powervc_nova.network.powerkvm.agent.common import micro_op
from powervc_nova.network.powerkvm.agent.common import exception
from powervc_nova.network.powerkvm.agent import commandlet
from powervc_nova import logcall

LOG = logging.getLogger(__name__)

CONF = cfg.CONF
CONF.import_opt('topic', 'nova.conductor.api', group='conductor')


class PingQpid(micro_op.MicroOperation):
    """
    Will ping the Message Bus server and throw an exception if a connection can
    not be made ie no host adapter is able to ping the Message bus server
    """

    def __init__(self, context, rollback):
        self.dom_obj_converter = commandlet.DOMObjectConverter()
        self.rollback = rollback

    @logcall
    def validate(self, current_dom):
        """
        No validation required.

        :param current_dom: State of the system prior to running
        :returns updated_dom: State of the system post running
        :returns warning_list: List of warnings that were found during
                               validation
        """
        LOG.info(_("Validating if a ping to Qpid server can be "
                   "performed"))
        return current_dom, []

    @logcall
    def execute(self):
        """
        Overrides parent method.

        Will throw an error if the ping to the Message Bus server fails.  If it
        succeeds this method does nothing.
        """
        LOG.info(_("Attempting to ping the Message Bus Server"))
        if self.rollback:
            LOG.error(_("Rollback due to API parameter."))
            raise exception.MessageBusPingFailure(timeout="0")
        adapter = self.dom_obj_converter.find_management_adapter()
        if adapter is None:
            LOG.error(_("Failed to ping the Message Bus Server."))
            raise exception.MessageBusPingFailure(timeout="1")

    @logcall
    def undo(self):
        """
        Overrides parent method.

        Undoing a ping doesn't do anything.
        """
        pass
