#
#
# Copyright 2016 IBM Corp.
# All Rights Reserved.
#
#    Copyright 2013 Red Hat Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log
from oslo_serialization import jsonutils
from oslo_utils import versionutils

from nova import objects
from nova.objects import base as obj_base
from nova.objects import fields
from nova.objects import migrate_data as mig_obj

LOG = log.getLogger(__name__)


@obj_base.NovaObjectRegistry.register
class PowerVCLiveMigrateData(mig_obj.LiveMigrateData):
    # Version 1.0: Initial version
    VERSION = '1.0'

    fields = {
        'host_mig_data': fields.DictOfNullableStringsField(),
        'dest_ip': fields.StringField(),
        'dest_user_id': fields.StringField(),
        'dest_sys_name': fields.StringField(),
        'dest_mtms_name': fields.StringField(),
        'dest_lmb': fields.IntegerField(),
        'dest_spp_id': fields.StringField(),
        'dest_msp': fields.StringField(),
        'public_key': fields.StringField(nullable=True),
        'fc_mappings': fields.StringField(nullable=True),
        'scsi_mappings': fields.StringField(nullable=True),
        'dest_proc_compat': fields.StringField(),
        'dest_cred': fields.StringField(),
        'managed_by': fields.StringField(),
        'src_managed_by': fields.StringField(nullable=True, default=None),
        'refresh_conn_info': fields.BooleanField(nullable=True),
        'src_sys_name': fields.StringField(),
        'src_msp': fields.StringField(),
        'pre_migrate_bdms': fields.ObjectField('BlockDeviceMappingList'),
        'vol_data': fields.DictOfStringsField(nullable=True),
    }


@obj_base.NovaObjectRegistry.register
class PowerVCLibvirtLiveMigrateData(mig_obj.LibvirtLiveMigrateData):
    # Version 1.0: Initial version
    VERSION = '1.0'

    fields = {
        'refresh_conn_info': fields.BooleanField(nullable=True),
        'src_sys_name': fields.StringField(),
        'dest_sys_name': fields.StringField(),
        'dest_hyp_name': fields.StringField()
    }

    def _convert_libvirtobj_to_pvckvm(self, libvirt_mig_ob):
        for key in libvirt_mig_ob.obj_fields:
            LOG.debug("libvirt_mig_ob %s" % key)
            if hasattr(libvirt_mig_ob, key):
                val = getattr(libvirt_mig_ob, key)
                LOG.debug("setattr %s" % val)
                setattr(self, key, val)
