# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from powervc_nova import _, translate_message


class IBMPowerVCStorageWarning():
    """
    Define some translated health status warnings for storage using this
    class. Include an _init_() method and a getter for simple message
    generation.
    """
    NonConnFCPorts =\
        _("On Virtual I/O Server '%(vios)s', %(count)d of %(total)d physical "
          "Fibre Channel ports are not available for %(conn_type)s "
          "connectivity. %(reasons)s")
    NoStorageProviders =\
        _("There are no registered storage providers applicable to "
          "this storage connectivity group, or all applicable providers "
          "are in error state.")
    SCGNotEnabled =\
        _("The storage connectivity group is not enabled for new deploys.")
    NoUsablePorts =\
        _("Virtual I/O Server '%(vios)s' has no effective connectivity "
          "because it owns no usable Fibre Channel ports. ")
    NoHostVIOSReady =\
        _("Host %(host)s cannot provide %(bord)s volume connectivity because "
          "there are not enough member Virtual I/O Servers (VIOSs) on the host"
          " that provide at least one of the specified connectivity types. "
          "Boot volume VIOSs ready is %(boot)d. Data volume VIOSs ready is "
          "%(data)d. Required number of VIOSs for any connection: %(req)d.")
    VIOSRedundancyNotMet =\
        _("Host %(host)s does not provide '%(type)s' connectivity "
          "because it does not have enough Virtual I/O "
          "Servers that provide '%(type)s' connectivity. Required number: "
          "%(req)d.")
    FabricsPerVIOSNotMet =\
        _("Virtual I/O Server '%(vios)s' is excluded from NPIV connectivity "
          "because it does not meet the redundant fabric requirement. "
          "Required number of fabrics: %(num)d.")
    FabricsPerHostNotMet =\
        _("The host %(host)s is excluded from NPIV connectivity because it "
          "does not meet the redundant fabric requirement. Required number of "
          "fabrics: %(num)d. Virtual I/O Servers excluded from NPIV "
          "connectivity: %(vioses)s ")
    FabricsSetPerVIOSNotMet =\
        _("Virtual I/O Server '%(vios)s' is excluded from NPIV connectivity "
          "because it does not meet the redundant fabric requirements. "
          "Required fabrics from 'fabric_set_npiv': %(fab_list)s. Required "
          "'ports_per_fabric_npiv': %(port_num)d. Vios fabrics mappings to "
          "ports: %(port_map)s")
    FabricsSetPerHostNotMet =\
        _("The Host %(host)s is excluded from NPIV connectivity because "
          "it does not meet the redundant fabric requirements. Required "
          "fabrics: %(fab_list)s. For each host required number of "
          "'ports_per_fabric_npiv': %(req_port)d. Host fabrics mappings to "
          "vios: %(vios_map)s. Virtual I/O Servers excluded from NPIV "
          "connectivity: %(vioses)s.")
    NoConnectivityForHost =\
        _("The host %(host)s does not meet the %(vol_types)s connectivity "
          "requirements of the storage connectivity group.")
    NoConnectivityForSCG =\
        _("No host members of the storage connectivity group meet the "
          "%(vol_types)s connectivity requirements.")
    VIOSBadState =\
        _("The Virtual I/O Server '%(vios)s' is not running or RMC is not "
          "active on it.")

    def __init__(self, xlated_base_msg):
        self.xlated_base = xlated_base_msg

    def getMsg(self, context, dictarg={}):
        """ Return message with replacement fields. This method could add
            a message prefix like nova exception does, but since these are
            just storage warnings/health messages, it may not be needed.
        """
        msg = self.xlated_base % dictarg
        translated_msg = translate_message(context, msg)
        return translated_msg
