# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from powervc_nova import _

from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper import EntryWrapper
from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper import DELIM
from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper import ROOT
from powervc_nova.virt.ibmpowervm.hmc.k2.entry_wrapper \
    import hmc_sharing_mode_to_nova

from oslo_log import log as logging

from nova.compute import power_state

LOG = logging.getLogger(__name__)

PARTITION_STATE = ROOT + 'PartitionState'
PARTITION_NAME = ROOT + 'PartitionName'
LPAR_ID = ROOT + 'PartitionID'
PARTITION_TYPE = ROOT + 'PartitionType'
AVAIL_PRIORITY = ROOT + 'AvailabilityPriority'
MEM_CONFIG = ROOT + 'PartitionMemoryConfiguration'
PROC_CONFIG = ROOT + 'PartitionProcessorConfiguration'
ASSOC_PROF = ROOT + 'AssociatedPartitionProfile'
LPAR_PROFS = ROOT + 'PartitionProfiles'
CAPABILITIES = ROOT + 'PartitionCapabilities'
PARTITION_IO = ROOT + 'PartitionIOConfiguration'
IPL_SOURCE = ROOT + 'DesignatedIPLSource'
KEY_LOCK = ROOT + 'KeylockPosition'
RESTRICTED_IO = ROOT + 'IsRestrictedIOPartition'
PROF_LINK = LPAR_PROFS + DELIM + 'link'
CNA_LINKS = ROOT + 'ClientNetworkAdapters' + DELIM + 'link'

# procs
CURR_USE_DED_PROCS = PROC_CONFIG + DELIM + 'CurrentHasDedicatedProcessors'
USE_DED_PROCS = PROC_CONFIG + DELIM + 'HasDedicatedProcessors'
DED_PROC_CONFIG = (PROC_CONFIG + DELIM +
                   'DedicatedProcessorConfiguration')
CURR_DED_PROC_CONFIG = (PROC_CONFIG + DELIM +
                        'CurrentDedicatedProcessorConfiguration')
SHARED_PROC_CONFIG = (PROC_CONFIG + DELIM +
                      'SharedProcessorConfiguration')
CURR_SHARED_PROC_CONFIG = (PROC_CONFIG + DELIM +
                           'CurrentSharedProcessorConfiguration')
CURR_SHARING_MODE = PROC_CONFIG + DELIM + 'CurrentSharingMode'
SHARING_MODE = PROC_CONFIG + DELIM + 'SharingMode'

# dedicated proc
CURR_PROCS = CURR_DED_PROC_CONFIG + DELIM + 'CurrentProcessors'
CURR_MAX_PROCS = CURR_DED_PROC_CONFIG + DELIM + 'CurrentMaximumProcessors'
CURR_MIN_PROCS = CURR_DED_PROC_CONFIG + DELIM + 'CurrentMinimumProcessors'

RUN_PROCS = CURR_DED_PROC_CONFIG + DELIM + 'RunProcessors'

DES_PROCS = DED_PROC_CONFIG + DELIM + 'DesiredProcessors'
DES_MAX_PROCS = DED_PROC_CONFIG + DELIM + 'MaximumProcessors'
DES_MIN_PROCS = DED_PROC_CONFIG + DELIM + 'MinimumProcessors'

# shared proc
CURR_VCPU = CURR_SHARED_PROC_CONFIG + DELIM + 'AllocatedVirtualProcessors'
CURR_MAX_VCPU = (
    CURR_SHARED_PROC_CONFIG + DELIM + 'CurrentMaximumVirtualProcessors')
CURR_MIN_VCPU = (
    CURR_SHARED_PROC_CONFIG + DELIM + 'CurrentMinimumVirtualProcessors')

RUN_VCPU = CURR_SHARED_PROC_CONFIG + DELIM + 'AllocatedVirtualProcessors'

CURR_PROC_UNITS = CURR_SHARED_PROC_CONFIG + DELIM + 'CurrentProcessingUnits'
DES_VCPU = SHARED_PROC_CONFIG + DELIM + 'DesiredVirtualProcessors'
DES_MAX_VCPU = SHARED_PROC_CONFIG + DELIM + 'MaximumVirtualProcessors'
DES_MIN_VCPU = SHARED_PROC_CONFIG + DELIM + 'MinimumVirtualProcessors'
CURR_PROC_UNITS = CURR_SHARED_PROC_CONFIG + DELIM + 'CurrentProcessingUnits'
CURR_MAX_PROC_UNITS = (CURR_SHARED_PROC_CONFIG + DELIM +
                       'CurrentMaximumProcessingUnits')
CURR_MIN_PROC_UNITS = (CURR_SHARED_PROC_CONFIG + DELIM +
                       'CurrentMinimumProcessingUnits')
DES_PROC_UNITS = SHARED_PROC_CONFIG + DELIM + 'DesiredProcessingUnits'
MAX_PROC_UNITS = SHARED_PROC_CONFIG + DELIM + 'MaximumProcessingUnits'
MIN_PROC_UNITS = SHARED_PROC_CONFIG + DELIM + 'MinimumProcessingUnits'
CURR_UNCAPPED_WEIGHT = (CURR_SHARED_PROC_CONFIG + DELIM +
                        'CurrentUncappedWeight')
UNCAPPED_WEIGHT = SHARED_PROC_CONFIG + DELIM + 'UncappedWeight'
SHARED_PROC_POOL_ID = SHARED_PROC_CONFIG + DELIM + 'SharedProcessorPoolID'

# memory
CURR_MEM = MEM_CONFIG + DELIM + 'CurrentMemory'
CURR_MAX_MEM = MEM_CONFIG + DELIM + 'CurrentMaximumMemory'
CURR_MIN_MEM = MEM_CONFIG + DELIM + 'CurrentMinimumMemory'
DES_MEM = MEM_CONFIG + DELIM + 'DesiredMemory'
DES_MAX_MEM = MEM_CONFIG + DELIM + 'MaximumMemory'
DES_MIN_MEM = MEM_CONFIG + DELIM + 'MinimumMemory'

# ame
AME_ENABLED = MEM_CONFIG + DELIM + 'ActiveMemoryExpansionEnabled'
EXP_FACTOR = MEM_CONFIG + DELIM + 'ExpansionFactor'
ME_ENABLED = MEM_CONFIG + DELIM + 'MemoryExpansionEnabled'
CURR_EXP_FACTOR = MEM_CONFIG + DELIM + 'CurrrentExpansionFactor'

RUN_MEM = MEM_CONFIG + DELIM + 'RuntimeMemory'

SHARED_MEM_ENABLED = MEM_CONFIG + DELIM + 'SharedMemoryEnabled'

DEFAULT_SHARING_MODE = 'share_idle_procs'
DEFAULT = 'default'
FALSE = 'false'
ZERO = '0'
CURRENT_PROC_MODE = ROOT + 'CurrentProcessorCompatibilityMode'
PENDING_PROC_MODE = ROOT + 'PendingProcessorCompatibilityMode'
OPERATING_SYSTEM_VER = ROOT + 'OperatingSystemVersion'
SRR_CAPABLE = ROOT + 'SimplifiedRemoteRestartCapable'
DLPAR_MEM_CAPABLE = CAPABILITIES + DELIM + \
    'DynamicLogicalPartitionMemoryCapable'
DLPAR_PROC_CAPABLE = CAPABILITIES + DELIM + \
    'DynamicLogicalPartitionProcessorCapable'
RMC_OS_SHUTDOWN_CAPABLE = CAPABILITIES + DELIM + \
    'ResourceMonitoringControlOperatingSystemShutdownCapable'
RMC_STATE = 'ResourceMonitoringControlState'
RMC_IPADDR = 'ResourceMonitoringIPAddress'
REF_CODE = 'ReferenceCode'
MIGRATION_STATE = 'MigrationState'  # TODO - Not defined yet (SW219614)
SRR_STATE = ROOT + 'RemoteRestartState'

# Tagged IO
TAGGED_IO = PARTITION_IO + DELIM + 'TaggedIO'
LOAD_SOURCE = TAGGED_IO + DELIM + 'LoadSource'

hmc_state_to_nova = {
    "migrating running": power_state.RUNNING,
    "running": power_state.RUNNING,
    "starting": power_state.RUNNING,

    "migrating not active": power_state.SHUTDOWN,
    "not activated": power_state.SHUTDOWN,

    "hardware discovery": power_state.NOSTATE,
    "not available": power_state.NOSTATE,
    # map open firmware state to active since it can be shut down
    "open firmware": power_state.RUNNING,
    "resuming": power_state.NOSTATE,
    "shutting down": power_state.NOSTATE,
    "suspending": power_state.NOSTATE,
    "unknown": power_state.NOSTATE,

    "suspended": power_state.SUSPENDED,

    "error": power_state.CRASHED
}


class LPAREntryWrapper(EntryWrapper):

    def get_lpar_state(self):
        """
        Find the current state of the lpar and convert it to
        the appropriate nova.compute.power_state
        @return The appropriate integer state value from power_state
        """
        partition_state = self.get_partition_state()
        if partition_state is None:
            return power_state.NOSTATE

        try:
            nova_state = hmc_state_to_nova[partition_state.lower()]
        except KeyError:
            nova_state = None

        if nova_state is None:
            message = _('Virtual machine state for uuid %(uuid)s '
                        'is not an expected value: %(partition_state)s') % \
                {"uuid": self.get_uuid(),
                 "partition_state": partition_state}
            LOG.error(message)
            nova_state = power_state.NOSTATE

        return nova_state

    def get_partition_state(self):
        partition_state = self.get_parm_value(PARTITION_STATE)
        return partition_state

    def get_lpar_name(self):
        return self.get_parm_value(PARTITION_NAME)

    def get_lpar_id(self):
        return int(self.get_parm_value(LPAR_ID, ZERO))

    def get_lpar_type(self):
        return self.get_parm_value(PARTITION_TYPE)

    def get_current_mem(self):
        return self.get_parm_value(CURR_MEM, ZERO)

    def get_current_max_mem(self):
        return self.get_parm_value(CURR_MAX_MEM, ZERO)

    def get_current_min_mem(self):
        return self.get_parm_value(CURR_MIN_MEM, ZERO)

    def get_desired_mem(self):
        return self.get_parm_value(DES_MEM, ZERO)

    def get_max_mem(self):
        return self.get_parm_value(DES_MAX_MEM, ZERO)

    def get_min_mem(self):
        return self.get_parm_value(DES_MIN_MEM, ZERO)

    def get_ame_enabled(self):
        return self.get_parm_value(AME_ENABLED, FALSE)

    def get_exp_factor(self):
        return self.get_parm_value(EXP_FACTOR, ZERO)

    def get_me_enabled(self):
        return self.get_parm_value(ME_ENABLED, FALSE)

    def get_curr_exp_factor(self):
        return self.get_parm_value(CURR_EXP_FACTOR, ZERO)

    def get_run_mem(self):
        return self.get_parm_value(RUN_MEM, ZERO)

    def get_current_sharing_mode(self):
        mode = self.get_parm_value(CURR_SHARING_MODE, DEFAULT_SHARING_MODE)
        return hmc_sharing_mode_to_nova.get(mode)

    def get_sharing_mode(self):
        mode = self.get_parm_value(SHARING_MODE, DEFAULT_SHARING_MODE)
        return hmc_sharing_mode_to_nova.get(mode)

    def get_current_mem_share_enabled(self):
        # The default is None instead of False so that the caller
        # can know if the value is not set
        return self.get_parm_value_bool(SHARED_MEM_ENABLED, None)

    def get_current_proc_mode(self):
        # Returns true if dedicated or false if shared
        return self.get_parm_value(CURR_USE_DED_PROCS, FALSE).lower()

    def get_proc_mode(self):
        # Returns true if dedicated or false if shared
        return self.get_parm_value(USE_DED_PROCS, FALSE).lower()

    def get_current_procs(self):
        return self.get_parm_value(CURR_PROCS, ZERO)

    def get_current_max_procs(self):
        return self.get_parm_value(CURR_MAX_PROCS, ZERO)

    def get_current_min_procs(self):
        return self.get_parm_value(CURR_MIN_PROCS, ZERO)

    def get_desired_procs(self):
        return self.get_parm_value(DES_PROCS, ZERO)

    def get_max_procs(self):
        return self.get_parm_value(DES_MAX_PROCS, ZERO)

    def get_min_procs(self):
        return self.get_parm_value(DES_MIN_PROCS, ZERO)

    def get_current_vcpus(self):
        return self.get_parm_value(CURR_VCPU, ZERO)

    def get_current_max_vcpus(self):
        return self.get_parm_value(CURR_MAX_VCPU, ZERO)

    def get_current_min_vcpus(self):
        return self.get_parm_value(CURR_MIN_VCPU, ZERO)

    def get_desired_vcpus(self):
        return self.get_parm_value(DES_VCPU, ZERO)

    def get_max_vcpus(self):
        return self.get_parm_value(DES_MAX_VCPU, ZERO)

    def get_min_vcpus(self):
        return self.get_parm_value(DES_MIN_VCPU, ZERO)

    def get_current_proc_units(self):
        return self.get_parm_value(CURR_PROC_UNITS, ZERO)

    def get_current_max_proc_units(self):
        return self.get_parm_value(CURR_MAX_PROC_UNITS, ZERO)

    def get_current_min_proc_units(self):
        return self.get_parm_value(CURR_MIN_PROC_UNITS, ZERO)

    def get_desired_proc_units(self):
        return self.get_parm_value(DES_PROC_UNITS, ZERO)

    def get_max_proc_units(self):
        return self.get_parm_value(MAX_PROC_UNITS, ZERO)

    def get_min_proc_units(self):
        return self.get_parm_value(MIN_PROC_UNITS, ZERO)

    def get_run_procs(self):
        return self.get_parm_value(RUN_PROCS, ZERO)

    def get_run_vcpus(self):
        return self.get_parm_value(RUN_VCPU, ZERO)

    def get_curr_uncapped_weight(self):
        return self.get_parm_value(CURR_UNCAPPED_WEIGHT, ZERO)

    def get_uncapped_weight(self):
        return self.get_parm_value(UNCAPPED_WEIGHT, ZERO)

    def get_shared_proc_pool_id(self):
        return int(self.get_parm_value(SHARED_PROC_POOL_ID, ZERO))

    def get_avail_priority(self):
        return self.get_parm_value(AVAIL_PRIORITY, ZERO)

    def get_migration_state(self):
        return self.get_parm_value(MIGRATION_STATE, 'Not Migrating')

    def get_proc_compat_modes(self):
        """
        returns: A tuple containing the current and pending
                 processor compatibility modes.
        """
        current = self.get_parm_value(CURRENT_PROC_MODE)
        pending = self.get_parm_value(PENDING_PROC_MODE)

        # Format the proc compat modes from K2 strings to PowerVC strings
        if current is not None:
            current = current.replace('_Plus', '+')
        if pending is not None:
            pending = pending.replace('_Plus', '+')

        return current, pending

    def get_operating_system(self):
        """
        returns: A string representing the operating system and version
                 or 'unknown'.
        """
        return self.get_parm_value(OPERATING_SYSTEM_VER, 'Unknown')

    def get_lpar_prof_path(self):
        """
        Find the last activated lpar profile. If there isn't one, pick
        the first profile.
        """
        prof_elem = self._k2entry.element.find(ASSOC_PROF)
        prof_path = None
        if prof_elem is not None:
            prof_path = prof_elem.getattrib()['href']
        if prof_path is None:
            prof_elem = self._k2entry.element.find(PROF_LINK)
            if prof_elem is not None:
                prof_path = prof_elem.getattrib()['href']

        return prof_path

    def get_cna_links(self):
        """Return a list of hrefs to the LPAR's ClientNetworkAdapters"""
        ret_links = list()
        cna_links = self._k2entry.element.findall(CNA_LINKS)

        # Loop through what was found, if anything
        if cna_links:
            for link in cna_links:
                ret_links.append(link.getattrib()['href'])

        return ret_links

    def get_rmc_state(self):
        return self.get_parm_value(RMC_STATE)

    def get_rmc_ipaddr(self):
        return self.get_parm_value(RMC_IPADDR)

    def get_ref_code(self):
        return self.get_parm_value(REF_CODE)

    def get_restrictedio(self):
        return self.get_parm_value_bool(RESTRICTED_IO, False)

    def get_srr_capability(self):
        return self.get_parm_value(SRR_CAPABLE, FALSE)

    def get_srr_state(self):
        return self.get_parm_value(SRR_STATE)

    def get_rmc_os_shutdown_capability(self):
        return self.get_parm_value_bool(RMC_OS_SHUTDOWN_CAPABLE)

    def check_dlpar_connectivity(self):
        """Check the partition for DLPAR capability and rmc state

        :returns: Returns true or false if DLPAR capable
        :returns: Returns RMC state as string
        """

        # Pull the dlpar and rmc values from K2
        mem_dlpar = self.get_parm_value_bool(DLPAR_MEM_CAPABLE)
        proc_dlpar = self.get_parm_value_bool(DLPAR_PROC_CAPABLE)
        rmc = self.get_rmc_state()

        dlpar = mem_dlpar and proc_dlpar

        return dlpar, rmc

    def set_desired_mem(self, value):
        self.set_parm_value(DES_MEM, value)

    def set_max_mem(self, value):
        self.set_parm_value(DES_MAX_MEM, value)

    def set_min_mem(self, value):
        self.set_parm_value(DES_MIN_MEM, value)

    def set_ame_enabled(self, value):
        self.set_parm_value(AME_ENABLED, value)

    def set_exp_factor(self, value):
        self.set_parm_value(EXP_FACTOR, value)

    def set_proc_compat_mode(self, value):
        self.set_parm_value(PENDING_PROC_MODE, value)

    def set_avail_priority(self, value):
        self.set_parm_value(AVAIL_PRIORITY, value)

    def set_sharing_mode(self, value):
        self.set_parm_value(SHARING_MODE, value)

    def set_desired_procs(self, value):
        self.set_parm_value(DES_PROCS, value)

    def set_max_procs(self, value):
        self.set_parm_value(DES_MAX_PROCS, value)

    def set_min_procs(self, value):
        self.set_parm_value(DES_MIN_PROCS, value)

    def set_desired_vcpus(self, value):
        self.set_parm_value(DES_VCPU, value)

    def set_max_vcpus(self, value):
        self.set_parm_value(DES_MAX_VCPU, value)

    def set_min_vcpus(self, value):
        self.set_parm_value(DES_MIN_VCPU, value)

    def set_desired_proc_units(self, value):
        self.set_parm_value(DES_PROC_UNITS, value)

    def set_max_proc_units(self, value):
        self.set_parm_value(MAX_PROC_UNITS, value)

    def set_min_proc_units(self, value):
        self.set_parm_value(MIN_PROC_UNITS, value)

    def set_uncapped_weight(self, value):
        return self.set_parm_value(UNCAPPED_WEIGHT, value)

    def set_shared_proc_pool_id(self, value):
        return self.set_parm_value(SHARED_PROC_POOL_ID, value)

    def set_srr_capability(self, value):
        return self.set_parm_value(SRR_CAPABLE, value)

    def set_srr_state(self, value):
        return self.set_parm_value(SRR_STATE, value)

    def set_proc_mode(self, value):
        # Returns true for dedicated or false for shared
        self.set_parm_value(USE_DED_PROCS, value)
