# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from powervc_nova import logcall
from powervc_nova.virt.ibmpowervm.hmc.k2 import constants as k2const

from oslo_log import log as logging

LOG = logging.getLogger(__name__)

SCSI_CLIENT = 'VirtualSCSIClientAdapter'
FC_CLIENT = 'VirtualFibreChannelClientAdapter'
SERVER_LPAR_ID_PATH = './ServerAdapter/LocalPartitionID'


@logcall
def get_vios_lpar_id_from_vadapters(vclient_resp):
    vios_ids = set()
    for entry in vclient_resp.feed.entries:
        p = entry.element.findtext(SERVER_LPAR_ID_PATH)
        if p is not None:
            vios_ids.add(int(p))
    return list(vios_ids)


@logcall
def get_vios_lpar_id_hosting_scsi(oper, lpar_uuid):
    """
    Gather all the virtual scsi client adapters allocated for the lpar and
    extract the VIOS partition ID
    """
    client_scsi_resp = oper.read(k2const.LPAR,
                                 lpar_uuid,
                                 SCSI_CLIENT)
    if client_scsi_resp is not None:
        vios_lpar_ids = get_vios_lpar_id_from_vadapters(client_scsi_resp)
        if len(vios_lpar_ids) > 0:
            return vios_lpar_ids
    return None


@logcall
def get_vios_lpar_id_hosting_fc(oper, lpar_uuid):
    """
    Gather all the virtual scsi client adapters allocated for the lpar and
    extract the VIOS partition ID
    """
    client_fc_resp = oper.read(k2const.LPAR,
                               lpar_uuid,
                               FC_CLIENT)
    if client_fc_resp is not None:
        vios_lpar_ids = get_vios_lpar_id_from_vadapters(client_fc_resp)
        if len(vios_lpar_ids) > 0:
            return vios_lpar_ids
    return None
