# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
from oslo_log import log as logging

from powervc_nova.virt.ibmpowervm.hmc import exception as hmc_ex

from powervc_nova import _

LOG = logging.getLogger(__name__)


class VirtualIOAdapter(object):
    """
    Object represents virtual io adapter information queried from HMC.
    The goal is only to store attributes needed by PowerVC.
    """
    def __init__(self, lpar_id, slot_num=None):
        """
        Create a generic virtual io adapter object.
        We should only use the child classes like VirtualSCSIClientAdapter..
        :param lpar_id: lpar_id owning the virtual adapter
        :param slot_num: virtual slot number used by the virtual adapter. If
                         it's None, HMC will pick the next available slot to
                         use.
        """
        if lpar_id is None:
            LOG.error(_("lpar_id is required parameter to create a virtual"
                        " adapter."))
            raise hmc_ex.IBMPowerVMInvalidVIOConfig()

        self.lpar_id = lpar_id
        self.slot_num = slot_num

    def create_via_hmc(self, k2op):
        """
        Create virtual adapter via HMC K2 api.
        This implementation would be different based adapter type.
        """
        raise NotImplementedError()

    def delete_via_hmc(self, k2op):
        """
        Delete virtual adapter via HMC K2 api.
        This implementation would be different based adapter type.
        """
        raise NotImplementedError()
